/******************************************************************************
* UNIT DESCRIPTION:  Base class for UI input values used by charting functions
* SPEC REF: (if used)
* DESIGN REF: (if used)
*
* Copyright (c) 2009 Schweitzer Engineering Laboratories, Pullman, Washington
******************************************************************************/
using System;
using System.ComponentModel;

namespace SELChartClasses
{
//type
//  TIntegerArray  = array of integer;
//  TBooleanArray = array of boolean;
    public enum TTimeSeriesChartRenderType 
    {
        sctUnknown = -1,
        sctLine,
        sctArea
    }

    public class TTimeSeriesInputSettings 
    {
        private Boolean FChartAxisDateTime;
        private TTimeSeriesChartRenderType FChartStyle;
        private int FChartTransparency;
        private Boolean FChartQuery;
        private Boolean FChartLabels;
        private Boolean FChart3D;
        private Boolean FChartSecondaryUnits;

        private int FElevation;
        private int FRotation;
        private int FZoom;
        private int FAspectZoom;
        private int FHorizOffset;
        private int FVertOffset;
        private int FPercent3D;

        private Boolean[]   FActiveChannels = new Boolean[1];
        private Boolean[]   FActiveEvents = new Boolean[1];
        private int FWindowX;
        private int FWindowY;
        private int FWindowHeight;
        private int FWindowWidth;

        //public 
        //property ActiveChannelArray : TBooleanArray read FActiveChannels;
        //property ActiveEventArray : TBooleanArray read FActiveEvents;

        public TTimeSeriesChartRenderType ChartStyle
        {
            get
            {
                return FChartStyle;
            }
            set
            {
                FChartStyle = value;
            }
        }
        public Boolean ChartAxisDateTime
        {
            get
            {
                return FChartAxisDateTime;
            }
            set
            {
                FChartAxisDateTime = value;
            }
        }

        public Boolean ChartQuery
        {
            get
            {
                return FChartQuery;
            }
            set
            {
                FChartQuery = value;
            }
        }

        public Boolean ChartLabels
        {
            get
            {
                return FChartLabels;
            }
            set
            {
                FChartLabels = value;
            }
        }

        public Boolean ChartSecondaryUnits
        {
            get
            {
                return FChartSecondaryUnits;
            }
            set
            {
                FChartSecondaryUnits = value;
            }
        }

        public Boolean Chart3D
        {
            get
            {
                return FChart3D;
            }
            set
            {
                FChart3D = value;
            }
        }

        public int Elevation
        {
            get
            {
                return FElevation;
            }
            set
            {
                FElevation = value;
            }
        }

        public int ChartTransparency
        {
            get
            {
                return FChartTransparency;
            }
            set
            {
                FChartTransparency = value;
            }
        }

        public int Rotation
        {
            get
            {
                return FRotation;
            }
            set
            {
                FRotation = value;
            }
        }

        public int Zoom
        {
            get
            {
                return FZoom;
            }
            set
            {
                FZoom = value;
            }
        }

        public int AspectZoom
        {
            get
            {
                return FAspectZoom;
            }
            set
            {
                FAspectZoom = value;
            }
        }

        public int HorizOffset
        {
            get
            {
                return FHorizOffset;
            }
            set
            {
                FHorizOffset = value;
            }
        }

        public int VertOffset
        {
            get
            {
                return FVertOffset;
            }
            set
            {
                FVertOffset = value;
            }
        }

        public int Percent3D
        {
            get
            {
                return FPercent3D;
            }
            set
            {
                FPercent3D = value;
            }
        }

        public int WindowX
        {
            get
            {
                return FWindowX;
            }
            set
            {
                FWindowX = value;
            }
        }

        public int WindowY
        {
            get
            {
                return FWindowY;
            }
            set
            {
                FWindowY = value;
            }
        }

        public int WindowHeight
        {
            get
            {
                return FWindowHeight;
            }
            set
            {
                FWindowHeight = value;
            }
        }

        public int WindowWidth
        {
            get
            {
                return FWindowWidth;
            }
            set
            {
                FWindowWidth = value;
            }
        }

        public Boolean[] ActiveChannels
        {
            get{return FActiveChannels;}
            set{FActiveChannels = value;}
        }
        public Boolean[] ActiveEvents
        {
            get{return FActiveEvents;}
            set{FActiveEvents = value;}
        }
        // Indexer - does this work for individual private arrays in the class?
        //public Boolean this[int pos]
        //{
        //    get
        //    {
        //        return FActiveChannels[pos];
        //    }
        //    set
        //    {
        //        FActiveChannels[pos] = value;
        //    }
        //}


/******************************************************************************
* MODULE NAME ...  Create
* SYSTEM NAME ...  Acselerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Create the TTimeSeriesInputSettings object.
*
* INPUT PARAMETERS:
*                    None:
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.12.03  Kent Myers        Initial implementation
*******************************************************************************/
        public TTimeSeriesInputSettings()
        {
          Initialize();
        }
/*******************************************************************************
* MODULE NAME ...  Initialize
* SYSTEM NAME ...  Acselerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Initialize Input setttings.
*
* INPUT PARAMETERS:
*                    None:
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.12.03  Kent Myers        Initial implementation
*******************************************************************************/
        public void Initialize()
        {
          // Initialize input settings
          FChartAxisDateTime = false;     // Date/time labels disabled
          FChartStyle = TTimeSeriesChartRenderType.sctLine;           // Use line chart
          FChartLabels = true;            // Chart labels active by default
          FChartSecondaryUnits = true;    // Chart primary/secondary units (secondary = true)
          FChartTransparency = 0;         // Percentage transparent (opaque by default)
          FChartQuery = false;            // Chart query function disabled
          FChart3D = false;               // Chart is 2D by default
          FElevation = 0;                 // Percent elevation (none)
          FRotation = 0;                  // Percent rotation (none)
          FZoom = 100;                    // Percent zoom (fully zoomed)
          FHorizOffset = 0;               // Pixel offset in horizontal direction
          FVertOffset = 0;                // Pixel offset in vertical direction
          FPercent3D = 100;               // Percentage 3D
          FWindowX = 150;
          FWindowY = 50;
          FWindowWidth = 700;
          FWindowHeight = 650;
        }
/************************************************************************
* NAME:           GetChartInputControlSettings
* FUNCTIONAL DESCRIPTION:   getter for chart input controls
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void GetChartInputControlSettings()
        {
           // intentionally empty for now
        }
/************************************************************************
* NAME:           SetChartInputControlSettings
* FUNCTIONAL DESCRIPTION:   setter for chart input controls
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void SetChartInputControlSettings()
        {
           // intentionally empty for now
        }
/******************************************************************************
* MODULE NAME ...  GetActiveChannels
* SYSTEM NAME ...  Acselerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Getter for ActiveChannels dynamic array.
*
* INPUT PARAMETERS:
*                    index: integer  index into array
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    result:  boolean  state of array element
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.12.04  Kent Myers        Initial implementation
*******************************************************************************/
        public Boolean GetActiveChannels(int index) 
        {
          if((index >= 0) &&
             (index < FActiveChannels.Length)) 
            {
            return FActiveChannels[index];
            } 
          else
            {
            return false;
            }
        }
/******************************************************************************
* MODULE NAME ...  SetActiveChannels
* SYSTEM NAME ...  Acselerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Setter for ActiveChannels dynamic array.
*
* INPUT PARAMETERS:
*                    index: integer  index into array
*                    state : boolean  state of array element
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.12.04  Kent Myers        Initial implementation
*******************************************************************************/
        public void SetActiveChannels(int index, Boolean state)
        {
          if((index >= 0) &&
             (index < FActiveChannels.Length)) 
          {
            FActiveChannels[index] = state;
          }
        }
/******************************************************************************
* MODULE NAME ...  SetActiveChannelLength
* SYSTEM NAME ...  Acselerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Allocate ActiveChannels dynamic array.
*
* INPUT PARAMETERS:
*                    size : integer size of array
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.12.04  Kent Myers        Initial implementation
*******************************************************************************/
        public void SetActiveChannelLength(int size)
        {
            FActiveChannels = new bool[size];
        }
/******************************************************************************
* MODULE NAME ...  GetActiveEvents
* SYSTEM NAME ...  Acselerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Getter for ActiveEvents dynamic array.
*
* INPUT PARAMETERS:
*                    index: integer  index into array
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    result : boolean  state of array element
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.12.04  Kent Myers        Initial implementation
*******************************************************************************/
        public Boolean GetActiveEvents(int index)
        {
          if((index >= 0) &&
             (index < FActiveEvents.Length)) 
            {
                return FActiveEvents[index];
            }
            else
            {
                return false;
            }
        }
/******************************************************************************
* MODULE NAME ...  SetActiveEvents
* SYSTEM NAME ...  Acselerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Setter for ActiveEvents dynamic array.
*
* INPUT PARAMETERS:
*                    index: integer  index into array
*                    state : boolean  state of array element
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.12.04  Kent Myers        Initial implementation
*******************************************************************************/
        public void SetActiveEvents(int index, Boolean state )
        {
          if((index >= 0) &&
             (index < FActiveEvents.Length))
            {
                FActiveEvents[index] = state;
            }
        }
/******************************************************************************
* MODULE NAME ...  SetActiveEventLength
* SYSTEM NAME ...  Acselerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Allocate ActiveEvents dynamic array.
*
* INPUT PARAMETERS:
*                    size : integer size of array
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.12.04  Kent Myers        Initial implementation
*******************************************************************************/
        public void SetActiveEventLength(int size)
        {

            FActiveEvents = new Boolean[size];
        }
    }
}
