﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml;
using System.Collections.ObjectModel;
using SELChartClasses;
using SELGlobalStringMethods;
using Steema.TeeChart.WPF.Styles;
using Steema.TeeChart.WPF.Tools;

namespace WpfChartViewer
{
    //public struct TTimeStamp {
    //    public int Time;
    //    public int Date;
    //}

    public partial class PageChart : Page
    {
        const string INVALID_CHART_MSG = "Invalid chart type.";
        const string CRLF = "\r\n";     // carriage return line feed
        const string CR = "\r";     // carriage return
        const string TAB = "\t";
        Color INTERVAL_COLOR = Colors.LightGray;
        Color PARENTEVENT_COLOR = Colors.Yellow;
        Color SUBEVENT_COLOR = Colors.Red;
        Color EVENTMARKER_COLOR = Colors.Yellow;
        //const string DATETIME_WITH_MS = "MM/dd/yy hh:mm:ss.zzz";
        //const string DATETIME_WITH_SEC = "MM/dd/yy hh:mm:ss";
        //const string TIME_WITH_MS = "hh:mm:ss.zzz";
        //const string TIME_WITH_SEC = "hh:mm:ss";
        const string DATETIME_WITH_MS = "{0:MM/dd/yy hh:mm:ss.fff}";
        const string DATETIME_WITH_SEC = "{0:MM/dd/yy hh:mm:ss}";
        const string TIME_WITH_MS = "{0:hh:mm:ss.fff}";
        const string TIME_WITH_SEC = "{0:hh:mm:ss}";
        const int SHGFP_TYPE_CURRENT = 0;
        const System.Environment.SpecialFolder CSIDL_PERSONAL = System.Environment.SpecialFolder.Personal;  // Doc and settings > User > My Documents

        //private Steema.TeeChart.WPF.TChart ChartHMIDisplay;
        //private Steema.TeeChart.WPF.TChart ChartHMICBEMA;
        private TTimeSeriesChartType FChartType;   // chart type: ctLDP, ctSSI
        private TTimeSeriesDeviceType FDeviceType;  // device type: dtSEL7XX...
        private TTimeSeriesChart FChartTimeSeries;
        //private TFormDeviceInfo FDeviceInfoDialog;     // information window
//       private Custom[] ChartSeries;
        private Bar BarSeries;
        private Points PointSeries;  // used for LDP events
        private Steema.TeeChart.WPF.Styles.Line[] CBEMALimitSeries = new Steema.TeeChart.WPF.Styles.Line[2];   
        private Points CBEMADataSeries;
        private TTimeSeriesInputSettingsSSI FChartInputSettingsSSI;
        private TTimeSeriesInputSettingsLDP FChartInputSettingsLDP;
        private string FDateFormat;  // date format string
        //private TFormNotepad FNotepadForm;
        private Boolean FMenuChart;   // true if launched from tools menu
        private int FChartInstance;  // index identifying chart
        private CursorTool ChartTool1 = new CursorTool();
        private CursorTool ChartTool2 = new CursorTool();

        //public TChartMenuEvent ChartMenuHandler; // delegate (method pointer)


        public TTimeSeriesChartType ChartType
        {
            get{return FChartType;}
    //        set{FChartType = value;}
        }

        public TTimeSeriesDeviceType DeviceType
        {
            get{return FDeviceType;}
    //        set{FDeviceType = value;}
        }


        public TTimeSeriesChart ChartTimeSeries
        {
            get{return FChartTimeSeries;}
    //        set{FChartTimeSeries = value;}
        }

        //public TFormDeviceInfo DeviceInfoDialog
        //{
        //    get{return FDeviceInfoDialog;}
        //    set{FDeviceInfoDialog = value;}
        //}

        public TTimeSeriesInputSettingsSSI ChartInputSettingsSSI
        {
            get{return FChartInputSettingsSSI;}
            set{FChartInputSettingsSSI = value;}
        }

        public TTimeSeriesInputSettingsLDP ChartInputSettingsLDP
        {
            get{return FChartInputSettingsLDP;}
            set{FChartInputSettingsLDP = value;}
        }

        public string DateFormat
        {
            get{return FDateFormat;}
            set{FDateFormat = value;}
        }

        public Boolean MenuChart
        {
            get{return FMenuChart;}
            set{FMenuChart = value;}
        }

        public int ChartInstance
        {
            get{return FChartInstance;}
            set{FChartInstance = value;}
        }

/************************************************************************
* NAME:  InitChart
* FUNCTIONAL DESCRIPTION: Initialize chart class members before importing report file
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void InitChart(TTimeSeriesFileType aFileType)
        {
            ChartTimeSeries.InitChart(aFileType);
        }
/************************************************************************
* NAME:     EraseChartDisplay
* FUNCTIONAL DESCRIPTION:  Erase content of chart
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void EraseChartDisplay()
        {
            ChartTimeSeries.EraseChartDisplay();
        }
/************************************************************************
* NAME:     ClearChartWindow
* FUNCTIONAL DESCRIPTION:  Erase content of chart and reset window
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void ClearChartWindow()
        {
            ChartTimeSeries.EraseChartDisplay();
            FChartType = TTimeSeriesChartType.ctUndefined;
            ResetChartWindow();
        }
/************************************************************************
* NAME:   TrimFileName
* FUNCTIONAL DESCRIPTION:  trim filename from the end of a file path name
* INPUT PARAMETERS: filename : string  is the original filename
* OUTPUT PARAMETERS: none
* RETURN:             result : string is the trimmed pathname
************************************************************************/
        public string TrimFileName(string filename)
        {
            return ChartTimeSeries.TrimFileName(filename);
        }
/************************************************************************
* NAME:   SaveStringToFile
* FUNCTIONAL DESCRIPTION: Save string to given file name
* INPUT PARAMETERS:
*                    filePath : string  name of file to save string to
*                    aString : string   string to save
* OUTPUT PARAMETERS: none
* RETURN:            none
************************************************************************/
        public void SaveStringToFile(string filePath, string aString)
        {
            // Save data in file so that it may be read back as individual records
            ChartTimeSeries.SaveStringToFile(filePath,aString);
        }
/************************************************************************
* NAME:   RenameOldFile
* FUNCTIONAL DESCRIPTION: Rename file if it exists
* INPUT PARAMETERS:
*                    filePath : string  name of file to save string to
* OUTPUT PARAMETERS: none
* RETURN:            none
************************************************************************/
        public void RenameOldFile(string filePath)
        {
            // Save data in file so that it may be read back as individual records
            ChartTimeSeries.RenameOldFile(filePath);
        }
/************************************************************************
* NAME:    AppendCSVHeader
* FUNCTIONAL DESCRIPTION:  Append header records for CSV to string
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: MeterBuffer : string  file content
* RETURN: result : boolean   status of export (true = success)
************************************************************************/
        public Boolean AppendCSVHeader(ref string MeterBuffer)
        {
            return ChartTimeSeries.AppendCSVHeader(ref MeterBuffer);
        }
/************************************************************************
* NAME:    AppendCSVData
* FUNCTIONAL DESCRIPTION: Append CSV data to string from HMI data stored
*                         in memory resident TimeSeries structure
* INPUT PARAMETERS:       none
* OUTPUT PARAMETERS:      MeterBuffer : string  file content
* RETURN:                 result : boolean   true if file exported, otherwise false
************************************************************************/
        public Boolean AppendCSVData(ref string MeterBuffer)
        {
            return ChartTimeSeries.AppendCSVData(ref MeterBuffer);
        }
/************************************************************************
* NAME:    AppendReport
* FUNCTIONAL DESCRIPTION: Append report summary to string
* INPUT PARAMETERS:    none
* OUTPUT PARAMETERS:   MeterBuffer : string  file content
* RETURN:              result : boolean   true if report appended, otherwise false
************************************************************************/
        public Boolean AppendReport(out string MeterBuffer)
        {
            return ChartTimeSeries.AppendReport(out MeterBuffer);
        }
/************************************************************************
* NAME:   FillEventIndexTable
* FUNCTIONAL DESCRIPTION:  Scan raw data, identify events, build index of events
* INPUT PARAMETERS:
*                    filter: boolean  use filter controls true/false
* OUTPUT PARAMETERS: none
* RETURN:  result : boolean  true if events found, otherwise false
************************************************************************/
        public Boolean FillEventIndexTable(Boolean filter)
        {
            return ChartTimeSeries.FillEventIndexTable(filter);
        }
/************************************************************************
* NAME:       ReportType
* FUNCTIONAL DESCRIPTION:   return type of report file
* INPUT PARAMETERS: fileData : TStringList   contains content of file
* OUTPUT PARAMETERS:
* RETURN: result : result : TTimeSeriesChartType  (ctSSI, ctLDP, ctUnknown)
************************************************************************/
        public TTimeSeriesChartType ReportType(string filePath)
        {
            return ChartTimeSeries.EventData.ReportType(filePath);
        }
/************************************************************************
* NAME:    SetChartDateRangeFilters
* FUNCTIONAL DESCRIPTION:
* Set the date range filters based on date/time range in event data
 INPUT PARAMETERS:    none
* OUTPUT PARAMETERS:   none
* RETURN:              none
************************************************************************/
        public void SetChartDateRangeFilters()
        {
            DateTime myDateTime1, myDateTime2;
            TTimeSeriesRecord dataRecord;
            GlobalString gs = new GlobalString();
            string myFormat;

            myFormat = gs.GetDateControlFormat(FDateFormat);
            //datePickerBegin.Format = myFormat;
            //datePickerEnd.Format = myFormat;

            if(ChartTimeSeries.EventData.RawDataTable.Count > 0) 
            {
                dataRecord = ChartTimeSeries.EventData.RawDataTable[0];   // first record
                myDateTime1 = dataRecord.DateTime;
                dataRecord = ChartTimeSeries.EventData.RawDataTable[ChartTimeSeries.EventData.RawDataTable.Count-1];   // last record
                myDateTime2 = dataRecord.DateTime;

                datePickerBegin.DisplayDate = myDateTime1;
                datePickerBegin.SelectedDate = myDateTime1.Date;
                timePickerBegin.SelectedHour = myDateTime1.Hour;
                timePickerBegin.SelectedMinute = myDateTime1.Minute;
                timePickerBegin.SelectedSecond = myDateTime1.Second;

                datePickerEnd.DisplayDate = myDateTime2;
                datePickerEnd.SelectedDate = myDateTime2.Date;
                timePickerEnd.SelectedHour = myDateTime2.Hour;
                timePickerEnd.SelectedMinute = myDateTime2.Minute;
                timePickerEnd.SelectedSecond = myDateTime2.Second;
            }
            else
            {
                datePickerBegin.DisplayDate = DateTime.Now;
                datePickerBegin.SelectedDate = DateTime.Now.Date;
                timePickerBegin.SelectedHour = DateTime.Now.Hour;
                timePickerBegin.SelectedMinute = DateTime.Now.Minute;
                timePickerBegin.SelectedSecond = DateTime.Now.Second;

                datePickerEnd.DisplayDate = DateTime.Now;
                datePickerEnd.SelectedDate = DateTime.Now.Date;
                timePickerEnd.SelectedHour = DateTime.Now.Hour;
                timePickerEnd.SelectedMinute = DateTime.Now.Minute;
                timePickerEnd.SelectedSecond = DateTime.Now.Second;
            }
        }
/************************************************************************
* NAME:   ChartSeriesActiveState
* FUNCTIONAL DESCRIPTION: Return "active" state of chart series
* INPUT PARAMETERS:  Num : integer  chart series index
* OUTPUT PARAMETERS: none
* RETURN:            result : boolean   active state
************************************************************************/
        public Boolean ChartSeriesActiveState(int Num)
        {
            return ChartHMIDisplay.Series[Num].Active;
        }
/************************************************************************
* NAME:   GetChartFilename
* FUNCTIONAL DESCRIPTION: Get filename where chart data is saved by default
* INPUT PARAMETERS:  aChartType : THMIChartType (type of chart)
*                    TTimeSeriesFileType aFileType  (type of file)
*                    ExportPath : default path to file
*                    aID        : ID of meter
* OUTPUT PARAMETERS: none
* RETURN:            filePath : string  path to file that was saved
************************************************************************/
        public string GetChartFilename(TTimeSeriesChartType aChartType,
            TTimeSeriesFileType aFileType, string ExportPath, string aID)
        {
            SetChartType(aChartType);
            return ChartTimeSeries.GetChartFilename(aFileType, ExportPath, aID);
        }
/************************************************************************
* NAME:     SetChartType
* FUNCTIONAL DESCRIPTION:   Set the chart type
* INPUT PARAMETERS:  aChartType : TTimeSeriesChartType  new chart type
* OUTPUT PARAMETERS: none
* RETURN:            none
************************************************************************/
        public void SetChartType(TTimeSeriesChartType aChartType)
        {
            /*This function is used to change the chart type based on the incoming
            argument.  The argument indicates the chart type desired.  It is compared
            to the current chart type, and if different, the input settings for the
            current chart object will be saved and the charting data will be freed.
            A new chart object of the requested type will be created and the input
            settings corresponding to the requested chart type will be retrieved and
            made current.
            */
            if(FChartType != aChartType) 
            {
                switch (aChartType)
                {
                    case TTimeSeriesChartType.ctUndefined:
                    case TTimeSeriesChartType.ctLDP:    // need something for Unknown

                        // If input values not saved previously, create a default set for LDP
                        if(FChartInputSettingsLDP == null) 
                        {
                            FChartInputSettingsLDP = new TTimeSeriesInputSettingsLDP();  // preserved input settings
                            FChartInputSettingsLDP.GetChartInputControlSettings();   // Get UI control settings
                            FChartInputSettingsLDP.ChartAxisDateTime = true;
                        }
                        // InputSettings is a container for the input of the active chart
                        if(ChartTimeSeries.EventData.InputSettings == null) 
                        {
                            FChartTimeSeries.EventData.InputSettings = FChartInputSettingsLDP;
                        }

                        FChartTimeSeries.SaveChartActiveChannels();  // save settings in previous chart
                        FChartTimeSeries.SaveChartActiveEvents();    // save SSI Channels before switching

//                        FreeAndNil(FChartTimeSeries);
                        FChartTimeSeries = new TTimeSeriesChartLDP();
                        FChartTimeSeries.ChartDisplay = ChartHMIDisplay;
                        FChartTimeSeries.CursorTool = ChartTool1;
                        FChartTimeSeries.EventData.DateFormat = DateFormat;
                        // Make "InputSettings" a reference to the LDP input settings
                        FChartTimeSeries.EventData.InputSettings = FChartInputSettingsLDP;
                    break;

                    case TTimeSeriesChartType.ctSSI:
                        // If input values not saved previously, create a default set for SSI
                        if(FChartInputSettingsSSI == null) 
                        {
                            FChartInputSettingsSSI = new TTimeSeriesInputSettingsSSI();  // preserved input settings
                            FChartInputSettingsSSI.GetChartInputControlSettings();  // Get UI control settings
                        }
                        // InputSettings is a container for the input of the active chart
                        if(FChartTimeSeries.EventData.InputSettings == null) 
                        {
                            FChartTimeSeries.EventData.InputSettings = FChartInputSettingsSSI;
                        }

                        FChartTimeSeries.SaveChartActiveChannels();   // save settings in previous chart

//                        FreeAndNil(FChartTimeSeries);
                        FChartTimeSeries = new TTimeSeriesChartSSI();
                        FChartTimeSeries.ChartDisplay = ChartHMIDisplay;
                        FChartTimeSeries.CursorTool = ChartTool1;
                        FChartTimeSeries.EventData.DateFormat = DateFormat;
                        // Make "InputSettings" a reference to the SSI input settings
                        FChartTimeSeries.EventData.InputSettings = FChartInputSettingsSSI;
                    break;

                    case TTimeSeriesChartType.ctInvalid:
                    // do nothing
                    break;

                    default:
                    throw new Exception(INVALID_CHART_MSG);
//                    break;
                }
                FChartType = aChartType;   // now update the chart type
            }
        }
/************************************************************************
* NAME:       ResetInput
* FUNCTIONAL DESCRIPTION:  Reset chart to match present input settings
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void ResetInput()
        {
            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is nil");

            ChartTimeSeries.ChartDisplay.Panning.Allow = Steema.TeeChart.WPF.ScrollModes.Both;
            ChartTimeSeries.ChartDisplay.Zoom.Allow = true;
            ChartTimeSeries.ChartDisplay.Aspect.View3D = ChartTimeSeries.EventData.InputSettings.Chart3D;
            ChartTimeSeries.CursorTool.Active = ChartTimeSeries.EventData.InputSettings.ChartQuery;
            ChartTimeSeries.CursorTool.Snap = ChartTimeSeries.EventData.InputSettings.ChartQuery;
            ChartTimeSeries.ChartDisplay.Aspect.Zoom = ChartTimeSeries.EventData.InputSettings.AspectZoom;

            ChartTimeSeries.ChartDisplay.Aspect.Elevation = ChartTimeSeries.EventData.InputSettings.Elevation;
            ChartTimeSeries.ChartDisplay.Aspect.Rotation = ChartTimeSeries.EventData.InputSettings.Rotation;
            ChartTimeSeries.ChartDisplay.Aspect.Zoom = ChartTimeSeries.EventData.InputSettings.Zoom;
            ChartTimeSeries.ChartDisplay.Aspect.HorizOffset = ChartTimeSeries.EventData.InputSettings.HorizOffset;
            ChartTimeSeries.ChartDisplay.Aspect.VertOffset = ChartTimeSeries.EventData.InputSettings.VertOffset;

            // Restore window position and dimensions
            //ChartTimeSeries.Width = ChartTimeSeries.EventData.InputSettings.WindowWidth;  // these get reset by event handler
            //ChartTimeSeries.Left = ChartTimeSeries.EventData.InputSettings.WindowX;
            //ChartTimeSeries.Top = ChartTimeSeries.EventData.InputSettings.WindowY;
            //ChartTimeSeries.Height = ChartTimeSeries.EventData.InputSettings.WindowHeight;
        }
/************************************************************************
* NAME:     ResetChartWindow
* FUNCTIONAL DESCRIPTION:   Reset panels of chart window based on chart type
* INPUT PARAMETERS:  none
* OUTPUT PARAMETERS: none
* RETURN:            none
************************************************************************/
        public void ResetChartWindow()
        {
            //string[] TABLIST = new string[3] {"General","Channels","Events"};
            //const int MARGIN_SIZE = 3;

            //int oldTab;
            Boolean scaleVisible;

            if(FChartType == TTimeSeriesChartType.ctSSI)
            {
                groupBoxFilters.Visibility = Visibility.Visible;
            }
            else 
            {
                groupBoxFilters.Visibility = Visibility.Collapsed;
            }

            scaleVisible = (FChartType == TTimeSeriesChartType.ctLDP) &&
                  (FChartTimeSeries.EventData.LDPChannelTable.Count > 0);

            if (scaleVisible)
            {
                labelScale.Visibility = Visibility.Visible;
                comboBoxScale.Visibility = Visibility.Visible;
            }
            else
            {
                labelScale.Visibility = Visibility.Collapsed;
                comboBoxScale.Visibility = Visibility.Collapsed;
            }

            // start window configuation

            //oldTab = FDeviceInfoDialog.TabControlDeviceInfo.TabIndex;  // original tab selection

            groupBoxFilters.Header = "Filter Events";
            if (FChartType == TTimeSeriesChartType.ctSSI)    // SSI always has chart and event summary
            {
                //    ListViewEventList.Width = ScrollBoxEventSelection.Left-MARGIN_SIZE;

                // If panel was previously hidden, alignment must be reset
                listViewChart.Visibility = Visibility.Visible;
                tabControlChart.Visibility = Visibility.Visible;
                tabItemCBEMA.Visibility = Visibility.Visible;
                tabItemChart.Visibility = Visibility.Visible;
                tabItemToolsDeviceInfoChannels.Visibility = Visibility.Collapsed;
                tabItemToolsDeviceInfoEvents.Visibility = Visibility.Collapsed;
                //tabItemChart.Focusable = true;
                //tabItemCBEMA.TabIndex = 0;
                //tabItemChart.TabIndex = 1;
                //tabControlChart.SelectedIndex = 0;

                tabTools.Visibility = Visibility.Visible;
                tabTools.SelectedIndex = 0;
                tabItemToolsDeviceInfo.Visibility = Visibility.Visible;
                tabItemToolsFilter.Visibility = Visibility.Visible;

                tabItemChart.Header = "VSSI";
                rowListView.Height = new System.Windows.GridLength(220);
                columnFilters.Width = new System.Windows.GridLength(350);
            }
            else if (ChartType == TTimeSeriesChartType.ctLDP)    // LDP always has chart, sometimes event summary
            {
                //    TabSetChart.Hide;     // hide the SSI tab selector
                tabControlChart.Visibility = Visibility.Visible;
                tabItemCBEMA.Visibility = Visibility.Collapsed;
                tabItemChart.Visibility = Visibility.Visible;
                tabControlChart.SelectedIndex = 1;
                tabItemChart.Header = "LDP";

                tabTools.Visibility = Visibility.Visible;
                tabItemToolsDeviceInfo.Visibility = Visibility.Visible;
                tabItemToolsDeviceInfoChannels.Visibility = Visibility.Visible;

                if (FChartTimeSeries.EventData.LDPEventTable.Count > 0)
                {
                    tabTools.SelectedIndex = 0;
                    tabItemToolsDeviceInfoEvents.Visibility = Visibility.Visible;
                }
                else
                {
                    tabTools.SelectedIndex = 1;
                    tabItemToolsDeviceInfoEvents.Visibility = Visibility.Collapsed;
                }

                if (FChartTimeSeries.EventData.IndexTable.Count > 0)    // there are SER events
                {
                    listViewChart.Visibility = Visibility.Visible;
                    columnFilters.Width = new System.Windows.GridLength(0);
                    rowListView.Height = new System.Windows.GridLength(200);
                    tabItemToolsFilter.Visibility = Visibility.Visible;
                }
                else
                {
                    listViewChart.Visibility = Visibility.Collapsed;
                    columnFilters.Width = new System.Windows.GridLength(0);
                    tabItemToolsFilter.Visibility = Visibility.Collapsed;
                }
            }
            else  // unknown chart type: display only chart in entire window
            {
                tabTools.Visibility = Visibility.Visible;
                tabTools.SelectedIndex = 2;
                tabItemToolsDeviceInfo.Visibility = Visibility.Collapsed;
                tabItemToolsFilter.Visibility = Visibility.Collapsed;

                tabControlChart.Visibility = Visibility.Collapsed;
                tabItemCBEMA.Visibility = Visibility.Collapsed;
                tabItemChart.Visibility = Visibility.Collapsed;
                listViewChart.Visibility = Visibility.Collapsed;
                columnFilters.Width = new System.Windows.GridLength(10);
                rowListView.Height = new System.Windows.GridLength(200);

                //FDeviceInfoDialog.TabControlDeviceInfo.Tabs.Clear;
                CBEMAChartVisible(false);
            }
        }
/************************************************************************
* NAME:  TestChartData
* FUNCTIONAL DESCRIPTION: Test contents of chart file for validity
* INPUT PARAMETERS:
*                    aChartType : THMIChartType  type of chart to test data for
*                    filePath : string   path to chart data file
* OUTPUT PARAMETERS:
*                    HasData : boolean  true if header records found, otherwise false
* RETURN:
*                   result : boolean  true if data file was valid, otherwise false
************************************************************************/
        public Boolean TestChartData(TTimeSeriesChartType aChartType, string filePath, ref Boolean hasData)
        {
            TTimeSeriesChartType oldChartType;
            Boolean result;

            oldChartType = FChartType;     // remember original chart type
            result = LoadData(filePath, out hasData);  // LoadData might change FChartType
            if(result == false)     // data was missing or invalid
            {
                FChartType = oldChartType;
            }
            return result;
        }
/************************************************************************
* NAME:   ReadChartData
* FUNCTIONAL DESCRIPTION:  Read HMI data into chart object for display or export
* INPUT PARAMETERS:  aChartType : THMIChartType (type of chart)
*                    fileReset : boolean   True if sending new data,
*                                          False if just showing chart with old data
*                    filePath  : string    path to chart data file
* OUTPUT PARAMETERS:
*                   ChartReset : boolean   true/false initialize chart
*                   ChartValid : boolean   true/false if file has valid data
* RETURN:
*                   result:  true if chartable data found in file, otherwise false
************************************************************************/
        public Boolean ReadChartData(TTimeSeriesChartType aChartType,
                        Boolean fileReset, string filePath,
                        ref Boolean chartReset, out Boolean chartValid )
        {
            Boolean found,hasData;
            TTimeSeriesChartType oldChartType,newChartType;

            found = false;  // assume no data is found
            chartValid = true;   // assume if chart data is found it is valid
            chartReset = (fileReset || (ChartType != ChartTimeSeries.EventData.ChartType));

            oldChartType = FChartType;  // original type

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is nil");

            if(chartReset) 
            {
                newChartType = FChartTimeSeries.EventData.ReadChartType(filePath);   // test file for chart type

                if(newChartType != TTimeSeriesChartType.ctUndefined) 
                {
                    aChartType = newChartType;   // override incoming chart type
                }

                switch (aChartType)
                {
                    case TTimeSeriesChartType.ctSSI:
                        SetChartType(TTimeSeriesChartType.ctSSI);
                        found = LoadData(filePath, out hasData);
                        if(!found) 
                        {
                            if(hasData)     // there was a valid header but some invalid data
                            {
                            chartValid = false;
                            }
                        }
                        else
                        {
                            ResetInput();
                            if(oldChartType != aChartType) 
                            {
                                FChartTimeSeries.EraseChartDisplay();   // clear content of chart
                                if(fileReset) 
                                {
                                    ResetChartDisplay();     // force chart back to 2D un-rotated view
                                }
                            }
                            if(fileReset) 
                            {
                                SetChartDateRangeFilters();  // reset filters to new range
                            }
                            SetChartLabels();
                            FChartTimeSeries.FillEventIndexTable(true);  // find events with active filters
                            FillEventListViewSSI(false);
                            FChartTimeSeries.InitChartVariableCount();
                            ChartHMIDisplay.Series.Clear();   // throw away previous chart data
                            ChartHMICBEMA.Series.Clear();   // throw away previous chart data
                            if(fileReset)                    // allocate and initialize channel state
                            {
                                FChartTimeSeries.InitChartActiveChannelsDefault(true);
                                FChartTimeSeries.InitChartActiveEventsDefault();
                            }
                            else
                            {
                                FChartTimeSeries.InitChartActiveEvents();
                            }
                        }
                    break;

                    case TTimeSeriesChartType.ctLDP:
                        SetChartType(TTimeSeriesChartType.ctLDP);
                        found = LoadData(filePath, out hasData);
                        if(!found) 
                        {
                            if(hasData)     // there was a valid header but some invalid data
                            {
                                chartValid = false;
                            }
                        }
                        else
                        {
                            ResetInput();
                            if(oldChartType != aChartType) 
                            {
                                FChartTimeSeries.EraseChartDisplay();   // clear content of chart
                                if(fileReset) 
                                {
                                    ResetChartDisplay();     // force chart back to 2D un-rotated view
                                }
                            }
                            if(fileReset) 
                            {
                                SetChartDateRangeFilters();  // reset filters to new range
                            }
                            FChartTimeSeries.FillEventIndexTable(true);  // find events with active filters
                            FillEventListViewLDP(false);
                            FChartTimeSeries.InitChartVariableCount();
                            ChartHMIDisplay.Series.Clear();   // throw away previous chart data
                            ChartHMICBEMA.Series.Clear();   // throw away previous chart data
                            if(fileReset)                // allocate and initialize channel state
                            {
                                FChartTimeSeries.InitChartActiveChannelsDefault(true);
                            }
                        }
                    break;

                    default:

                    chartValid = false;
                    found = false;  // invalid file
                    break;
                }

                // There must be at least 2 records to chart something
                if(FChartTimeSeries.EventData.RawDataTable.Count < 2) 
                {
                found = false;  // nothing to chart
                }
            }

            return found;
        }
/******************************************************************************
* MODULE NAME ...  LoadData
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Read report data from file and insert into data object
*
* INPUT PARAMETERS:
*                    filePath : string  name of file containing ASCII report data
*
* OUTPUT PARAMETERS:
*                    HasData : boolean   true if file has valid header data, otherwise false
* RETURN VALUE:
*                    found : boolean  true if valid data found, otherwise false
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation (obtained from internet)
* 2008.11.17  Kent Myers        Add HasData argument
* 2008.12.04  Kent Myers        Automatically detect file type
*******************************************************************************/
        public Boolean LoadData(string filePath, out Boolean hasData)
        {
            const int SKIP_COLUMNS = 3;  // rec, date, time
            const int STATUS_COLUMN = 4;

            List<string> fileData;
            List<string> fileHeader;
            TTimeSeriesFileType fileType;
            Boolean found = false, hasStatus;
            int i, count;

            fileData = new List<string>();
            fileHeader = new List<string>();
            hasData = false;

            try
            {
                filePath = filePath.ToUpper();

                fileType = TTimeSeriesFileType.ftCSV;   // default value
                FChartTimeSeries.EventData.Clear();    // now it is safe to clear old data

                // read file and get data and labels (header) in string lists

                FChartTimeSeries.EventData.ChartType = FChartType;  // default chart type
                found = ChartTimeSeries.EventData.ReadFileData(filePath, fileData,
                                fileHeader, ref fileType);
                FChartType = ChartTimeSeries.EventData.ChartType; // may have been reset from CSV header data

                hasData = found;  // true if there was valid header data, otherwise false

                // extract raw data from fileData string list and load into time series object
                if(found) 
                {
                    switch (FChartType)
                    {
                        case TTimeSeriesChartType.ctSSI:     // columns must be previously initialized by calling method
                        break;

                        case TTimeSeriesChartType.ctLDP: // automatically create list of active columns from string list of labels
                            if(fileType == TTimeSeriesFileType.ftReport)   // must initialize data columns
                            {
                                // look ahead at column names
                                hasStatus = ChartTimeSeries.EventData.ReportFileHasStatusColumn(fileData,out count);
                                if(hasStatus) 
                                {
                                    FChartTimeSeries.EventData.HeaderColumns[STATUS_COLUMN] = fileHeader.Count-1;  // status column
                                    FChartTimeSeries.EventData.SetLengthDataColumns(fileHeader.Count-STATUS_COLUMN);
                                    // skip first 3 columns (record, date, time)... and last (status)
                                    for( i = SKIP_COLUMNS; i< fileHeader.Count - 1; i++) 
                                    {
                                        FChartTimeSeries.EventData.DataColumns[i-SKIP_COLUMNS] = i;
                                    }
                                }
                                else       // generic LDP report
                                {
                                    FChartTimeSeries.EventData.HeaderColumns[STATUS_COLUMN] = -1;  // no status column
                                    FChartTimeSeries.EventData.SetLengthDataColumns(fileHeader.Count-SKIP_COLUMNS);
                                    // skip first 3 columns (record, date, time)
                                    for (i = SKIP_COLUMNS; i<fileHeader.Count; i++)
                                    {
                                        FChartTimeSeries.EventData.DataColumns[i-SKIP_COLUMNS] = i;
                                    }
                                }
                            }
                        break;

                        default:
                            throw new Exception(INVALID_CHART_MSG);
 //                       break;
                    }
                    FChartTimeSeries.EventData.LoadData(fileData,
                                                        FChartTimeSeries.EventData.HeaderColumnArray,
                                                        FChartTimeSeries.EventData.DataColumnArray);
                    FDeviceType = FChartTimeSeries.EventData.DeviceType;
                }
            }
            catch
            {
            }
             //finally
            //  FreeAndNil(fileData);
            //  FreeAndNil(fileHeader);
            //}

            return found;
        }
/******************************************************************************
* MODULE NAME ...  ResetChartControls
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Reset chart controls based on input settings for current HMI context
*
* INPUT PARAMETERS:  None
*
* OUTPUT PARAMETERS:
*                    None:
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        public void ResetChartControls()
        {
            comboBoxChartType.SelectedIndex = (int) FChartTimeSeries.EventData.InputSettings.ChartStyle;
            SetDateTimeLabels();
        }
/************************************************************************
* NAME:  UpdateActiveEvents
* FUNCTIONAL DESCRIPTION: Update list view with active events
* INPUT PARAMETERS:    none
* OUTPUT PARAMETERS:  none
* RETURN:             none
************************************************************************/
        public void UpdateActiveEvents()
        {
        int i;

            for(i=0;  i<ChartTimeSeries.EventData.InputSettings.ActiveEvents.Length; i++)
            {
 //               XmlElement mySelectedElement = (XmlElement)listViewChart.Items[i];
                //mySelectedElement.GetAttribute("checkEventActive").
                //   NowPlaying.Text = mySelectedElement.GetAttribute("Name").ToString()
                //    + " by " +mySelectedElement.GetAttribute("Artist").ToString(); 
                //listViewChart.Items[i].Checked = ChartTimeSeries.EventData.InputSettings.ActiveEvents[i];
            }
        }
/************************************************************************
* NAME:  UpdateChart
* FUNCTIONAL DESCRIPTION:  Update chart display after updating chart data
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void UpdateChart()
        {
            Boolean doCBEMA;

            doCBEMA = false;
            SetChartDateRangeFilters();  // reset filters to new range
            FChartTimeSeries.FillEventIndexTable(true);  // find events with active filters
            FChartTimeSeries.InitChartVariableCount();   // reset variable count if appropriate
            ChartHMIDisplay.Series.Clear();   // throw away previous chart data
            ChartHMICBEMA.Series.Clear();   // throw away previous chart data
            FChartTimeSeries.InitChartActiveChannelsDefault(true);   // re-allocate if necessary

            switch(ChartType)
            {
                case TTimeSeriesChartType.ctSSI:
                    ChartTimeSeries.EventData.ChartName = "VSSI";
                    ChartTimeSeries.InitChartActiveEventsDefault();
                    FillEventListViewSSI(true);
                    SetCBEMA(false);
                    doCBEMA = true;
                break;

                case TTimeSeriesChartType.ctLDP:
                    ChartTimeSeries.EventData.ChartName = "LDP";
                    FillEventListViewLDP(true);
                    doCBEMA = false;
                break;
            }

            DisplayChart(true,true,true,true,doCBEMA); // display the chart
            buttonBrowse.Visibility = System.Windows.Visibility.Hidden;
        }
/******************************************************************************
* MODULE NAME ...  UpdateChart
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Identify which check box was selected in list view
*
* INPUT PARAMETERS:
*                    reset : boolean  true if regenerating chart, false if updating chart
*
* OUTPUT PARAMETERS:
*                    None:
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        public void UpdateChart(Boolean reset)
        {
            Boolean doCBEMA;

            ResetChartWindow();

            doCBEMA = GetCBEMA();

            FChartTimeSeries.SaveChartActiveChannels();   // remember which channels are active
            FChartTimeSeries.InitChartVariableCount();   // reset variable count if appropriate
            FChartTimeSeries.InitChartActiveChannelsDefault(false);   // re-allocate if necessary
            ChartHMIDisplay.Series.Clear();   // throw away previous chart data
            ChartHMICBEMA.Series.Clear();   // throw away previous chart data
            ResetInput();  // Reset to current input settings
            if(doCBEMA) 
            {
                DisplayCBEMA(reset);  // regenerate display based on currently checked events
                ChartHMICBEMA.UpdateLayout();  // refresh display
            }
            else
            {
                Display(reset);  // regenerate display based on currently checked events
                ChartHMIDisplay.UpdateLayout();  // refresh display
            }

            ResetDeviceInformation();     // update information dialog
        }
/******************************************************************************
* MODULE NAME ...  ResetChartDisplay
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Reset chart to the default 2D view
*
* INPUT PARAMETERS:
*                    None:
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.11.20  Kent Myers        Original implementation
*******************************************************************************/
        public void ResetChartDisplay()
        {
            FChartTimeSeries.ChartDisplay.Panning.Allow = Steema.TeeChart.WPF.ScrollModes.Both;
            FChartTimeSeries.ChartDisplay.Zoom.Allow = true;
            FChartTimeSeries.ChartDisplay.Aspect.View3D = false;
            FChartTimeSeries.ChartDisplay.Aspect.Zoom = 100;

            FChartTimeSeries.ChartDisplay.Aspect.Elevation = 0;
            FChartTimeSeries.ChartDisplay.Aspect.Rotation = 0;
            FChartTimeSeries.ChartDisplay.Aspect.Zoom = 100;
            FChartTimeSeries.ChartDisplay.Aspect.HorizOffset = 0;
            FChartTimeSeries.ChartDisplay.Aspect.VertOffset = 0;

            labelHelp.Content = "   Drag left button (down/right) to Zoom, Right button to Translate";

            FChartTimeSeries.EventData.InputSettings.Chart3D = false;
            FChartTimeSeries.EventData.InputSettings.Elevation = 0;
            FChartTimeSeries.EventData.InputSettings.Rotation = 0;
            FChartTimeSeries.EventData.InputSettings.Zoom = 100;
            FChartTimeSeries.EventData.InputSettings.AspectZoom = 100;
            FChartTimeSeries.EventData.InputSettings.HorizOffset = 0;
            FChartTimeSeries.EventData.InputSettings.VertOffset = 0;

            FChartTimeSeries.AutoScaleAxes();
        }
/************************************************************************
* NAME:   DisplayChart
* FUNCTIONAL DESCRIPTION:  Display Chart data found in report string
* INPUT PARAMETERS:   report : string  is ASCII string containing data to be parsed
* OUTPUT PARAMETERS: none
* RETURN:             result : boolean  true if data could be parsed
************************************************************************/
        public Boolean DisplayChart(string report)
        {
            string filePath;
            Boolean result;
            GlobalString gs = new GlobalString();

            filePath = GetChartFilename(TTimeSeriesChartType.ctLDP,TTimeSeriesFileType.ftReport,
                                        gs.GetSpecialFolder(CSIDL_PERSONAL)+"\\"," ");
            SaveStringToFile(filePath,report);
            result = DisplayChartFile(filePath);
            File.Delete(filePath);
            return result;
        }
/************************************************************************
* NAME:   DisplayChartFile
* FUNCTIONAL DESCRIPTION:  Display Chart data found in file
* INPUT PARAMETERS:   filePath : filename containing data to be parsed
* OUTPUT PARAMETERS: none
* RETURN:             result : boolean  true if data could be parsed
************************************************************************/
public Boolean DisplayChartFile(string filePath)
{
  TTimeSeriesChartType myChartType;
  TTimeSeriesFileType myFileType;

  myChartType = ChartTimeSeries.EventData.ReadChartType(filePath,out myFileType);
  SetChartType(myChartType);
  InitChart(myFileType);
  return DisplayChart(myChartType,true,filePath,true);
}
/************************************************************************
* NAME:   DisplayChart
* FUNCTIONAL DESCRIPTION:  Display Chart data found in file
* INPUT PARAMETERS:   aChartType : THMIChartType  type of chart to display
*                     fileReset : boolean  Initialize display of chart (true/false)
*                     filename : string  is the original filename
*                     showChart : boolean  display the chart (true/false)
* OUTPUT PARAMETERS: none
* RETURN:             result : boolean  true if data could be parsed
************************************************************************/
        public Boolean DisplayChart(TTimeSeriesChartType aChartType, Boolean fileReset, string filePath, Boolean showChart)
        {
            Boolean chartReset, chartValid, found, doCBEMA;
            TTimeSeriesChartType oldChartType;
            TTimeSeriesInputSettingsSSI myInputSettings;

            oldChartType = FChartType;  // original type
            chartReset = true;

            if(fileReset)     // cancel date/time filter controls
            {
                checkBoxBegin.IsChecked = false;
                checkBoxEnd.IsChecked = false;
                if(FChartType == TTimeSeriesChartType.ctSSI)
                {
                    myInputSettings = (TTimeSeriesInputSettingsSSI) ChartTimeSeries.EventData.InputSettings;
                    myInputSettings.BeginDateActive = false;
                    myInputSettings.EndDateActive = false;
                }
            }

            doCBEMA = GetCBEMA();
            SetCBEMA(false);       // must disable CBEMA so that VSSI chart will be generated

            // Read chart data from file  (chartReset true if fileReset or change chart type)
            found = ReadChartData(aChartType, fileReset, filePath, ref chartReset, out chartValid);
            if(!chartValid)
            {
                MessageBox.Show("Error reading data into chart.");
            }

            DisplayChart(found,chartReset,showChart,fileReset,doCBEMA);

            SetCBEMA(doCBEMA);       // Update to present state

            if(found || (!chartReset && (FChartTimeSeries.EventData.RawDataTable.Count > 1)))
            {
                // successful chart
            }
            else
            {      // something went wrong
                FChartType = oldChartType;  // restore original chart type
            }
            return found;
        }
/************************************************************************
* NAME:   DisplayChart
* FUNCTIONAL DESCRIPTION:  Display Chart data already loaded into memory
* INPUT PARAMETERS:   found : boolean   data is in memory
*                     chartReset : boolean true fileReset or ChartType changed
*                     fileReset : boolean  true if new file imported
*                     showChart : boolean  display the chart (true/false)
*                     doCBEMA : boolean true if CBEMA is default chart
* OUTPUT PARAMETERS: none
* RETURN:             none
************************************************************************/
        public void DisplayChart(Boolean found,Boolean chartReset,Boolean showChart,Boolean fileReset,Boolean doCBEMA)
        {
            if(found)      // valid charting data found in file
            {
                UpdateChart(true);   // update the chart
                UpdateActiveEvents();  // update list view with active events
            }

            if(found || (!chartReset && (FChartTimeSeries.EventData.RawDataTable.Count > 1)))
            {
                //if (showChart || Visible)           // Display the chart window
                if (showChart)           // Display the chart window
                    {
                    //WindowState = wsNormal;   // cancel window maximization
                    //Show();
                    if((FChartType == TTimeSeriesChartType.ctSSI) && (fileReset || doCBEMA))
                    {
                        if(tabControlChart.TabIndex != 0)  // VSSI chart is active
                        {
                            tabControlChart.TabIndex = 0;   // force CBEMA chart to be active
                        }
                        else
                        {                         // CBEMA tab active, draw CBEMA explicitly
                            CBEMAChartVisible(doCBEMA);  // Make CBEMA chart visible
                            DisplayCBEMA(true);  // regenerate CBEMA chart
                            ChartHMICBEMA.UpdateLayout();  // refresh display
                        }
                    }
                }
                else
                {
                    SetCBEMA(doCBEMA);  // Set CBEMA flag in case it was disabled.
                }
            }
        }
        public void Displayx(Boolean reset)
        {
            FChartTimeSeries.ChartDisplay.Series.Clear();
            Steema.TeeChart.WPF.Styles.Line lineSeries = new Steema.TeeChart.WPF.Styles.Line();
            FChartTimeSeries.ChartDisplay.Series.Add(lineSeries);
            ChartTimeSeries.ChartDisplay.Series[0].FillSampleValues(10);

        }
/******************************************************************************
* MODULE NAME ...  Display
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*                       SetChartVariableCount
* FUNCTIONAL DESCRIPTION:  Create the chart, assign attributes and insert data for display
*
* INPUT PARAMETERS:
*                    reset: boolean  true if regenerating display (rescale axis limits),
*                                    false if updating display (retain current axis limits)
* OUTPUT PARAMETERS:
*                    None:
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
* 2008.12.24  Kent Myers        Fix event indicators when they overlap
*******************************************************************************/
        public void Display(Boolean reset)
        {
            const int MARKERS_PER_EVENT = 4;
            const string SKIP_RECORD = "(5)";
            const string TEST_RECORD = "(6)";

            int index,i,j,checkCount,channelCount,rec;
            double myRecord,xMin,xMax,yMin,yMax;
            TTimeSeriesRecord dataRecord;
            TTimeSeriesLDPChannelRecord channelRecord;
            TTimeSeriesIndexRecord dataIndex;
            int start, stop, estart, estop;
 //           string time_format;
            float[] channelScale;
            int parentEvent, activeCount,aCount,markCount;
            int[] chartMarker,chartRec,eventRec,eventNumber;   // event number of charted events
            float[] chartHeight;
            Color[] eventColor;
            int mycount;

            if(ChartTimeSeries.ChartVariableCount == 0) return;   // no data

            CBEMAChartVisible(false);

            // Event marker information (possible 4 markers per event)
            chartRec = new int[ChartTimeSeries.EventData.IndexTable.Count];  // sequential record number
            eventRec = new int[ChartTimeSeries.EventData.IndexTable.Count];  // actual record number
            chartMarker = new int[ChartTimeSeries.EventData.IndexTable.Count*MARKERS_PER_EVENT]; // seq. record for marker
            eventNumber = new int[ChartTimeSeries.EventData.IndexTable.Count*MARKERS_PER_EVENT]; // event # of marker
            eventColor = new Color[ChartTimeSeries.EventData.IndexTable.Count*MARKERS_PER_EVENT]; // color of marker
            chartHeight = new float[ChartTimeSeries.EventData.IndexTable.Count*MARKERS_PER_EVENT]; // height of marker

            checkCount = GetActiveEventCount();

            // Initialize channel scaling
            channelCount = ChartTimeSeries.EventData.RawDataLabels.Count;  // number of channels to display
            channelScale = new float[channelCount];    // scale factors for channels
            for(j=0; j<channelCount; j++)       // assign default scaling values
            {
                channelScale[j] = 1.0F;
            }

            switch (FChartType) 
            {
                case TTimeSeriesChartType.ctSSI:
//                    time_format = DATETIME_WITH_MS;    // display milliseconds
                break;

                case TTimeSeriesChartType.ctLDP:
 //                   time_format = DATETIME_WITH_SEC;
                    if(!ChartTimeSeries.EventData.InputSettings.ChartSecondaryUnits)
                    {
                        for(j=0; j<ChartTimeSeries.EventData.LDPChannelTable.Count; j++)
                        {
                            channelRecord = ChartTimeSeries.EventData.LDPChannelTable[j];
                            channelScale[j] = channelRecord.ChannelScale;
                        }
                    }
                break;
            }

//            ChartSeries = new Steema.TeeChart.WPF.Styles.Custom[FChartTimeSeries.ChartVariableCount];
//            ChartSeries.Series.Clear;
           
            // Initialize chart axis scale

            InitChartMinMax(reset, out xMin, out xMax, out yMin, out yMax);

            // Initialize chart attributes

            InitChartAttributes(channelCount);

            // Fill data into chart series

            if((FChartType == TTimeSeriesChartType.ctLDP) || (FChartTimeSeries.EventData.IndexTable.Count == 0))     // no events, display all data
            {
                for (index = 0; index < FChartTimeSeries.EventData.RawDataTable.Count; index++)
                {
                    dataRecord = FChartTimeSeries.EventData.RawDataTable[index];
                    if ((dataRecord.Status == SKIP_RECORD) || (dataRecord.Status == TEST_RECORD))
                    {
                        continue;  // don"t display empty records
                    }
                    myRecord = dataRecord.RecordNumber;
                    for (j = 0; j < channelCount; j++)   // add every channel for this record to the chart
                    {
                        FChartTimeSeries.ChartDisplay.Series[j].Add(myRecord, dataRecord.Value[j] * channelScale[j]);
                        mycount = FChartTimeSeries.ChartDisplay.Series[j].Count;
                    }
                }

                if (FChartType == TTimeSeriesChartType.ctLDP) // display SER events
                {
                    DisplaySEREvents(checkCount);
                }
            }
            else   // display only events in chart
            {
                //for (j = 0; j < channelCount; j++)   // add every channel for this record to the chart
                //{
                //    FChartTimeSeries.ChartDisplay.Series.Add(new Steema.TeeChart.WPF.Styles.Line());
                //    FChartTimeSeries.ChartDisplay.Series[j].FillSampleValues(FChartTimeSeries.EventData.RawDataTable.Count);
                //}


                if (FChartTimeSeries.ChartHasEvents)
                {
                    BarSeries.Color = INTERVAL_COLOR;
                }

                rec = 0;
                activeCount = 0;
                markCount = 0;
                for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)   // look at event index records
                {
                    dataIndex = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                    if ((checkCount > 0) && (dataIndex.Checked == false)) continue;   // event not selected for display

                    start = dataIndex.FirstRecord; // get first/last record for complete event
                    stop = dataIndex.LastRecord;   //      including pre-/post-disturbance
                    estart = dataIndex.StartEvent; // get first/last record for actual event
                    estop = dataIndex.StopEvent;   //      excluding pre-/post-disturbance

                    parentEvent = -1;  // assume no sub-events
                    if (FChartType == TTimeSeriesChartType.ctSSI) // test for sub-event only if SSI
                    {
                        aCount = 0;  // active events so far
                        for (j = 0; j < i; j++)
                        {
                            dataIndex = FChartTimeSeries.EventData.IndexTable[j];
                            if ((checkCount > 0) && !dataIndex.Checked) continue;
                            if (start < dataIndex.LastRecord)   // it is a sub-event of active event
                            {
                                parentEvent = aCount;   // index of parent event
                                break;
                            }
                            aCount++;
                        }
                    }

                    chartRec[activeCount] = rec;
                    eventRec[activeCount] = start;

                    for (index = start; index <= stop; index++)
                    {
                        if (parentEvent >= 0)   // this is a sub-event (don"t display)
                        {
                            myRecord = index - eventRec[parentEvent] + chartRec[parentEvent] + 1;
                        }
                        else
                        {              // standard event, display data trace
                            myRecord = rec + 1;
                            dataRecord = ChartTimeSeries.EventData.RawDataTable[index];
                            for (j = 0; j < channelCount; j++)    // add every channel for this record to the chart
                            {
                                FChartTimeSeries.ChartDisplay.Series[j].Add(myRecord, dataRecord.Value[j]);
                            }
                        }

                        RecordEventMarkers(i, parentEvent, index, start, stop, estart, estop, myRecord,
                                   ref chartMarker, ref eventNumber, ref chartHeight, ref eventColor, ref markCount);

                        if (parentEvent < 0)  // only increment record if not sub-event
                        {
                            rec++;
                        }
                    }
                    activeCount++;
                }

                //// Display event markers

                DisplayEventMarkers(checkCount, markCount, chartMarker, eventNumber,
                        chartHeight, eventColor);
            }

            // Update display attributes

            FChartTimeSeries.InitChartTitles();
            FChartTimeSeries.InitChartActiveChannels();
            SetChartLineStyles();
            if (reset)
            {
                ResetChartControls();   // update UI controls from present settings
            }
            UpdateChartMinMax(reset, xMin, xMax, yMin, yMax);

        }
/************************************************************************
* NAME:   GetActiveEventCount
* FUNCTIONAL DESCRIPTION: Get number of active events in the chart
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: result : integer  number of active events
************************************************************************/
        public int GetActiveEventCount()
        {
            int checkCount,j;
            TTimeSeriesIndexRecord dataIndex;

            checkCount = 0;    // initialize number events selected for display (SSI only)
            for(j=0; j<FChartTimeSeries.EventData.IndexTable.Count; j++)   // count selected events
            {
                dataIndex = FChartTimeSeries.EventData.IndexTable[j]; // get the event index record
                if(dataIndex.Checked)
                {
                    checkCount++;
                }
            }
            return checkCount;
        }
/************************************************************************
* NAME:   GetCBEMA
* FUNCTIONAL DESCRIPTION: Get the CBEMA option from the input settings object
* INPUT PARAMETERS:
* OUTPUT PARAMETERS:
* RETURN: result : boolean  true if CBEMA active otherwise false
************************************************************************/
        public Boolean GetCBEMA()
        {
            if(FChartType == TTimeSeriesChartType.ctSSI)
            {
                return (ChartTimeSeries.EventData.InputSettings as TTimeSeriesInputSettingsSSI).ChartCBEMA;
            }
            else
            {
                return false;
            }
        }
/************************************************************************
* NAME:   SetCBEMA
* FUNCTIONAL DESCRIPTION: Set the CBEMA option in the input settings object
* INPUT PARAMETERS: doCBEMA : boolean  true if CBEMA chart display is desired
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void SetCBEMA(Boolean doCBEMA)
        {
            if(FChartType == TTimeSeriesChartType.ctSSI)
            {
                (FChartTimeSeries.EventData.InputSettings as TTimeSeriesInputSettingsSSI).ChartCBEMA = doCBEMA;
            }
        }
/************************************************************************
* NAME:  InitChartMinMax
* FUNCTIONAL DESCRIPTION:  initialize chart axis scaling
* INPUT PARAMETERS: reset : boolean  true if discarding current scale information
* OUTPUT PARAMETERS:
*                   xMin : single  minimum value of x-axis scale
*                   xMax : single  maximum value of x-axis scale
*                   yMin : single  minimum value of y-axis scale
*                   yMax : single  maximum value of y-axis scale
* RETURN: none
************************************************************************/
        public void InitChartMinMax(Boolean reset, out double xMin,out double xMax,out double yMin,out double yMax)
        {
            if(reset)   // throw away current scale information
            {
                xMin = 0.0;
                xMax = 1.0;
                yMin = 0.0;
                yMax = 1.0;
            }
            else
            {           // remember current axis limits
                xMin = FChartTimeSeries.ChartDisplay.Axes.Bottom.Minimum;
                xMax = FChartTimeSeries.ChartDisplay.Axes.Bottom.Maximum;
                yMin = FChartTimeSeries.ChartDisplay.Axes.Left.Minimum;
                yMax = FChartTimeSeries.ChartDisplay.Axes.Left.Maximum;
            }
            FChartTimeSeries.ChartDisplay.Axes.Bottom.Automatic = false;
            FChartTimeSeries.ChartDisplay.Axes.Left.Automatic = false;
        }
/************************************************************************
* NAME:    UpdateChartMinMax
* FUNCTIONAL DESCRIPTION:   Update chart axis scaling after chart is finished
* INPUT PARAMETERS: reset : boolean  true if discarding current scale information
*                   xMin : single  minimum value of x-axis scale
*                   xMax : single  maximum value of x-axis scale
*                   yMin : single  minimum value of y-axis scale
*                   yMax : single  maximum value of y-axis scale
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void UpdateChartMinMax(Boolean reset, double xMin,double xMax,double yMin,double yMax)
        {
            if(reset)
            {
                ChartTimeSeries.AutoScaleAxes();   // use customized axis scaling
            }
            else
            {
                ChartTimeSeries.ChartDisplay.Axes.Bottom.Automatic = false;
                ChartTimeSeries.ChartDisplay.Axes.Left.Automatic = false;
                ChartTimeSeries.ChartDisplay.Axes.Bottom.Minimum = xMin;
                ChartTimeSeries.ChartDisplay.Axes.Bottom.Maximum = xMax;
                ChartTimeSeries.ChartDisplay.Axes.Left.Minimum = yMin;
                ChartTimeSeries.ChartDisplay.Axes.Left.Maximum = yMax;
            }
        }
/************************************************************************
* NAME:   InitChartAttributes
* FUNCTIONAL DESCRIPTION:   Initialize attributes of the chart
* INPUT PARAMETERS: channelCount : integer   number of indep. variables
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void InitChartAttributes(int channelCount)
        {
            string[] CBEMA_TITLES = new string[2] {"ITI Sag","ITI Swell"};
            const string EVENT_TITLE = "Event";
            Steema.TeeChart.WPF.Styles.Line lineSeries;
            Steema.TeeChart.WPF.Styles.Area areaSeries;
            Steema.TeeChart.WPF.Styles.Custom genericSeries;

            int j;

            ChartTimeSeries.ChartDisplay.Header.Font.Color = Colors.Black;
            ChartTimeSeries.ChartDisplay.Header.Font.Name = "Tahoma";
            ChartTimeSeries.ChartDisplay.Header.Font.Size = 12;
            ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.RoundFirstLabel = true;
            ChartTimeSeries.ChartDisplay.Legend.CheckBoxes = true;
            ChartTimeSeries.ChartDisplay.Legend.Title.Visible = true;
            //ChartTimeSeries.ChartDisplay.Legend.Title.Clear;
            ChartTimeSeries.ChartDisplay.Legend.Title.Text= "Channel";

            if(FChartTimeSeries.EventData.InputSettings.ChartAxisDateTime)    // display date/time X axis labels
            {
                FChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Angle = 90;
            }
            else                               // display record number X axis labels
            {
                FChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Angle = 0;
                FChartTimeSeries.ChartDisplay.Axes.Bottom.Automatic = true;
            }

            // Allocate and initialize chart series

            FChartTimeSeries.ChartDisplay.Series.Clear();

            for(j=0; j<FChartTimeSeries.ChartVariableCount; j++)        // for each channel
            {
                switch(FChartTimeSeries.EventData.InputSettings.ChartStyle)
                {
                    case TTimeSeriesChartRenderType.sctLine:
                        lineSeries = new Steema.TeeChart.WPF.Styles.Line();
                        FChartTimeSeries.ChartDisplay.Series.Add(lineSeries);
                        genericSeries = lineSeries;
//                        ChartSeries[j] = new Steema.TeeChart.WPF.Styles.Line();    // create line chart
                    break;
                    
                    case TTimeSeriesChartRenderType.sctArea:
                        areaSeries = new Steema.TeeChart.WPF.Styles.Area();
                        FChartTimeSeries.ChartDisplay.Series.Add(areaSeries);
                    // ChartSeries[j] = new Steema.TeeChart.WPF.Styles.Area();   // create area chart
                        areaSeries.Transparency = FChartTimeSeries.EventData.InputSettings.ChartTransparency;  // percent transparency
                        genericSeries = areaSeries;
                    break;

                    default:
                        System.Diagnostics.Debug.Assert(false,"Invalid chart style.");
                        genericSeries = null;
                    break;
                }
//                FChartTimeSeries.ChartDisplay.Series[j].Clear();

                if((FChartTimeSeries.EventData.RawDataTable.Count > 50) && 
                    (FChartTimeSeries.EventData.InputSettings.ChartStyle == TTimeSeriesChartRenderType.sctArea))
                    genericSeries.LinePen.Visible = false;   // suppress grid lines
                else
                    genericSeries.LinePen.Visible = true;


                if(j >= channelCount)  // it is a CBEMA curve
                {
                    System.Diagnostics.Debug.Assert(((j-channelCount) >= 0) &&
                      ((j-channelCount) < CBEMA_TITLES.Length-1),
                      "Array length mismatch");
                    genericSeries.Stairs = true;  // use step function style for CBEMA
                    genericSeries.Title = CBEMA_TITLES[j - channelCount];  // assign channel name to legend
                }
                else
                {
                    genericSeries.Title = ChartTimeSeries.EventData.RawDataLabels[j];  // assign channel name to legend
                }

//                FChartTimeSeries.ChartDisplay.Series.Add(ChartSeries[j]);           // add the new channel series to the chart
            }
 //           ChartSeries = FChartTimeSeries.ChartDisplay.Series;

            if ((FChartType == TTimeSeriesChartType.ctSSI) && FChartTimeSeries.ChartHasEvents)
            {
                BarSeries = new Steema.TeeChart.WPF.Styles.Bar();
                BarSeries.Title = EVENT_TITLE;  // assign channel name to legend
                BarSeries.Marks.Visible = ChartTimeSeries.EventData.InputSettings.ChartLabels;
                BarSeries.Marks.Clip = true;
                //    BarSeries.Marks.Shadow.Clip = true;
                BarSeries.Marks.Shadow.Visible = false;   // hide shadow (doesn"t clip)
                FChartTimeSeries.ChartDisplay.Series.Add(BarSeries);  // add the new series to the chart
            }

            if((ChartType == TTimeSeriesChartType.ctLDP) && ChartTimeSeries.EventsInInterval())
            {
                PointSeries = new Steema.TeeChart.WPF.Styles.Points();
                PointSeries.Title = EVENT_TITLE;  // assign channel name to legend
                PointSeries.Marks.Visible = ChartTimeSeries.EventData.InputSettings.ChartLabels;
                PointSeries.Marks.Clip = true;
                //   PointSeries.Marks.Shadow.Clip = true;
                PointSeries.Marks.ArrowLength = 20;
                PointSeries.Marks.Shadow.Visible = false;   // hide shadow (doesn"t clip)
                FChartTimeSeries.ChartDisplay.Series.Add(PointSeries);  // add the new series to the chart
            }
        }
/************************************************************************
* NAME:    DisplaySEREvents
* FUNCTIONAL DESCRIPTION:   Display LDP events in chart
* INPUT PARAMETERS: checkCount : number of active channels
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void DisplaySEREvents(int checkCount)
        {
            double value,yvalue;
            int rec,i;
            string myLabel;
            TTimeSeriesIndexRecord dataIndex;

            // display all event symbols along a line where LDP chart data is minimum
            //yvalue = FChartTimeSeries.ChartDisplay.MinYValue(FChartTimeSeries.ChartDisplay.Axes.Left);
            yvalue = FChartTimeSeries.ChartDisplay.Axes.Left.Minimum;
            rec = 0;
            for(i=0;  i<FChartTimeSeries.EventData.IndexTable.Count; i++)
            {
                dataIndex = FChartTimeSeries.EventData.IndexTable[i];
                // See if event occurred within range of LDP date/time limits
                value = FChartTimeSeries.GetRecordFromEventTime(dataIndex.DateTime);
                if(value < 0.0)  // not in range of LDP date/time limits
                {
                    continue;
                }
                if((checkCount > 0) && !dataIndex.Checked)  // not selected by user
                {
                    continue;
                }
                // trim off the leading # sign
                myLabel = dataIndex.EventName.Substring(1);
                PointSeries.Add(value,yvalue); // value is interpolated record number
                PointSeries.Marks.Items[rec].Color = EVENTMARKER_COLOR;
                PointSeries.Marks.Items[rec].Text = myLabel;  // marker is event number
                rec++;
            }
        }
/************************************************************************
* NAME:    RecordEventMarkers
* FUNCTIONAL DESCRIPTION:   Record information about SSI event markers 
* INPUT PARAMETERS: iEvent : integer event number
*                   parentEvent : integer  event number of parent event
*                   index : integer  record number
*                   start : integer  beginning of pre-disturbance interval
*                   stop : integer   end of post-disturbance interval
*                   estart : integer beginning of actual event
*                   estop  : integer end of actual event
* OUTPUT PARAMETERS:
*                   chartMarker : array of integer  rec where marker appears
*                   eventNumber : array of integer  event number label of marker
*                   chartHeight : array of single   height of marker
*                   eventColor : array of TColor   color of marker
*                   markCount : integer number of markers
* RETURN: none
************************************************************************/
        public void RecordEventMarkers(int iEvent, int parentEvent,
                 int index, int start, int stop, int estart, int estop,
                 double myRecord,
                 ref int[] chartMarker, ref int[] eventNumber,
                 ref float[] chartHeight,
                 ref Color[] eventColor,
                 ref int markCount)
        {
            const float INTERVAL_HEIGHT = 100.0F;
            const float EVENT_HEIGHT = 50.0F;

            if(FChartTimeSeries.ChartHasEvents)
            {
                if((index == start) || (index == stop))     // pre/post event interval
                {
                    chartHeight[markCount] = INTERVAL_HEIGHT;
                    if((index == estart) || (index == estop))  // also limit of actual event
                    {
                        eventColor[markCount] = PARENTEVENT_COLOR;   // default color
                        if(parentEvent >= 0)       // sub-event
                        {
                            eventColor[markCount] = SUBEVENT_COLOR;
                            chartHeight[markCount] = EVENT_HEIGHT;
                        }
                        if(index == estart)
                        {
                            eventNumber[markCount] = -(iEvent+1);   // neg. indicates beginning
                        }
                        else
                        {
                            eventNumber[markCount] = iEvent+1;      // pos. indicates end
                        }
                        chartMarker[markCount] = Convert.ToInt32(Math.Truncate(myRecord));
                        markCount++;
                    }
                    else
                    {
                        eventNumber[markCount] = 0; // don"t display label
                        chartMarker[markCount] = Convert.ToInt32(Math.Truncate(myRecord));
                        eventColor[markCount] = INTERVAL_COLOR;
                        markCount++;
                    }
                }
                else if((index == estart) || (index == estop))    // Must end with interrupt sub-event
                {
                    chartHeight[markCount] = EVENT_HEIGHT;
                    if(index == estart)
                    {
                        eventNumber[markCount] = -(iEvent+1);   // neg. indicates beginning
                    }
                    else
                    {
                        eventNumber[markCount] = iEvent+1;     // pos. indicates end
                    }
                    chartMarker[markCount] = Convert.ToInt32(Math.Truncate(myRecord));
                    eventColor[markCount] = PARENTEVENT_COLOR;
                    markCount++;
                }
            }
        }
/************************************************************************
* NAME:    DisplayEventMarkers
* FUNCTIONAL DESCRIPTION:  Display SSI event markers in chart
* INPUT PARAMETERS: checkCount : integer number of active channels in legend
*                   markCount : integer number of markers
*                   chartMarker : array of integer  rec where marker appears
*                   eventNumber : array of integer  event number label of marker
*                   chartHeight : array of single   height of marker
*                   eventColor : array of TColor   color of marker
* OUTPUT PARAMETERS: (if used)
* RETURN: (if not a procedure)
************************************************************************/
        public void DisplayEventMarkers(int checkCount,int markCount,
                         int[] chartMarker,int[] eventNumber,
                         float[] chartHeight ,
                         Color[] eventColor)
        {
            const int EN_MAX = 5;   // maximum number of coincident event markers

            int rec, start, stop;
            int i, j, index,lastMarker;
            TTimeSeriesIndexRecord dataIndex;
            int ivalue;
            Color icolor;
            float rvalue;
            int[] enList = new int[6];   // list of coincident markers
            int enCount,enFound ;
            string myLabel;

            /*Event markers identify the beginning and end of each event.  There are two
            markers to delimit the entire event including pre-disturbance and post-disturbance.
            There are two markers to delimit the actual event excluding the pre-/post-disturbance
            intervals.  Sometimes there is no pre- or post-disturbance interval.

            Sorting is required in case an event interval contains one or more sub-events.
            If this is the case, the sub-events have been merged into the parent event
            and the terminating event has been removed.   The terminating marker from
            the parent event must follow the markers for the sub-events (interrupts).
            */
            // sort by event markers by record number where event ends

            for(i=0;  i<markCount - 1; i++)
            {
                for (j = i+1; j<markCount; j++)
                {
                    if(chartMarker[i] > chartMarker[j])
                    {
                        ivalue = chartMarker[j];
                        chartMarker[j] = chartMarker[i];
                        chartMarker[i] = ivalue;

                        ivalue = eventNumber[j];
                        eventNumber[j] = eventNumber[i];
                        eventNumber[i] = ivalue;

                        icolor = eventColor[j];
                        eventColor[j] = eventColor[i];
                        eventColor[i] = icolor;

                        rvalue = chartHeight[j];
                        chartHeight[j] = chartHeight[i];
                        chartHeight[i] = rvalue;
                    }
                }
            }

            rec = 0;
            stop = 0;  // use to test for sub-events
            lastMarker = 0;  // last marker drawn, don"t repeat search of previously drawn markers
            for(i=0;  i<FChartTimeSeries.EventData.IndexTable.Count; i++)   // look at event index records
            {
                dataIndex = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                if((checkCount > 0) && (dataIndex.Checked == false)) continue;   // event not selected for display
                if(dataIndex.LastRecord <= stop) continue;  // sub-event, skip it
                start = dataIndex.FirstRecord; // get first/last record for this event
                stop = dataIndex.LastRecord;
                enFound = 0;    // position in marker list where marker matches record
                // NOTE: index is just a counter, it is not used inside this for loop
                for (index = start; index<stop+2; index++)  // range of records for this event
                {
                    enCount = 0;                   // count number of coincident markers
                    for (j = lastMarker; j< markCount; j++)   // See if this record should be marked
                    {
                        if(rec < chartMarker[j]) break;   // no need to keep looking at markers
                        if(rec == chartMarker[j])  // display marker for this record
                        {
                            if(enCount < EN_MAX)  // make list of coincident markers
                            {
                                enList[enCount] = eventNumber[j];
                                enFound = j;       // use this position to get height, color
                                lastMarker = j+1;  // next marker to start search
                                enCount++;       // count number of coincident markers
                            }
                        }
                    }

                    if(enCount > 0)         // display event markers
                    {
                        FChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Style = Steema.TeeChart.WPF.AxisLabelStyle.Value;

                        // NOTE: pos/neg event number may be used to distinguish start/end of event

                        myLabel = "";
                        for(j=0; j<enCount; j++) // create a label for up to 5 concident markers
                        {
                            if(enList[j] < 0)   // beginning of event interval
                            {
                                myLabel = myLabel + string.Format("{0:d},",-enList[j]);
                            }
                            else if(enList[j] > 0)   // end of event interval
                            {
                                myLabel = myLabel + string.Format("{0:d},",enList[j]);
                            }
                        }

                        if(myLabel.Length > 0)   // add a label to the marker
                        {
                            myLabel = myLabel.Substring(0,myLabel.Length-1);  // trim trailing comma
//                            BarSeries.Marks.Items[rec].Text = myLabel;
                            //          BarSeries.Marks[rec].Text.Add(myLabel);
                        }
                        else
                        {
                            myLabel = "";
//                            BarSeries.Marks.Items[rec].Text = "";
                            //         BarSeries.Marks[rec].Text.Add("");  // no label
                        }
                        //BarSeries.Add(rec, chartHeight[enFound], myLabel, eventColor[enFound]);
                        BarSeries.Add(rec, chartHeight[enFound],eventColor[enFound]);
                        if (myLabel.Length == 0)
                        {
                            BarSeries.Marks.Items[rec].Visible = false;
                        }
                        else
                        {
 //                           BarSeries.Marks.Items[rec].Text = myLabel;
                            BarSeries.Labels[rec] = myLabel;
                        }
                        //       BarSeries.ValueColor[rec] = eventColor[enFound];
                    }
                    else
                    {                 // add placeholder markers
                        BarSeries.Add(rec,0.0,INTERVAL_COLOR);
                        BarSeries.Marks.Items[rec].Visible = false;
                        //BarSeries.Marks.Items[rec].Text = "";
                        //        BarSeries.Marks[rec].Text.Add("");
                        //BarSeries.ValueColor[rec] = INTERVAL_COLOR;
                    }
                    rec++;
                }
            }
        }
/******************************************************************************
* MODULE NAME ...  DisplayCBEMA
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
* FUNCTIONAL DESCRIPTION:  Create the chart, assign attributes and insert data
*                          display of CBEMA curves
*
* INPUT PARAMETERS:
*                    reset: boolean  true if regenerating display (rescale axis limits),
*                                    false if updating display (retain current axis limits)
* OUTPUT PARAMETERS:
*                    None:
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        public void DisplayCBEMA(Boolean reset)
        {
            Color[] CBEMA_COLOR = new Color[8]
            {
                Colors.Green, Colors.Olive, Colors.GreenYellow, Colors.Yellow, 
                Colors.Brown, Colors.DarkOrange, Colors.OrangeRed, Colors.Red
            };

            int i,checkCount,rec,start,stop;
            double xMin,xMax,yMin,yMax;
            double duration, amplitude;
            TTimeSeriesIndexRecord dataIndex;
            //TTimeStamp myTimeStamp;
            double[] sDuration, sAmplitude;
            int[] sEvent;
            TTimeSeriesCBEMAViolationType[] sEventType;
            double[] sagValues, swellValues;
            double maxDuration;

            if(ChartTimeSeries.ChartVariableCount == 0) return;   // no data

            maxDuration = 30.0;   // initialize to minimum reasonable value

            checkCount = GetActiveEventCount();

            InitChartMinMax(reset,out xMin,out xMax,out yMin,out yMax);

            InitChartAttributesCBEMA();

            sDuration = new double[ChartTimeSeries.EventData.IndexTable.Count];
            sEvent = new int[ChartTimeSeries.EventData.IndexTable.Count];
            sEventType = new TTimeSeriesCBEMAViolationType[ChartTimeSeries.EventData.IndexTable.Count];
            sAmplitude = new double[ChartTimeSeries.EventData.IndexTable.Count];

            rec = 0;
            for(i=0;  i<ChartTimeSeries.EventData.IndexTable.Count; i++)   // look at event index records
            {
                dataIndex = ChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                if((checkCount > 0) && (dataIndex.Checked == false)) continue;   // event not selected for display

                start = dataIndex.FirstRecord; // get first/last record for this event
                stop = dataIndex.LastRecord;

                sagValues = new double[stop-start+1];
                swellValues = new double[stop-start+1];
                sEventType[rec] = dataIndex.Violation;

                // Convert a TDateTime duration to milliseconds

                //myTimeStamp = DateTimeToTimeStamp(dataIndex.Duration);
                //// get number of days since reference date
                //Dec(myTimeStamp.Date, TTimeStamp(DateTimeToTimeStamp(0)).Date);

                //// number of milliseconds
                //duration = (myTimeStamp.Date * 24 * 60 * 60) + (myTimeStamp.Time / 1000.0);

                duration = dataIndex.Duration.TotalSeconds;

                amplitude = dataIndex.Magnitude;
                sDuration[rec] = duration;
                sAmplitude[rec] = amplitude;
                if(maxDuration < duration)
                {
                    maxDuration = duration;
                }
                sEvent[rec] = i+1;
                rec++;
            }

            SortEventsByDuration(rec,ref sDuration,ref sAmplitude,ref sEvent,ref sEventType);

 //           ChartTimeSeries.ChartDisplay.Series[0].Marks.Style = MarksStyles.Label;
            for(i=0;  i<rec; i++)   // Update event marker color/number
            {
                ChartTimeSeries.ChartDisplay.Series[0].Add(sDuration[i],sAmplitude[i],CBEMA_COLOR[(int)sEventType[i]]);
                ChartTimeSeries.ChartDisplay.Series[0].Labels[i] = (string.Format("{0:d}", sEvent[i]));
//                ChartTimeSeries.ChartDisplay.Series[0].Marks.Items[i].Text = (string.Format("{0:d}",sEvent[i]));
//                ChartTimeSeries.ChartDisplay.Series[0].Marks.Items[i].Color = CBEMA_COLOR[(int)sEventType[i]];
            }

            // Display Sag/Swell limits

            DisplayCBEMALimits(maxDuration);

            // Update display attributes  (titles and scaling)

            FChartTimeSeries.InitChartTitles();
            FChartTimeSeries.ChartDisplay.Axes.Bottom.Increment = 0.001;
            FChartTimeSeries.ChartDisplay.Axes.Bottom.Minimum = 0.0;
            FChartTimeSeries.ChartDisplay.Axes.Bottom.AutomaticMinimum = false;
            FChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Angle = 0;
            FChartTimeSeries.ChartDisplay.Axes.Bottom.Automatic = true;
            FChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Style = Steema.TeeChart.WPF.AxisLabelStyle.Value;

            UpdateChartMinMax(reset,xMin,xMax,yMin,yMax);
        }
/************************************************************************
* NAME:       InitChartAttributesCBEMA
* FUNCTIONAL DESCRIPTION:   Initialize chart attributes for CBEMA chart
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void InitChartAttributesCBEMA()
        {
            string[] CBEMA_LABELS =new string[3] {"Sag Limit","Swell Limit","Event"};

            FChartTimeSeries.ChartDisplay.Header.Font.Color = Colors.Black;
            FChartTimeSeries.ChartDisplay.Header.Font.Name = "Tahoma";
            FChartTimeSeries.ChartDisplay.Header.Font.Size = 12;
            FChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.RoundFirstLabel = true;
            FChartTimeSeries.ChartDisplay.Legend.CheckBoxes = true;
            FChartTimeSeries.ChartDisplay.Legend.Title.Visible = true;
            FChartTimeSeries.ChartDisplay.Legend.Title.Text = "";
            FChartTimeSeries.ChartDisplay.Legend.Title.Text = "Legend";

            FChartTimeSeries.ChartDisplay.Series.Clear();   // throw away previous chart data

            // Allocate and initialize event series

            CBEMADataSeries = new Steema.TeeChart.WPF.Styles.Points();    // create line chart
            CBEMADataSeries.Title = CBEMA_LABELS[2];  // assign channel name to legend
            CBEMADataSeries.Pointer.Style = PointerStyles.Circle;
            CBEMADataSeries.Pointer.HorizSize = 5;
            CBEMADataSeries.Pointer.VertSize = 5;
            CBEMADataSeries.Pointer.Visible = true;
            CBEMADataSeries.Marks.Visible = FChartTimeSeries.EventData.InputSettings.ChartLabels;
            CBEMADataSeries.Marks.Clip = true;
            //CBEMADataSeries.Marks.Shadow.Clip = true;
            CBEMADataSeries.Marks.Shadow.Visible = false;  // hide shadow (doesn"t clip)
            CBEMADataSeries.Marks.ArrowLength = 20;
            FChartTimeSeries.ChartDisplay.Series.Add(CBEMADataSeries);  // add the new channel series to the chart
        }
/************************************************************************
* NAME:          SortEventsByDuration
* FUNCTIONAL DESCRIPTION:  Sort events by duration
* INPUT PARAMETERS: eventCount : integer number of events
* OUTPUT PARAMETERS:
*                   sDuration : array of double   event duration
*                   sAmplitude : array of double   event amplitude
*                   sEvent  : array of integer    event number
*                   sEventType : TTimeSeriesCBEMAViolation  violation type
* RETURN: none
************************************************************************/
        public void SortEventsByDuration(int eventCount,
                           ref double[] sDuration, ref double[] sAmplitude,
                           ref int[] sEvent,
                           ref TTimeSeriesCBEMAViolationType[] sEventType)
        {
            int i,j,myEvent;
            TTimeSeriesCBEMAViolationType myEventType;
            double duration, amplitude;

            for(i=0;  i<eventCount - 1; i++)   // sort events by duration
            {
                for (j =(i+1); j< eventCount; j++)
                {
                    if(sDuration[i] > sDuration[j])
                    {
                        duration = sDuration[i];
                        sDuration[i] = sDuration[j];
                        sDuration[j] = duration;
                        amplitude = sAmplitude[i];
                        sAmplitude[i] = sAmplitude[j];
                        sAmplitude[j] = amplitude;
                        myEvent = sEvent[i];
                        sEvent[i] = sEvent[j];
                        sEvent[j] = myEvent;
                        myEventType = sEventType[i];
                        sEventType[i] = sEventType[j];
                        sEventType[j] = myEventType;
                    }
                }
            }
        }
/************************************************************************
* NAME:  DisplayCBEMALimits
* FUNCTIONAL DESCRIPTION:  Display the sag and swell limit curves
* INPUT PARAMETERS: (if used)
* OUTPUT PARAMETERS: (if used)
* RETURN: (if not a procedure)
************************************************************************/
        public void DisplayCBEMALimits(double maxDuration)
        {
            string[] CBEMA_PT_LABELS = new string[3] {"Swell Limit","Sag Limit","Event"};
            double[] CBEMA_SAG_DURATION = new double[8] {0.0199, 0.02,0.5,10.0,15.0,20.0,25.0,10000.0};
            double[] CBEMA_SAG_AMPLITUDE = new double[8] {0.0,70.0,80.0,90.0,90.0,90.0,90.0,90.0};
            double[] CBEMA_SWELL_DURATION = new double[9] {0.0001,0.001,0.003,0.0031,0.5,1.0,10.0,20.0,10000.0};
            double[] CBEMA_SWELL_AMPLITUDE = new double[9] {500.0,200.0,140.0,120.0, 110.0, 110.0, 110.0, 110.0, 110.0};

            int i;
            double[] myCBEMA_SAG_DURATION = new double[8];
            double[] myCBEMA_SWELL_DURATION = new double[9];

            for(i=0;  i<2; i++)  // create each CBEMA sag/swell limit curve
            {
                CBEMALimitSeries[i] = new Steema.TeeChart.WPF.Styles.Line();
                CBEMALimitSeries[i].Title = CBEMA_PT_LABELS[i];  // assign channel name to legend
                CBEMALimitSeries[i].Stairs = true;  // use step function style for CBEMA
                CBEMALimitSeries[i].Visible = true;  // display step curves
                CBEMALimitSeries[i].LinePen.Width = 2;
                ChartTimeSeries.ChartDisplay.Series.Add(CBEMALimitSeries[i]); // add the new channel series to the chart
            }

            for (i = 0; i<CBEMA_SAG_DURATION.Length; i++)
            {
                myCBEMA_SAG_DURATION[i] = CBEMA_SAG_DURATION[i];
            }
            myCBEMA_SAG_DURATION[CBEMA_SAG_DURATION.Length-1] = maxDuration;

            for (i = 0; i<CBEMA_SWELL_DURATION.Length; i++)
            {
                myCBEMA_SWELL_DURATION[i] = CBEMA_SWELL_DURATION[i];
            }
            myCBEMA_SWELL_DURATION[CBEMA_SWELL_DURATION.Length-1] = maxDuration;

            for (i = 0; i<CBEMA_SWELL_DURATION.Length; i++)
            {
                ChartTimeSeries.ChartDisplay.Series[1].Add(myCBEMA_SWELL_DURATION[i],CBEMA_SWELL_AMPLITUDE[i]);
            }

            for (i = 0; i<CBEMA_SAG_DURATION.Length; i++)
            {
                ChartTimeSeries.ChartDisplay.Series[2].Add(myCBEMA_SAG_DURATION[i],CBEMA_SAG_AMPLITUDE[i]);
            }
        }
/******************************************************************************
* MODULE NAME ...  SetDateTimeLabels
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  set chart date/time labels
*
* INPUT PARAMETERS:
*                    Sender : TObject
*
* OUTPUT PARAMETERS:
*                    None:
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.11.13  Kent Myers        Original implementation
*******************************************************************************/
        public void SetDateTimeLabels()
        {
            const int DATE_PT_LENGTH_LDP = 110;
            const int DATE_PT_LENGTH_SSI = 125;
            const int RECORD_PT_LENGTH = 15;
            TTimeSeriesInputSettingsSSI mySSI;

            if (ChartTimeSeries == null) return;

            if(ChartTimeSeries.EventData.ChartType == TTimeSeriesChartType.ctSSI)      // Time/date not used for CBEMA
            {
                mySSI = (TTimeSeriesInputSettingsSSI)ChartTimeSeries.EventData.InputSettings;
                if (mySSI.ChartCBEMA) return;
            }

            if(ChartTimeSeries.EventData.InputSettings.ChartAxisDateTime)
            {
                ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Angle = 90;
                if(ChartTimeSeries.EventData.ChartType == TTimeSeriesChartType.ctSSI)
                {
                    ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.CustomSize = DATE_PT_LENGTH_SSI;
//                    ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Font.Size = DATE_PT_LENGTH_SSI;
                }
                else
                {
                    ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.CustomSize = DATE_PT_LENGTH_LDP;
                    //ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Font.Size = DATE_PT_LENGTH_LDP;
                }
            }
            else
            {
                ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Angle = 0;
                ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.CustomSize = RECORD_PT_LENGTH;
                //ChartTimeSeries.ChartDisplay.Axes.Bottom.Labels.Font.Size = RECORD_PT_LENGTH;
            }
            ChartTimeSeries.InitChartTitles();
//            ChartTimeSeries.ChartDisplay.
            ChartTimeSeries.ChartDisplay.Axes.Visible = false;
            ChartTimeSeries.ChartDisplay.Axes.Visible = true;
            //ChartTimeSeries.ChartDisplay.Update();
        }
/******************************************************************************
* MODULE NAME ...  SetChartLineStyles
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Set chart line styles (color, type)
*
* INPUT PARAMETERS:
*                    None:
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.10.1  Kent Myers        Original implementation
*******************************************************************************/
        public void SetChartLineStyles()
        {
            int i , pen_width;
            THMIChannelPhase cPhase;
            THMIChannelType cType;
            TTimeSeriesInputSettingsSSI mySSI;
            Steema.TeeChart.WPF.Styles.Custom genericSeries;

            if(FChartTimeSeries.ChartVariableCount == 0) return;   // no data

            if(FChartTimeSeries.EventData.ChartType == TTimeSeriesChartType.ctSSI)
            {
                mySSI = (TTimeSeriesInputSettingsSSI)FChartTimeSeries.EventData.InputSettings;
                if (mySSI.ChartCBEMA)
                {
                    return;
                }
            }

 //           if (ChartSeries.Length < 1)   // no data loaded into chart
            if (FChartTimeSeries.ChartDisplay.Series.Count < 1)   // no data loaded into chart
                {
                return;
            }

            // choose default pen widths.  Some channels will use different pen widths.
            if(FChartTimeSeries.ChartDisplay.Aspect.View3D)
            {
                pen_width = 1;    // want thin lines for 3D
            }
            else
            {
                pen_width = 3;    // want thick lines for 2D  (voltages and other channels)
            }

            // update pen width for each channel
            for(i=0;  i<FChartTimeSeries.EventData.RawDataLabels.Count; i++)
            {
                genericSeries = (Steema.TeeChart.WPF.Styles.Custom) FChartTimeSeries.ChartDisplay.Series[i];
                cPhase = FChartTimeSeries.GetChannelPhase(ChartTimeSeries.EventData.RawDataLabels[i]);
                cType = FChartTimeSeries.GetChannelType(ChartTimeSeries.EventData.RawDataLabels[i]);
                genericSeries.LinePen.Width = pen_width;
                genericSeries.Color = FChartTimeSeries.ChartLineColor(cPhase, i); // assign color to channel

                switch(FChartTimeSeries.EventData.InputSettings.ChartStyle)
                {
                    case TTimeSeriesChartRenderType.sctLine:
                        if(cType == THMIChannelType.ctCurrent)    // channel is current so use different pen width
                        {
                            if(FChartTimeSeries.ChartDisplay.Aspect.View3D)
                            {
                                genericSeries.LinePen.Width = 1;    // want thin lines for 3D
                            }
                            else
                            {
                                genericSeries.LinePen.Width = 2;
                            }
                        }
                        //         ChartSeries[i]AreaLinesPen.Style = FChartTimeSeries.ChartLineStyle(cType);
                        genericSeries.LinePen.Style = FChartTimeSeries.ChartLineStyle(cType);
                    break;

                    case TTimeSeriesChartRenderType.sctArea:
                    break;

                    default:
                        System.Diagnostics.Debug.Assert(false,"Invalid chart style.");
                    break;
                }
            }
        }
/******************************************************************************
* MODULE NAME ...  SetChartLabels
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:
* Set the chart variable names that are Form 5/9 specific
*
* INPUT PARAMETERS:
*                    None:
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                   None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.11.06  Kent Myers        Original code
*******************************************************************************/
        public void SetChartLabels()
        {
            const int FIRST_VOLTAGE_4XX = 0;
            const int FIRST_VOLTAGE = 5;

            int voltageColumn;    // skip currents
            GlobalString gs = new GlobalString();

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is nil");

            if(FChartTimeSeries.EventData.RawDataLabels.Count < 7)   // SEL4XX
            {
                voltageColumn = FIRST_VOLTAGE_4XX;
            }
            else
            {
                voltageColumn = FIRST_VOLTAGE;
            }

            switch (FChartType)
            {
                case TTimeSeriesChartType.ctSSI:              // update SSI chart variable names
                    if(gs.CompareText(FChartTimeSeries.EventData.RawDataLabels[voltageColumn],"VAB"))
                    {
                        comboBoxPhase.Items[0] = "AB";
                        comboBoxPhase.Items[1] = "BC";
                        comboBoxPhase.Items[2] = "CA";
                    }
                    else
                    {
                        comboBoxPhase.Items[0] = "A";
                        comboBoxPhase.Items[1] = "B";
                        comboBoxPhase.Items[2] = "C";
                    }
                break;

                case TTimeSeriesChartType.ctLDP:
                break;

                default:
                    System.Diagnostics.Debug.Assert(false, "unknown chart type");
                break;
            }
        }
/************************************************************************
* NAME:    CBEMAChartVisible
* FUNCTIONAL DESCRIPTION: select type of chart to display for SSI
* INPUT PARAMETERS: doCBEMA : boolean  true if CBEMA chart is visible
* OUTPUT PARAMETERS: none
* RETURN: none     result : boolean   true if display should be refreshed
************************************************************************/
        public Boolean CBEMAChartVisible(Boolean doCBEMA)
        {
            const int SSI_TAB = 1;

            Boolean reset;
            TTimeSeriesInputSettingsSSI mySSI;

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is nil");

            FChartTimeSeries.EventData.InputSettings.ChartQuery = false;    // cancel cursor query
            labelChartType.IsEnabled = !doCBEMA;
            comboBoxChartType.IsEnabled = !doCBEMA;
            if (doCBEMA)
            {
                buttonClock.Visibility = Visibility.Collapsed;
            }
            else
            {
                buttonClock.Visibility = Visibility.Visible;
            }
            toolBarChart.UpdateLayout();
            // Realign;

            if(doCBEMA)    // CBEMA
            {
                mySSI = (TTimeSeriesInputSettingsSSI)FChartTimeSeries.EventData.InputSettings;
                mySSI.ChartCBEMA = true;
                //    ChartHMIDisplay.SendToBack;
                reset = (ChartHMICBEMA.Series.Count == 0);
                FChartTimeSeries.ChartDisplay = ChartHMICBEMA;   // set active chart pointer
                FChartTimeSeries.CursorTool = ChartTool2;
                labelTransparency.IsEnabled = false;
                scrollBarTransparency.IsEnabled = false;
            }
            else              // SSI/LDP
            {
                if (ChartType == TTimeSeriesChartType.ctSSI)
                {
                    mySSI = (TTimeSeriesInputSettingsSSI)FChartTimeSeries.EventData.InputSettings;
                    mySSI.ChartCBEMA = false;
               }
                //    ChartHMICBEMA.SendToBack;
                reset = false;  // no need to reset
                FChartTimeSeries.ChartDisplay = ChartHMIDisplay;   // set active chart pointer
                FChartTimeSeries.CursorTool = ChartTool1;
                labelTransparency.IsEnabled = (comboBoxChartType.SelectedIndex == SSI_TAB);
                scrollBarTransparency.IsEnabled = (comboBoxChartType.SelectedIndex == SSI_TAB);
            }
            FChartTimeSeries.CursorTool.Active = false;
            FChartTimeSeries.CursorTool.Snap = true;
            return reset;
        }
/************************************************************************
* NAME:   FillEventListViewLDP
* FUNCTIONAL DESCRIPTION:  Fill list with LDP event information
* INPUT PARAMETERS: aReset : boolean  true will initialize checked entries
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void FillEventListViewLDP(Boolean aReset)
        {
            const string NO_EVENT_MSG = "Status: No events were found within the LDP data acquisition interval.";
            const string EVENT_MSG = "Status: {0:d} events found.";
            const int COLUMN_COUNT = 4;
            string[] COLUMN_NAMES = new string[COLUMN_COUNT] { "Event", "Date", "Time", "Description"};
            string[] BINDING_NAMES = new string[COLUMN_COUNT] { "Event", "Date", "Time", "Description"};
            int[] COLUMN_SIZE = new int[COLUMN_COUNT] { 60, 80, 80, 350 };

            int i;
            TTimeSeriesIndexRecord indexRecord;
            string myDate, myTime;
            //Boolean doRestore;
            //Boolean[] checkList, selectList;
            int[] myColumns = new int[COLUMN_COUNT];
            GlobalString gs = new GlobalString();
            TFormatSettings myFormat = gs.InitTFormatSettings();

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null, "EventData is nil");

            if (FChartTimeSeries.EventData.IndexTable.Count == 0)
            {
                listViewGrid.Columns.Clear();
                labelStatus.Content = NO_EVENT_MSG;
                return;  // no data
            }

            FChartTimeSeries.EventData.InitDateTimeFormat(TDateFormatType.ftInternational, ref myFormat);
            ObservableCollection<LDPEvent> events = new ObservableCollection<LDPEvent>();


            // preserve state if count is the same

            //doRestore = (FChartTimeSeries.EventData.IndexTable.Count == listViewChart.Items.Count);
            //if (doRestore)
            //{
            //    checkList = new Boolean[listViewChart.Items.Count];
            //    selectList = new Boolean[listViewChart.Items.Count];
            //    for (i = 0; i < itemSource.Count; i++)
            //    {
            //        checkList[i] = itemSource[i].Checked;
            //        selectList[i] = itemSource[i].Checked;
            //    }
            //}
            //else
            //{
            //    checkList = new Boolean[1];
            //    selectList = new Boolean[1];
            //}

            //// Restore previous column width if rows present
            //if ((listViewChart.Items.Count == COLUMN_COUNT) &&
            //   (itemSource.Count > 0))
            //{
            //    for (i = 0; i < COLUMN_COUNT; i++)   // restore previous columns widths
            //    {
            //        gvColumn = listViewGrid.Columns[i];
            //        myColumns[i] = Convert.ToInt32(gvColumn.Width);
            //    }
            //}
            //else
            //{
            //    for (i = 0; i < COLUMN_COUNT; i++)   // set default column width
            //    {
            //        myColumns[i] = COLUMN_SIZE[i];
            //    }
            //}

            //CreateEventListViewColumns(COLUMN_COUNT, COLUMN_NAMES, BINDING_NAMES, myColumns); 

           
            // fill list view with contents of device information

            CreateEventListViewColumns(COLUMN_COUNT, COLUMN_NAMES, BINDING_NAMES, COLUMN_SIZE);
            listViewChart.DataContext = events;

            for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
            {
                indexRecord = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                myDate = gs.DateToStr(indexRecord.DateTime, myFormat);  // setting date format
                myTime = string.Format(TIME_WITH_MS, indexRecord.DateTime);
                events.Add(new LDPEvent()
                {
                    Event = indexRecord.EventName,
                    Date = myDate,
                    Time = myTime,
                    Description = indexRecord.Description,
                });
            }

            // Update status label based on number of events found in index

            labelStatus.Content = string.Format(EVENT_MSG,FChartTimeSeries.EventData.IndexTable.Count);

            // Restore selected item(s)

            //if (doRestore && !aReset)
            //{
            //    for (i = 0; i < itemSource.Count; i++)
            //    {
            //        itemSource[i].Checked = checkList[i];
            //        //  ListViewEventList.Items[i].Checked = checkList[i];
            //        //ListViewEventList.Items[i].Selected = selectList[i];
            //        //ListViewEventList.Items[i].Focused = selectList[i];
            //    }
            //}
        }
        //public void FillEventListViewLDP(Boolean aReset)
        //{
        //    const string NO_EVENT_MSG = "Status: No events were found within the LDP data acquisition interval.";
        //    const string EVENT_MSG = "Status: {0:d} events found.";
        //    const int COLUMN_COUNT = 5;
        //    int[] COLUMN_SIZE = new int[COLUMN_COUNT] { 10, 60, 80, 80, 350 };
        //    string[] COLUMN_NAMES = new string[COLUMN_COUNT] { " ", "Event", "Date", "Time", "Description" };
        //    string[] BINDING_NAMES = new string[COLUMN_COUNT] { "Checked", "EventName", "Date", "Time", "Description" };

        //    int i;
        //    TTimeSeriesIndexRecord indexRecord;
        //    string myDate, myTime;
        //    Boolean doRestore;
        //    Boolean[] checkList, selectList;
        //    int[] myColumns = new int[COLUMN_COUNT];
        //    GlobalString gs = new GlobalString();
        //    TFormatSettings myFormat = gs.InitTFormatSettings();
        //    ObservableCollection<LDPEvent> itemSource;
        //    GridViewColumn gvColumn;

        //    FChartTimeSeries.EventData.InitDateTimeFormat(TDateFormatType.ftInternational, ref myFormat);
        //    itemSource = (ObservableCollection<LDPEvent>)listViewChart.ItemsSource;

        //    System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null, "EventData is nil");

        //    if (FChartTimeSeries.EventData.IndexTable.Count == 0)
        //    {
        //        listViewChart.Items.Clear();
        //        labelStatus.Content = NO_EVENT_MSG;
        //        return;  // no data
        //    }

        //    // preserve state if count is the same

        //    doRestore = (FChartTimeSeries.EventData.IndexTable.Count == listViewChart.Items.Count);
        //    if (doRestore)
        //    {
        //        checkList = new Boolean[listViewChart.Items.Count];
        //        selectList = new Boolean[listViewChart.Items.Count];
        //        for (i = 0; i < itemSource.Count; i++)
        //        {
        //            checkList[i] = itemSource[i].Checked;
        //            selectList[i] = itemSource[i].Checked;
        //        }
        //    }
        //    else
        //    {
        //        checkList = new Boolean[1];
        //        selectList = new Boolean[1];
        //    }

        //    // Restore previous column width if rows present
        //    if ((listViewChart.Items.Count == COLUMN_COUNT) &&
        //       (itemSource.Count > 0))
        //    {
        //        for (i = 0; i < COLUMN_COUNT; i++)   // restore previous columns widths
        //        {
        //            gvColumn = listViewGrid.Columns[i];
        //            myColumns[i] = Convert.ToInt32(gvColumn.Width);
        //        }
        //    }
        //    else
        //    {
        //        for (i = 0; i < COLUMN_COUNT; i++)   // set default column width
        //        {
        //            myColumns[i] = COLUMN_SIZE[i];
        //        }
        //    }

        //    CreateEventListViewColumns(COLUMN_COUNT, COLUMN_NAMES, BINDING_NAMES, myColumns);

        //    // fill list view with contents of event index
        //    itemSource = new ObservableCollection<LDPEvent>();
        //    listViewChart.ItemsSource = itemSource;

        //    for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
        //    {
        //        indexRecord = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
        //        myDate = gs.DateToStr(indexRecord.DateTime, myFormat);  // setting date format
        //        myTime = string.Format(TIME_WITH_MS, indexRecord.DateTime);

        //        itemSource.Add(new LDPEvent(false, indexRecord.EventName, myDate, myTime, indexRecord.Description));
        //    }

        //    // Update status label based on number of events found in index

        //    labelStatus.Content = string.Format(EVENT_MSG, FChartTimeSeries.EventData.IndexTable.Count);

        //    // Restore selected item(s)

        //    if (doRestore && !aReset)
        //    {
        //        for (i = 0; i < itemSource.Count; i++)
        //        {
        //            itemSource[i].Checked = checkList[i];
        //            //  ListViewEventList.Items[i].Checked = checkList[i];
        //            //ListViewEventList.Items[i].Selected = selectList[i];
        //            //ListViewEventList.Items[i].Focused = selectList[i];
        //        }
        //    }
        //}
        /************************************************************************
        * NAME:     FillEventListViewSSI
        * FUNCTIONAL DESCRIPTION:  Create list of events in UI List View control
        * INPUT PARAMETERS: aReset : boolean  true will initialize checked entries
        * OUTPUT PARAMETERS: none
        * RETURN:           none
        ************************************************************************/
        //public void FillEventListViewSSIx(Boolean aReset)
        //{
        //    ObservableCollection<VSSIEvent> itemSource;
        //    GridViewColumn gvColumn;

        //    const int COLUMN_COUNT = 14;
        //    const string DATAFORMAT = "{0,5:0.0}";
        //    int[] COLUMNSIZE = new int[COLUMN_COUNT] 
        //     {10, 60,   45,   70,   80,    85,       80,        75,     60,   60,   60,   60,   60,   60};
        //    string[] COLUMN_NAMES = new string[COLUMN_COUNT]
        //     {" ","Event","Type","Date","Time","Duration","Event Depth","ITIC Region",
        //      "Va Min","Va Max","Vb Min","Vb Max","Vc Min","Vc Max"};
        //    string[] BINDING_NAMES = new string[COLUMN_COUNT]
        //     {"EventChecked","EventName","EventType","Date","Time","Duration","Magnitude","Violation",
        //      "VaMin","VaMax","VbMin","VbMax","VcMin","VcMax"};
        //    string violation, vaMin, vaMax, vbMin, vbMax, vcMin, vcMax;

        //    int i;
        //    int[] myColumns = new int[COLUMN_COUNT];
        //    string myDate,myTime, myDuration, eventMag, myName, myDesc;

        //    for (i = 0; i < COLUMN_COUNT; i++)   // set default column width
        //    {
        //        myColumns[i] = COLUMNSIZE[i];
        //    }

        //    listViewChart.Items.Clear();
        //    listViewGrid.Columns.Clear(); // Erase existing columns            
            
        //    // create controls in listview
        //    for(i=0;  i<COLUMN_COUNT; i++)
        //    {
        //        gvColumn = new GridViewColumn();
        //        gvColumn.Header = COLUMN_NAMES[i];
        //        gvColumn.Width = myColumns[i];

        //        if (i == 0)  // HERE IS WHERE I TRY TO DEFINE THE CHECKBOX
        //        {
        //            gvColumn.CellTemplate = new DataTemplate("StaticResource FirstCell");  // XAML resource
        //        }
        //        gvColumn.DisplayMemberBinding = new Binding(BINDING_NAMES[i]);
        //        listViewGrid.Columns.Add(gvColumn);
        //    }

        //    // fill list view with contents of event index
        //    itemSource = new ObservableCollection<VSSIEvent>();
        //    listViewChart.ItemsSource = itemSource;

        //    for(i=0;  i<2; i++)
        //    {
        //        myName = "Name" + i.ToString();
        //        myDesc = "Desc" + i.ToString();
        //        myDate = "1/1/2009";  // setting date format
        //        myTime = "10:00:00";
        //        myDuration = "10:10:10:000";
        //        eventMag = "$$$$$";    // Magnitude not meaningful
        //        violation = "Bad";
        //        vaMin = string.Format(DATAFORMAT,1.0);
        //        vaMax = string.Format(DATAFORMAT,2.0);
        //        vbMin = string.Format(DATAFORMAT,3.0);
        //        vbMax = string.Format(DATAFORMAT,4.0);
        //        vcMin = string.Format(DATAFORMAT,5.0);
        //        vcMax = string.Format(DATAFORMAT,6.0);
        //        itemSource.Add(new VSSIEvent(false, myName,
        //            myDesc, myDate, myTime, myDuration, eventMag, violation,
        //            vaMin, vaMax, vbMin, vbMax, vcMin, vcMax));
        //    }
        //}

        /************************************************************************
        * NAME:     FillEventListViewSSI
        * FUNCTIONAL DESCRIPTION:  Create list of events in UI List View control
        * INPUT PARAMETERS: aReset : boolean  true will initialize checked entries
        * OUTPUT PARAMETERS: none
        * RETURN:           none
        ************************************************************************/

        //public void FillEventListViewSSI(Boolean aReset)
        //{
        //    const string NO_EVENTS_MSG = "Status: No events were found matching the present event selection criteria.";
        //    const string EVENTS_FOUND_FMT = "Status: {0:d} events found.";
        //    const string EVENTS_FOUND_PHASE_FMT = "Status: {0:d} events found in which %s on Phase %s occurred.";
        //    const string EVENTS_FOUND_TYPE_FMT = "Status: {0:d} events found in which %s occurred.";
        //    const string EVENTS_FOUND_MIXED_FMT = "Status: {0:d} events found in which a Sag, Swell, or Interrupt on Phase %s occurred.";
        //    const int COLUMN_COUNT = 14;
        //    const string DATAFORMAT = "{0,5:0.0}";
        //    string[] MYPHASE = new string[4] { "A", "B", "C", "Any" };
        //    string[] MYTYPE = new string[5] { "None", "a Sag", "a Swell", "an Interrupt", "Any" };
        //    int[] COLUMN_SIZE = new int[COLUMN_COUNT] { 10, 60, 45, 70, 80, 85, 80, 75, 60, 60, 60, 60, 60, 60 };
        //    // Event, Type, Date, Start, Duration, Magnitude, Region, VaMin...
        //    string[] COLUMN_NAMES = new string[COLUMN_COUNT]
        //     {" ","Event","Type","Date","Time","Duration","Event Depth","ITIC Region",
        //      "Va Min","Va Max","Vb Min","Vb Max","Vc Min","Vc Max"};
        //    string[] BINDING_NAMES = new string[COLUMN_COUNT]
        //     {"EventChecked","EventName","EventType","Date","Time","Duration","Magnitude","Violation",
        //      "VaMin","VaMax","VbMin","VbMax","VcMin","VcMax"};
        //    string violation, vaMin, vaMax, vbMin, vbMax, vcMin, vcMax;

        //    int i;
        //    TTimeSeriesIndexRecord indexRecord;
        //    string eventMag;
        //    TTimeSeriesInputSettingsSSI inputSettings;
        //    int[] myColumns = new int[COLUMN_COUNT];
        //    GlobalString gs = new GlobalString();
        //    TFormatSettings myFormat = gs.InitTFormatSettings();
        //    Boolean doRestore;
        //    Boolean[] checkList, selectList;
        //    string myDate, myTime, myDuration;
        //    ObservableCollection<VSSIEvent> itemSource;
        //    GridViewColumn gvColumn;

        //    FChartTimeSeries.EventData.InitDateTimeFormat(TDateFormatType.ftInternational, ref myFormat);
        //    //if (listViewChart.ItemsSource. is (ObservableCollection<VSSIEvent>>))
        //    //{
        //    //}
        //    //else {
        //    //}
        //    itemSource = (ObservableCollection<VSSIEvent>)listViewChart.ItemsSource;

        //    if (FChartTimeSeries.EventData.IndexTable.Count == 0)
        //    {
        //        listViewChart.Items.Clear();
        //        labelStatus.Content = NO_EVENTS_MSG;
        //        return;  // no data
        //    }

        //    // preserve state if count is the same
        //    doRestore = (FChartTimeSeries.EventData.IndexTable.Count == listViewChart.Items.Count);
        //    if (doRestore)
        //    {
        //        checkList = new Boolean[listViewChart.Items.Count];
        //        selectList = new Boolean[listViewChart.Items.Count];
        //        for (i = 0; i < itemSource.Count; i++)
        //        {
        //            checkList[i] = itemSource[i].Checked;
        //            selectList[i] = itemSource[i].Checked;
        //        }
        //    }
        //    else
        //    {
        //        checkList = new Boolean[1];
        //        selectList = new Boolean[1];
        //    }

        //     // Restore previous column width if rows present

        //    if ((listViewChart.Items.Count == COLUMN_COUNT) &&
        //       (itemSource.Count > 0))
        //    {
        //        for (i = 0; i < COLUMN_COUNT; i++)   // restore previous columns widths
        //        {
        //            gvColumn = listViewGrid.Columns[i];
        //            myColumns[i] = Convert.ToInt32(gvColumn.Width);
        //        }
        //    }
        //    else
        //    {
        //        for (i = 0; i < COLUMN_COUNT; i++)   // set default column width
        //        {
        //            myColumns[i] = COLUMN_SIZE[i];
        //        }
        //    }

        //    CreateEventListViewColumns(COLUMN_COUNT, COLUMN_NAMES, BINDING_NAMES, myColumns); 


        //    // fill list view with contents of event index
        //    itemSource = new ObservableCollection<VSSIEvent>();
        //    listViewChart.ItemsSource = itemSource;

        //    for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
        //    {
        //        indexRecord = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
        //        myDate = gs.DateToStr(indexRecord.DateTime, myFormat);  // setting date format
        //        myTime = string.Format(TIME_WITH_MS, indexRecord.DateTime);
        //        gs.DurationToTimeString(out myDuration, indexRecord.Duration);
        //        if (indexRecord.EventType == TTimeSeriesEventType.evNone)    // Trigger event
        //        {
        //            eventMag = "$$$$$";    // Magnitude not meaningful
        //        }
        //        else
        //        {
        //            eventMag = string.Format(DATAFORMAT, indexRecord.Magnitude);
        //        }
        //        violation = ChartTimeSeries.CBEMAViolationString(indexRecord.Violation);
        //        vaMin = string.Format(DATAFORMAT, indexRecord.VAMin);
        //        vaMax = string.Format(DATAFORMAT, indexRecord.VAMax);
        //        vbMin = string.Format(DATAFORMAT, indexRecord.VBMin);
        //        vbMax = string.Format(DATAFORMAT, indexRecord.VBMax);
        //        vcMin = string.Format(DATAFORMAT, indexRecord.VCMin);
        //        vcMax = string.Format(DATAFORMAT, indexRecord.VCMax);
        //        itemSource.Add(new VSSIEvent(false, indexRecord.EventName,
        //            indexRecord.Description, myDate, myTime, myDuration, eventMag, violation,
        //            vaMin, vaMax, vbMin, vbMax, vcMin, vcMax));
        //    }

        //    // Update status label based on number of events found in index

        //    inputSettings = (TTimeSeriesInputSettingsSSI)FChartTimeSeries.EventData.InputSettings;
        //    if ((inputSettings.EventPhase == THMIEventPhase.epAll) &&
        //                (inputSettings.EventType == TTimeSeriesEventType.evAll))
        //    {
        //        labelStatus.Content = string.Format(EVENTS_FOUND_FMT,
        //            FChartTimeSeries.EventData.IndexTable.Count);
        //    }
        //    else
        //    {
        //        if ((inputSettings.EventPhase != THMIEventPhase.epAll) &&
        //            (inputSettings.EventType != TTimeSeriesEventType.evAll))
        //        {
        //            labelStatus.Content = string.Format(EVENTS_FOUND_PHASE_FMT,
        //               FChartTimeSeries.EventData.IndexTable.Count,
        //               MYTYPE[(int)inputSettings.EventType],
        //               MYPHASE[(int)inputSettings.EventPhase]);
        //        }
        //        else if (inputSettings.EventPhase == THMIEventPhase.epAll)
        //        {
        //            labelStatus.Content = string.Format(EVENTS_FOUND_TYPE_FMT,
        //               FChartTimeSeries.EventData.IndexTable.Count,
        //               MYTYPE[(int)inputSettings.EventType],
        //               MYPHASE[(int)inputSettings.EventPhase]);
        //        }
        //        else
        //        {
        //            labelStatus.Content = string.Format(EVENTS_FOUND_MIXED_FMT,
        //               FChartTimeSeries.EventData.IndexTable.Count,
        //               MYPHASE[(int)inputSettings.EventPhase]);
        //        }
        //    }

        //    // Restore selected item(s)
        //    if (doRestore && !aReset)
        //    {
        //        for (i = 0; i < itemSource.Count; i++)
        //        {
        //            itemSource[i].Checked = checkList[i];
        //            //  ListViewEventList.Items[i].Checked = checkList[i];
        //            //ListViewEventList.Items[i].Selected = selectList[i];
        //            //ListViewEventList.Items[i].Focused = selectList[i];
        //        }
        //    }

        //}
        public void FillEventListViewSSI(Boolean aReset)
        {
            const string NO_EVENTS_MSG = "Status: No events were found matching the present event selection criteria.";
            const string EVENTS_FOUND_FMT = "Status: {0:d} events found.";
            const string EVENTS_FOUND_PHASE_FMT = "Status: {0:d} events found in which %s on Phase %s occurred.";
            const string EVENTS_FOUND_TYPE_FMT = "Status: {0:d} events found in which %s occurred.";
            const string EVENTS_FOUND_MIXED_FMT = "Status: {0:d} events found in which a Sag, Swell, or Interrupt on Phase %s occurred.";
            const int COLUMN_COUNT = 13;
            const string DATAFORMAT = "{0,5:0.0}";
            string[] MYPHASE = new string[4] { "A", "B", "C", "Any" };
            string[] MYTYPE = new string[5] { "None", "a Sag", "a Swell", "an Interrupt", "Any" };
            int[] COLUMN_SIZE = new int[COLUMN_COUNT] { 60, 45, 70, 80, 85, 80, 75, 60, 60, 60, 60, 60, 60 };
            // Event, Type, Date, Start, Duration, Magnitude, Region, VaMin...
            string[] COLUMN_NAMES = new string[COLUMN_COUNT]
             {"Event","Type","Date","Time","Duration","Event Depth","ITIC Region",
              "Va Min","Va Max","Vb Min","Vb Max","Vc Min","Vc Max"};
            string[] BINDING_NAMES = new string[COLUMN_COUNT]
             {"EventName","EventType","Date","Time","Duration","Magnitude","Violation",
              "VaMin","VaMax","VbMin","VbMax","VcMin","VcMax"};
            string violation, vaMin, vaMax, vbMin, vbMax, vcMin, vcMax;

            int i;
            TTimeSeriesIndexRecord indexRecord;
            string eventMag;
            TTimeSeriesInputSettingsSSI inputSettings;
            //int[] myColumns = new int[COLUMN_COUNT];
            GlobalString gs = new GlobalString();
            TFormatSettings myFormat = gs.InitTFormatSettings();
            //Boolean doRestore;
            //Boolean[] checkList, selectList;
            string myDate, myTime, myDuration;
            //ObservableCollection<VSSIEvent> itemSource;
            //GridViewColumn gvColumn;

            if (FChartTimeSeries.EventData.IndexTable.Count == 0)
            {
                listViewChart.Items.Clear();
                labelStatus.Content = NO_EVENTS_MSG;
                return;  // no data
            }

            FChartTimeSeries.EventData.InitDateTimeFormat(TDateFormatType.ftInternational, ref myFormat);
            ObservableCollection<VSSIEvent> events = new ObservableCollection<VSSIEvent>();

            // preserve state if count is the same
            //doRestore = (FChartTimeSeries.EventData.IndexTable.Count == listViewChart.Items.Count);
            //if (doRestore)
            //{
            //    checkList = new Boolean[listViewChart.Items.Count];
            //    selectList = new Boolean[listViewChart.Items.Count];
            //    for (i = 0; i < itemSource.Count; i++)
            //    {
            //        checkList[i] = itemSource[i].Checked;
            //        selectList[i] = itemSource[i].Checked;
            //    }
            //}
            //else
            //{
            //    checkList = new Boolean[1];
            //    selectList = new Boolean[1];
            //}

            //// Restore previous column width if rows present

            //if ((listViewChart.Items.Count == COLUMN_COUNT) &&
            //   (itemSource.Count > 0))
            //{
            //    for (i = 0; i < COLUMN_COUNT; i++)   // restore previous columns widths
            //    {
            //        gvColumn = listViewGrid.Columns[i];
            //        myColumns[i] = Convert.ToInt32(gvColumn.Width);
            //    }
            //}
            //else
            //{
            //    for (i = 0; i < COLUMN_COUNT; i++)   // set default column width
            //    {
            //        myColumns[i] = COLUMN_SIZE[i];
            //    }
            //}

            CreateEventListViewColumns(COLUMN_COUNT, COLUMN_NAMES, BINDING_NAMES, COLUMN_SIZE);
            listViewChart.DataContext = events;

            // fill list view with contents of event index
 
            for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
            {
                indexRecord = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                myDate = gs.DateToStr(indexRecord.DateTime, myFormat);  // setting date format
                myTime = string.Format(TIME_WITH_MS, indexRecord.DateTime);
                gs.DurationToTimeString(out myDuration, indexRecord.Duration);
                if (indexRecord.EventType == TTimeSeriesEventType.evNone)    // Trigger event
                {
                    eventMag = "$$$$$";    // Magnitude not meaningful
                }
                else
                {
                    eventMag = string.Format(DATAFORMAT, indexRecord.Magnitude);
                }
                violation = ChartTimeSeries.CBEMAViolationString(indexRecord.Violation);
                vaMin = string.Format(DATAFORMAT, indexRecord.VAMin);
                vaMax = string.Format(DATAFORMAT, indexRecord.VAMax);
                vbMin = string.Format(DATAFORMAT, indexRecord.VBMin);
                vbMax = string.Format(DATAFORMAT, indexRecord.VBMax);
                vcMin = string.Format(DATAFORMAT, indexRecord.VCMin);
                vcMax = string.Format(DATAFORMAT, indexRecord.VCMax);

                events.Add(new VSSIEvent()
                {
                    EventName = indexRecord.EventName,
                    EventType = indexRecord.Description,
                    Date = myDate,
                    Time = myTime,
                    Duration = myDuration,
                    Magnitude = eventMag,
                    Violation = violation,
                    VaMin = vaMin,
                    VaMax = vaMax,
                    VbMin = vbMin,
                    VbMax = vbMax,
                    VcMin = vcMin,
                    VcMax = vcMax
                });
            }

            // Update status label based on number of events found in index

            inputSettings = (TTimeSeriesInputSettingsSSI)FChartTimeSeries.EventData.InputSettings;
            if ((inputSettings.EventPhase == THMIEventPhase.epAll) &&
                        (inputSettings.EventType == TTimeSeriesEventType.evAll))
            {
                labelStatus.Content = string.Format(EVENTS_FOUND_FMT,
                    FChartTimeSeries.EventData.IndexTable.Count);
            }
            else
            {
                if ((inputSettings.EventPhase != THMIEventPhase.epAll) &&
                    (inputSettings.EventType != TTimeSeriesEventType.evAll))
                {
                    labelStatus.Content = string.Format(EVENTS_FOUND_PHASE_FMT,
                       FChartTimeSeries.EventData.IndexTable.Count,
                       MYTYPE[(int)inputSettings.EventType],
                       MYPHASE[(int)inputSettings.EventPhase]);
                }
                else if (inputSettings.EventPhase == THMIEventPhase.epAll)
                {
                    labelStatus.Content = string.Format(EVENTS_FOUND_TYPE_FMT,
                       FChartTimeSeries.EventData.IndexTable.Count,
                       MYTYPE[(int)inputSettings.EventType],
                       MYPHASE[(int)inputSettings.EventPhase]);
                }
                else
                {
                    labelStatus.Content = string.Format(EVENTS_FOUND_MIXED_FMT,
                       FChartTimeSeries.EventData.IndexTable.Count,
                       MYPHASE[(int)inputSettings.EventPhase]);
                }
            }

            // Restore selected item(s)
            //if (doRestore && !aReset)
            //{
            //    for (i = 0; i < itemSource.Count; i++)
            //    {
            //        itemSource[i].Checked = checkList[i];
            //        //  ListViewEventList.Items[i].Checked = checkList[i];
            //        //ListViewEventList.Items[i].Selected = selectList[i];
            //        //ListViewEventList.Items[i].Focused = selectList[i];
            //    }
            //}

        }
        private void CreateEventListViewColumns(int COLUMN_COUNT, string[] COLUMN_NAMES, string[] BINDING_NAMES, int[] myColumns)
        {
            int i;
            GridViewColumn gvColumn;
            //Uri uri = new Uri("/WpfChartViewer;component/PageChart.xaml", UriKind.Relative);
            //ResourceDictionary dictionary = Application.LoadComponent(uri) as ResourceDictionary;


            listViewGrid.Columns.Clear(); // Erase existing columns            
//            listViewChart.Items.Clear();

            for (i = 0; i < COLUMN_COUNT; i++)
            {
                gvColumn = new GridViewColumn();
                gvColumn.Header = COLUMN_NAMES[i];
                gvColumn.Width = myColumns[i];
                //if (i == 0)
                //{
                //    //gvColumn.CellTemplate = (DataTemplate) dictionary ["FirstCell"];
                //}
                gvColumn.DisplayMemberBinding = new Binding(BINDING_NAMES[i]);
                listViewGrid.Columns.Add(gvColumn);
                // Columns[i].AutoSize = false;
            }
        }
/************************************************************************
* NAME:    ShowDeviceInformation
* FUNCTIONAL DESCRIPTION: Display information about the device
* INPUT PARAMETERS:       none
* OUTPUT PARAMETERS:      none
* RETURN:                 none
************************************************************************/
        public void ShowDeviceInformation()
        {
            const int COLUMN_COUNT = 3;
            string[] COLUMN_NAMES = new string[COLUMN_COUNT]
             {"Item","Property","Value"};
            string[] BINDING_NAMES = new string[COLUMN_COUNT] 
            { "Item", "Property", "Value" };
            int[] COLUMN_SIZE = new int[COLUMN_COUNT] {50,120,350};

            int i;
            TTimeSeriesSettingRecord settingRecord;
            GlobalString gs = new GlobalString();
            TFormatSettings myFormat = gs.InitTFormatSettings();
            string myDate, myTime;

            if (ChartTimeSeries == null) return;  // not active yet


            ObservableCollection<DeviceInfoGeneral> general = new ObservableCollection<DeviceInfoGeneral>();

            ChartTimeSeries.EventData.InitDateTimeFormat(TDateFormatType.ftInternational, ref myFormat);

            myDate = gs.DateToStr(ChartTimeSeries.EventData.ChartDateTime,myFormat);  // setting date format
            myTime = string.Format("{0:hh:mm:ss}",ChartTimeSeries.EventData.ChartDateTime);

            // fill list view with contents of device information

            CreateDeviceInfoGeneralColumns(COLUMN_COUNT, COLUMN_NAMES, BINDING_NAMES, COLUMN_SIZE);
            listViewDeviceInfoGeneral.DataContext = general;

            general.Add(new DeviceInfoGeneral()
            {
                Item = " 1",
                Property = "Chart Type",
                Value = ChartTimeSeries.EventData.ChartName
            });
            general.Add(new DeviceInfoGeneral()
            {
                Item = " 2",
                Property = "Date",
                Value = myDate
            });
            general.Add(new DeviceInfoGeneral()
            {
                Item = " 3",
                Property = "Time",
                Value = myTime
            });
            general.Add(new DeviceInfoGeneral()
            {
                Item = " 4",
                Property = "Time Source",
                Value = ChartTimeSeries.EventData.TimeSource
            });
            general.Add(new DeviceInfoGeneral()
            {
                Item = " 5",
                Property = "Device ID",
                Value = ChartTimeSeries.EventData.MeterID
            });
            general.Add(new DeviceInfoGeneral()
            {
                Item = " 6",
                Property = "Device FID",
                Value = ChartTimeSeries.EventData.MeterFID
            });
            general.Add(new DeviceInfoGeneral()
            {
                Item = " 7",
                Property = "Device TID",
                Value = ChartTimeSeries.EventData.MeterTID
            });

            for(i=0;  i<FChartTimeSeries.EventData.SettingTable.Count; i++)
            {
                settingRecord = ChartTimeSeries.EventData.SettingTable[i];
                general.Add(new DeviceInfoGeneral()
                {
                    Item = string.Format("{0:d2}",i+8),
                    Property = settingRecord.SettingName,
                    Value = string.Format("{0:f2}",settingRecord.SettingValue)
                });
            }
        }

        private void CreateDeviceInfoGeneralColumns(int COLUMN_COUNT, string[] COLUMN_NAMES, string[] BINDING_NAMES, int[] myColumns)
        {
            int i;
            GridViewColumn gvColumn;

 //           listViewDeviceInfoGeneral.Items.Clear();
            listViewDeviceInfoGeneralGrid.Columns.Clear(); // Erase existing columns            

            for (i = 0; i < COLUMN_COUNT; i++)
            {
                gvColumn = new GridViewColumn();
                gvColumn.Header = COLUMN_NAMES[i];
                gvColumn.Width = myColumns[i];
                gvColumn.DisplayMemberBinding = new Binding(BINDING_NAMES[i]);
                listViewDeviceInfoGeneralGrid.Columns.Add(gvColumn);
            }
        }
        /************************************************************************
        * NAME:    ShowChannelInformation
        * FUNCTIONAL DESCRIPTION: Display channel information about the device
        * INPUT PARAMETERS:       none
        * OUTPUT PARAMETERS:      none
        * RETURN:                 none
        ************************************************************************/
         public void ShowChannelInformation()
         {
            const int COLUMN_COUNT = 5;
            string[] COLUMN_NAMES = new string[COLUMN_COUNT] { "Name", "Function", "Units", "Scaling Rule", "Scaling Factor" };
            string[] BINDING_NAMES = new string[COLUMN_COUNT] { "Name", "Function", "Units", "ScalingRule", "ScalingFactor" };
            int[] COLUMN_SIZE = new int[COLUMN_COUNT] { 120, 100, 80, 110, 130 };

            int i;
            Boolean convertDecimal;
            TTimeSeriesLDPChannelRecord channelRecord;
            GlobalString gs = new GlobalString();
            string myScalingRule;

            if (ChartTimeSeries == null) return;  // not active yet

            convertDecimal = gs.ContainsText(string.Format("{0:f1}",1.0),",");  // test string for decimal delimiter

            ObservableCollection<DeviceInfoChannels> channels = new ObservableCollection<DeviceInfoChannels>();

            // fill list view with contents of device information

            CreateDeviceInfoChannelColumns(COLUMN_COUNT, COLUMN_NAMES, BINDING_NAMES, COLUMN_SIZE);
            listViewDeviceInfoChannels.DataContext = channels;

            for(i=0;  i<FChartTimeSeries.EventData.LDPChannelTable.Count; i++)
            {
                channelRecord = FChartTimeSeries.EventData.LDPChannelTable[i]; // get the event index record
                if(convertDecimal)
                {
                    myScalingRule = channelRecord.ChannelRules.Replace(".",",");
                }
                else
                {
                    myScalingRule = channelRecord.ChannelRules;
                }
                channels.Add(new DeviceInfoChannels()
                {
                    Name = channelRecord.ChannelName,
                    Function = channelRecord.ChannelFunction,
                    Units = channelRecord.ChannelUnits,
                    ScalingRule = myScalingRule,
                    ScalingFactor = string.Format("{0:f2}",channelRecord.ChannelScale)
                });
            }
        }

        private void CreateDeviceInfoChannelColumns(int COLUMN_COUNT, string[] COLUMN_NAMES, string[] BINDING_NAMES, int[] myColumns)
        {
            int i;
            GridViewColumn gvColumn;

//            listViewDeviceInfoChannels.Items.Clear();
            listViewDeviceInfoChannelsGrid.Columns.Clear(); // Erase existing columns            

            for (i = 0; i < COLUMN_COUNT; i++)
            {
                gvColumn = new GridViewColumn();
                gvColumn.Header = COLUMN_NAMES[i];
                gvColumn.Width = myColumns[i];
                gvColumn.DisplayMemberBinding = new Binding(BINDING_NAMES[i]);
                listViewDeviceInfoChannelsGrid.Columns.Add(gvColumn);
            }
        }
        /************************************************************************
        * NAME:    ShowEventInformation
        * FUNCTIONAL DESCRIPTION: Display information about the SER events
        * INPUT PARAMETERS:       none
        * OUTPUT PARAMETERS:      none
        * RETURN:                 none
        ************************************************************************/
        public void ShowEventInformation()
        {
            const int COLUMN_COUNT = 5;
            string[] COLUMN_NAMES = new string[COLUMN_COUNT] { "Event", "Date", "Time", "Description", "Code" };
            string[] BINDING_NAMES = new string[COLUMN_COUNT] { "Event", "Date", "Time", "Description", "Code" };
            int[] COLUMN_SIZE = new int[COLUMN_COUNT] { 60, 100, 120, 350, 60 };

            int i;
            TTimeSeriesLDPEventRecord eventRecord;
            string myDate, myTime;
            GlobalString gs = new GlobalString();
            TFormatSettings myFormat = gs.InitTFormatSettings();

            if (ChartTimeSeries == null) return;  // not active yet

            ObservableCollection<DeviceInfoEvents> events = new ObservableCollection<DeviceInfoEvents>();

            // fill list view with contents of device information

            CreateDeviceInfoEventColumns(COLUMN_COUNT, COLUMN_NAMES, BINDING_NAMES, COLUMN_SIZE);
            listViewDeviceInfoEvents.DataContext = events;

            for(i=0;  i<FChartTimeSeries.EventData.LDPEventTable.Count; i++)
            {
                eventRecord = FChartTimeSeries.EventData.LDPEventTable[i]; // get the event index record
                myDate = gs.DateToStr(eventRecord.DateTime,myFormat);  // setting date format
                myTime = string.Format(TIME_WITH_MS, eventRecord.DateTime);
                events.Add(new DeviceInfoEvents()
                {
                    Event = eventRecord.EventName,
                    Date = myDate,
                    Time = myTime,
                    Description = eventRecord.Description,
                    Code = eventRecord.EventCode
                });
            }
        }

        private void CreateDeviceInfoEventColumns(int COLUMN_COUNT, string[] COLUMN_NAMES, string[] BINDING_NAMES, int[] myColumns)
        {
            int i;
            GridViewColumn gvColumn;

 //           listViewDeviceInfoEvents.Items.Clear();
            listViewDeviceInfoEventGrid.Columns.Clear(); // Erase existing columns            

            for (i = 0; i < COLUMN_COUNT; i++)
            {
                gvColumn = new GridViewColumn();
                gvColumn.Header = COLUMN_NAMES[i];
                gvColumn.Width = myColumns[i];
                gvColumn.DisplayMemberBinding = new Binding(BINDING_NAMES[i]);
                listViewDeviceInfoEventGrid.Columns.Add(gvColumn);
            }
        }
        /************************************************************************
        * NAME:  ResetDeviceInformation
        * FUNCTIONAL DESCRIPTION: Reset device information dialog content
        * INPUT PARAMETERS: none
        * OUTPUT PARAMETERS: none
        * RETURN: none
        ************************************************************************/
        public void ResetDeviceInformation()
        {
            ShowDeviceInformation();
            if(ChartType == TTimeSeriesChartType.ctLDP) 
            {
                ShowChannelInformation();
                ShowEventInformation();
            }
            //FDeviceInfoDialog.ResetDeviceInfoTab();
        }

        /************************************************************************
        * NAME:     SetDateFormat
        * FUNCTIONAL DESCRIPTION:  Set the date format used for parsing dates
        * INPUT PARAMETERS: aDateFormat : string   MDY, YMD, DMY
        * OUTPUT PARAMETERS: none
        * RETURN:           none
        ************************************************************************/
        public void SetDateFormat(string aDateFormat)
        {
            GlobalString gs = new GlobalString();
            FDateFormat = gs.GetDateFormat(aDateFormat);
            comboBoxDate.Text = aDateFormat;
            if (FChartTimeSeries == null) return;  // not initialized yet            
            FChartTimeSeries.EventData.DateFormat = FDateFormat;
        }
/************************************************************************
* NAME:           GetChartInputControlSettingsSSI
* FUNCTIONAL DESCRIPTION:   getter for Chart input controls
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void GetChartInputControlSettingsSSI()
        {
            const int CBEMA_TAB_INDEX = 0;

            ChartInputSettingsSSI.BeginDateActive = (Boolean) checkBoxBegin.IsChecked;
            ChartInputSettingsSSI.EndDateActive = (Boolean) checkBoxEnd.IsChecked;
            ChartInputSettingsSSI.ChartCBEMA = (tabControlChart.SelectedIndex == CBEMA_TAB_INDEX);

            ChartInputSettingsSSI.BeginDateTime = (DateTime) datePickerBegin.SelectedDate + timePickerBegin.SelectedTime;
            ChartInputSettingsSSI.EndDateTime = (DateTime) datePickerEnd.SelectedDate + timePickerEnd.SelectedTime;

            // phase and event types selected by user
            ChartInputSettingsSSI.EventPhase = (THMIEventPhase)comboBoxPhase.SelectedIndex;
            ChartInputSettingsSSI.EventType = (TTimeSeriesEventType)comboBoxEventType.SelectedIndex;
        }
/************************************************************************
* NAME:           SetChartInputControlSettingsSSI
* FUNCTIONAL DESCRIPTION:   setter for Chart input controls
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void SetChartInputControlSettingsSSI()
        {
            const int CBEMA_TAB_INDEX = 0;
            const int SSI_TAB_INDEX = 1;

            checkBoxBegin.IsChecked = ChartInputSettingsSSI.BeginDateActive;
            checkBoxEnd.IsChecked = ChartInputSettingsSSI.EndDateActive;
            if(ChartInputSettingsSSI.ChartCBEMA)
            {
                tabControlChart.TabIndex = CBEMA_TAB_INDEX;   // CBEMA
            }
            else
            {
                tabControlChart.TabIndex = SSI_TAB_INDEX;   // SSI
            }

            comboBoxPhase.SelectedIndex = (int) ChartInputSettingsSSI.EventPhase;
            comboBoxEventType.SelectedIndex = (int) ChartInputSettingsSSI.EventType;
        }
/************************************************************************
* NAME:           GetChartInputControlSettings
* FUNCTIONAL DESCRIPTION:   getter for Chart input controls
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void GetChartInputControlSettingsLDP()
        {
            ChartInputSettingsLDP.BeginDateActive = (Boolean) checkBoxBegin.IsChecked;
            ChartInputSettingsLDP.EndDateActive = (Boolean) checkBoxEnd.IsChecked;

            ChartInputSettingsLDP.BeginDateTime = (DateTime) datePickerBegin.SelectedDate +  timePickerBegin.SelectedTime;
            ChartInputSettingsLDP.EndDateTime = (DateTime) datePickerEnd.SelectedDate + timePickerEnd.SelectedTime;
        }
/************************************************************************
* NAME:           SetChartInputControlSettings
* FUNCTIONAL DESCRIPTION:   setter for Chart input controls
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void SetChartInputControlSettingsLDP()
        {
            checkBoxBegin.IsChecked = ChartInputSettingsLDP.BeginDateActive;
            checkBoxEnd.IsChecked = ChartInputSettingsLDP.EndDateActive;
        }
/******************************************************************************
* MODULE NAME ...  ChartHMIDisplayMouseWheel
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Event handler for mouse wheel events
*
* INPUT PARAMETERS:
*                    Sender : TObject
*                    Shift : TShiftState   set containing list of depressed control keys
*                    WheelDelta : Integer  change in mouse wheel position
*                    MousePos : TPoint     screen coord. of mouse
*
* OUTPUT PARAMETERS:
*                    Handled: Boolean   true/false indicating whether event was handled
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        public void ChartHMIDisplayMouseWheel(object sender, MouseWheelEventArgs e)
        {
            double zoom2D;
            int myWheelDelta;
            Boolean isShift, isCtrl, isAlt;

            isShift = (Keyboard.IsKeyDown(Key.LeftShift) || Keyboard.IsKeyDown(Key.RightShift));
            isCtrl = (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl));
            isAlt = (Keyboard.IsKeyDown(Key.LeftAlt) || Keyboard.IsKeyDown(Key.RightAlt));
            

            //if(WindowFromPoint(MousePos) == FChartTimeSeries.ChartDisplay.Handle) 
            //{
//                Handled = true;  // event was handled here - don't handle it for other controls
                myWheelDelta = e.Delta / 100;
                // see if 3D checkbox checked or raw mouse wheel event (no control keys pressed)
                if(!FChartTimeSeries.ChartDisplay.Aspect.View3D ||
                (!isShift && !isCtrl && !isAlt))   // 2D chart wheel event
                {
                    zoom2D = 100.0 + myWheelDelta;
                    FChartTimeSeries.ChartDisplay.Zoom.ZoomPercent(zoom2D);
                }
                else
                {
                    if (isShift && isCtrl)   // zoom, X & Y rotate simultaneously
                    {
                        FChartTimeSeries.ChartRotate = true;
                        FChartTimeSeries.ChartTranslate = false;
                        FChartTimeSeries.ChartZoom = true;
                        FChartTimeSeries.ChartThickness = false;
                        FChartTimeSeries.ChartDisplay.Aspect.Orthogonal = false;
                        FChartTimeSeries.ChartDisplay.Aspect.Elevation =
                                ((FChartTimeSeries.ChartDisplay.Aspect.Elevation + myWheelDelta) % 90) + 270;
                        FChartTimeSeries.ChartDisplay.Aspect.Rotation =
                                (FChartTimeSeries.ChartDisplay.Aspect.Rotation + myWheelDelta) % 360;
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom =
                                FChartTimeSeries.ChartDisplay.Aspect.Zoom + myWheelDelta;
                    }
                    else if(isShift) 
                    {
                        FChartTimeSeries.ChartRotate = false;
                        FChartTimeSeries.ChartTranslate = false;
                        FChartTimeSeries.ChartZoom = true;
                        FChartTimeSeries.ChartThickness = false;
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom =
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom + myWheelDelta;
                    }
                    else if (isCtrl) 
                    {
                        FChartTimeSeries.ChartRotate = true;
                        FChartTimeSeries.ChartTranslate = false;
                        FChartTimeSeries.ChartZoom = false;
                        FChartTimeSeries.ChartThickness = false;
                        FChartTimeSeries.ChartDisplay.Aspect.Orthogonal = false;
                        FChartTimeSeries.ChartDisplay.Aspect.Rotation =
                                (FChartTimeSeries.ChartDisplay.Aspect.Rotation + myWheelDelta) % 360;
                    }
                    else if (isAlt) 
                    {
                        FChartTimeSeries.ChartRotate = true;
                        FChartTimeSeries.ChartTranslate = false;
                        FChartTimeSeries.ChartZoom = false;
                        FChartTimeSeries.ChartThickness = false;
                        FChartTimeSeries.ChartDisplay.Aspect.Orthogonal = false;
                        FChartTimeSeries.ChartDisplay.Aspect.Elevation =
                                ((FChartTimeSeries.ChartDisplay.Aspect.Elevation + myWheelDelta) % 90) + 270;
                    }
                }
            //}
        }
/************************************************************************
* NAME:   DisplayMouseQueryFeedback
* FUNCTIONAL DESCRIPTION:  Display feedback for chart object at mouse cursor location
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void DisplayMouseQueryFeedback(MouseEventArgs e)
        {
            Point P;
            Boolean found, isCBEMA;
            double myX, myY;
            int i, j, ecount, count, nActiveRecords, match, index, lastRecord;
            TTimeSeriesIndexRecord indexRecord;
            TTimeSeriesRecord dataRecord;
            string myLabel, myDT;
            GlobalString gs = new GlobalString();
            TFormatSettings myFormat = gs.InitTFormatSettings();
            TTimeSeriesInputSettingsSSI mySSI;

            if (FChartTimeSeries.EventData.InputSettings.ChartQuery)
            {
                FChartTimeSeries.EventData.InitDateTimeFormat(TDateFormatType.ftInternational, ref myFormat);

                isCBEMA = false;
                switch( FChartType)
                {
                    case TTimeSeriesChartType.ctUndefined:
                    case TTimeSeriesChartType.ctLDP:
                    break;

                    case TTimeSeriesChartType.ctSSI:
                    mySSI = (TTimeSeriesInputSettingsSSI)FChartTimeSeries.EventData.InputSettings;
                    if (mySSI.ChartCBEMA) 
                        {
                            isCBEMA = true;
                        }
                    break;

                    default:
                        System.Diagnostics.Debug.Assert(false,"Invalid chart type");
                    break;
                }

                /*NOTE: A semi-colon separator is used (instead of a comma) when feeding back
                a list of data values in order to avoid confusion when the native language
                uses a comma instead of a decimal point*/

                if(FChartTimeSeries.ChartDisplay.Header.Text.Length <= 1) 
                {
                    //FChartTimeSeries.ChartDisplay.Title.Text.Add(' ');   // add title for feedback
                }
                // P=FChartTimeSeries.ChartDisplay.GetCursorPos();
                P = e.MouseDevice.GetPosition(this);
                //               P = FChartTimeSeries.ChartDisplay.PointFromScreen(e.MouseDevice.GetPosition(this));
                               //P = FChartTimeSeries.ChartDisplay.PointToScreen(e.MouseDevice.GetPosition(this));

                ecount = GetActiveEventCount();   // number of active events

                // Compare mouse location to each point in each series

                for(i=0; i< FChartTimeSeries.ChartDisplay.Series.Count; i++)
                {
                    match = FChartTimeSeries.ChartDisplay.Series[i].Clicked(P.X,P.Y);
                    if (match != -1)      // match is matching record in series
                    {
                        FChartTimeSeries.CursorTool.Series = FChartTimeSeries.ChartDisplay.Series[i];
                        if((!isCBEMA) &&
                                !(FChartTimeSeries.ChartDisplay.Series[i] is Steema.TeeChart.WPF.Styles.Bar)) 
                        {
                            match = FChartTimeSeries.CursorTool.SnapToPoint();   // not a point series, must identify data point
                        }
                        myX = FChartTimeSeries.ChartDisplay.Series[i].XValues[match];
                        myY = FChartTimeSeries.ChartDisplay.Series[i].YValues[match];

                        if((FChartType == TTimeSeriesChartType.ctLDP) && 
                                (ChartTimeSeries.ChartDisplay.Series[i] is Steema.TeeChart.WPF.Styles.Points)) 
                        {
                            index = FChartTimeSeries.GetEventRecordFromRecord(myX);
                            if(index >= 0) 
                            {
                                indexRecord = FChartTimeSeries.EventData.IndexTable[index];
                                myDT = gs.DateToStr(indexRecord.DateTime,myFormat) + " "  + string.Format(TIME_WITH_MS,indexRecord.DateTime);
                                myLabel = string.Format("Event {0:} ({1:}) = {2:}",
                                        indexRecord.EventName,myDT,indexRecord.Description);
                            }
                        }
                        else if(!isCBEMA && FChartTimeSeries.EventData.InputSettings.ChartAxisDateTime)     // display date/time X axis labels
                        {      // Indep. Var. is Date/Time
                            // Determine which record this data point refers to
                            found = false;
                            if(ChartType == TTimeSeriesChartType.ctSSI)  // update match based on event records
                            {
                                nActiveRecords = 0;
                                lastRecord = 0;   // upper limit of last active parent record
                                for (j = 0; j<FChartTimeSeries.EventData.IndexTable.Count; j++)
                                {
                                    indexRecord = FChartTimeSeries.EventData.IndexTable[j];  // get event summary record
                                    if((ecount > 0) && !indexRecord.Checked)  continue;  // skip inactive record
                                    if(indexRecord.LastRecord <= lastRecord)  continue;  // skip sub-event
                                    lastRecord = indexRecord.LastRecord; // remember last record of active parent event
                                    count = indexRecord.LastRecord - indexRecord.FirstRecord + 1;  // number of records in this event
                                    if(!found && (match > nActiveRecords) &&
                                            (match <= (nActiveRecords + count)))   // record is in this interval
                                    {
                                        found = true;      // record is in this event interval
                                        match = indexRecord.FirstRecord + match - nActiveRecords -1;
                                        break;
                                    }
                                    nActiveRecords += count;  // number of active records
                                }
                            }
                            else
                            {
                                found = true;  // LDP just use match already found
                            }

                            if(found) 
                            {
                                dataRecord = FChartTimeSeries.EventData.RawDataTable[match];
                                if(FChartType == TTimeSeriesChartType.ctSSI) 
                                {
                                    myDT = gs.DateToStr(dataRecord.DateTime,myFormat) + ' ' + string.Format(TIME_WITH_MS,dataRecord.DateTime);
                                }
                                else
                                {
                                    myDT = gs.DateToStr(dataRecord.DateTime,myFormat) + ' ' + string.Format(TIME_WITH_SEC,dataRecord.DateTime);
                                }
                                myLabel = string.Format("{0:}=({1:}; {2:f2})",FChartTimeSeries.ChartDisplay.Series[i].Title,myDT,myY);
                            }
                        }
                        else
                        {        // Indep. var. is record number or duration (if CBEMA)
                            if(ChartType == TTimeSeriesChartType.ctLDP)      // display LDP channel info
                            {
                                myLabel = string.Format("{0:}=({1:f0}; {2:f2})",FChartTimeSeries.ChartDisplay.Series[i].Title,myX,myY);
                            }
                            else
                            {
                                if(isCBEMA)      // display CBEMA event marker info
                                {
                                    if(i == 0)     // first series is the event marker
                                    {
                                        //myLabel = string.Format("{0:} {1:}=(Duration={2:f3}; Magnitude={3:f1})",
                                        //FChartTimeSeries.ChartDisplay.Series[i].Title,
                                        //    FChartTimeSeries.ChartDisplay.Series[i].Marks[match].Text[0],myX,myY);
                                    }
                                    else           // display sag/swell limit info
                                    {
                                        myLabel = string.Format("{0:} =(Duration={1:f3}; Magnitude={2:f1})",
                                            FChartTimeSeries.ChartDisplay.Series[i].Title,myX,myY);
                                    }
                                }
                                else             // display SSI channel series info
                                {
                                    myLabel = string.Format("{0:}=({1:f0}; {2:f2})",   // Rec #, Magnitude
                                        FChartTimeSeries.ChartDisplay.Series[i].Title,myX,myY);
                                }
                            }
                        }

                        // FChartTimeSeries.ChartDisplay.Header.Text[FChartTimeSeries.ChartDisplay.Header.Text.Length-1] = myLabel;
                    }
                }
            }
        }
/************************************************************************
* NAME:     UpdateChartHandler
* FUNCTIONAL DESCRIPTION:  Update chart from content of report window
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN:           none
************************************************************************/
        private void UpdateChartHandler()
        {
            Boolean success;

            success = DisplayChart(richTextNotepad.ToString().Trim());
            if(success) 
            {
                //Caption = Format('Chart %d - from Notepad',[FChartInstance]);
            }
            else
            {
                MessageBox.Show("Chart generation failed.");
            }
        }
/************************************************************************
* NAME:     UpdateReportHandler
* FUNCTIONAL DESCRIPTION:  Update report from content of chart window
* INPUT PARAMETERS: none
* OUTPUT PARAMETERS: none
* RETURN:           none
************************************************************************/
        private void UpdateReportHandler()
        {
            string buffer;
            Boolean success;

            success = AppendReport(out buffer);
            if(success) 
            {
                richTextNotepad.Clear();
                richTextNotepad.AppendText(buffer);
            }
            else
            {
                MessageBox.Show("Report generation failed.");
            }
        }
        /************************************************************************
        * NAME:     WhichEvent
        * FUNCTIONAL DESCRIPTION:  Get index of event matching name
        * INPUT PARAMETERS: eventName : string event name
        * OUTPUT PARAMETERS: none
        * RETURN:           index : integer index of matching event otherwise (-1)
        ************************************************************************/
        private int WhichEvent(string eventName)
        {
            TTimeSeriesIndexRecord indexRecord;
            int i;

            for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
            {
                indexRecord = FChartTimeSeries.EventData.IndexTable[i];  // get event summary record
                if (eventName == indexRecord.EventName) return (i);
            }
            return (-1);
        }

   }
}