﻿using System;
using System.ComponentModel;

namespace WpfChartViewer
{
    public class LDPEvent : INotifyPropertyChanged
    {
        #region [ Constructor ]

        /// <summary>
        /// Initializes a new instance of the <see cref="LDPEvent"/> class.
        /// </summary>
        /// <param name="eventChecked">The event state.</param>
        /// <param name="eventName">The event name.</param>
        /// <param name="date">The date.</param>
        /// <param name="time">The time.</param>
        public LDPEvent(Boolean eventChecked, String eventName, String date, String time, String description)
        {
            _eventChecked = eventChecked;
            _eventName = eventName;
            _date = date;
            _time = time;
            _description = description;
        }

        #endregion

        #region [ Fields ]

        private Boolean _eventChecked;
        private String _eventName;
        private String _date;
        private String _time;
        private String _description;


        #endregion

        #region [ Properties ]

        /// <summary>
        /// Gets or sets the event checked state.
        /// </summary>
        /// <value>The check state.</value>
        public Boolean Checked
        {
            get { return _eventChecked; }
            set
            {
                if (_eventChecked == value) return;
                _eventChecked = value;
                OnPropertyChanged("Checked");
            }
        }

        /// <summary>
        /// Gets or sets the first name.
        /// </summary>
        /// <value>The first name.</value>
        public String EventName
        {
            get { return _eventName; }
            set
            {
                if (_eventName == value) return;
                _eventName = value;
                OnPropertyChanged("EventName");
            }

        }


        /// <summary>
        /// Gets or sets the date.
        /// </summary>
        /// <value>The date.</value>
        public String Date
        {
            get { return _date; }
            set
            {
                if (_date == value) return;
                _date = value;
                OnPropertyChanged("Date");
            }
        }

        /// <summary>
        /// Gets or sets the time.
        /// </summary>
        /// <value>The time.</value>
        public String Time
        {
            get { return _time; }
            set
            {
                if (_time == value) return;
                _time = value;
                OnPropertyChanged("Time");
            }
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        /// <value>The description.</value>
        public String Description
        {
            get { return _description; }
            set
            {
                if (_description == value) return;
                _description = value;
                OnPropertyChanged("Description");
            }
        }


        #endregion

        #region [ Events ]

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region [ Methods ]

        /// <summary>
        /// Called when [property changed].
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        protected void OnPropertyChanged(String propertyName)
        {
            if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}

