﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml;
using System.Collections.ObjectModel;
using SELChartClasses;
using SELGlobalStringMethods;
using Microsoft.Win32;

namespace WpfChartViewer
{
    /// <summary>
    /// Interaction logic for PageChart.xaml
    /// </summary>
    public partial class PageChart : Page
    {
        public PageChart()
        {
            InitializeComponent();
            try
            {
 //               FillItemCollection();
                FChartTimeSeries = new TTimeSeriesChartLDP();
                FChartType = TTimeSeriesChartType.ctInvalid;    // nothing charted yet
                //FDeviceInfoDialog = TFormDeviceInfo.Create(null);
                //FDeviceInfoDialog.Parent =  self;
                //FDeviceInfoDialog.Visible = false;
                FDateFormat = "MM/dd/yyyy";

//                self.HelpKeyword = "Chart";
                datePickerBegin.DisplayDate = DateTime.Now;
                datePickerBegin.SelectedDate = DateTime.Now;
                datePickerEnd.DisplayDate = DateTime.Now;
                datePickerEnd.SelectedDate = DateTime.Now;
                scrollBarTransparency.IsEnabled = false;
                ChartTool1.Active = false;
                ChartTool1.Snap = true;
                ChartTool2.Active = false;
                ChartTool2.Snap = true;
                ChartTool1.Change += new Steema.TeeChart.WPF.Tools.CursorChangeEventHandler(ChartCursor_Change);
                ChartTool2.Change += new Steema.TeeChart.WPF.Tools.CursorChangeEventHandler(ChartCursor_Change);
                FChartInstance = 0;
                FMenuChart = false;

                // Initialization

                this.ChartTimeSeries.ChartDisplay = this.ChartHMIDisplay;
                this.ChartTimeSeries.CursorTool = this.ChartTool1;
                this.SetChartType(TTimeSeriesChartType.ctUndefined);
                this.ResetChartWindow();
                
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.Fail(ex.Message);
            }
        }

        void mySelectionChanged(object sender,
                               SelectionChangedEventArgs e)
        {
            TTimeSeriesIndexRecord indexRecord;
            int i,item;
            LDPEvent myLDPEvent;
            VSSIEvent myVSSIEvent;
            string eventName;

            try
            {
                for(i=0; i<e.AddedItems.Count; i++)
                {
                    if((e.AddedItems[i]) is VSSIEvent) 
                    {
                        myVSSIEvent = (VSSIEvent) e.AddedItems[i];
                        eventName = myVSSIEvent.EventName;
                    }
                    else if((e.AddedItems[i]) is LDPEvent) 
                    {
                        myLDPEvent = (LDPEvent) e.AddedItems[i];
                        eventName = myLDPEvent.Event;
                    }
                    else
                    {
                        System.Diagnostics.Debug.Assert(false, "Invalid event type");
                        eventName = "";
                    }
                    item = WhichEvent(eventName);
                    //if(!Item.Checked)      // only change state if item is not checked
                    //{
                    indexRecord = FChartTimeSeries.EventData.IndexTable[item];  // get event summary record
                    indexRecord.Checked = true;
                    FChartTimeSeries.EventData.IndexTable[item] = indexRecord;
                    //}
                    //if(Item.Selected)    // only update chart if item is selected
                    //{
                    //    UpdateChart(true);
                    //}
                }
                for (i = 0; i < e.RemovedItems.Count; i++)
                {
                    if ((e.RemovedItems[i]) is VSSIEvent)
                    {
                        myVSSIEvent = (VSSIEvent)e.RemovedItems[i];
                        eventName = myVSSIEvent.EventName;
                    }
                    else if ((e.RemovedItems[i]) is LDPEvent)
                    {
                        myLDPEvent = (LDPEvent)e.RemovedItems[i];
                        eventName = myLDPEvent.Event;
                    }
                    else
                    {
                        System.Diagnostics.Debug.Assert(false, "Invalid event type");
                        eventName = "";
                    }
                    item = WhichEvent(eventName);
                    indexRecord = FChartTimeSeries.EventData.IndexTable[item];  // get event summary record
                    indexRecord.Checked = false;
                    FChartTimeSeries.EventData.IndexTable[item] = indexRecord;
                }

            }
            catch
            {
            }
            UpdateChart(true);
        }
       
        private void Page_Loaded(object sender, RoutedEventArgs e)
        {

        }

        private void checkBoxBegin_Checked(object sender, RoutedEventArgs e)
        {
            datePickerBegin.IsEnabled = true;
            timePickerBegin.IsEnabled = true;
        }

        private void checkBoxBegin_Unchecked(object sender, RoutedEventArgs e)
        {
            datePickerBegin.IsEnabled = false;
            timePickerBegin.IsEnabled = false;
        }

        private void checkBoxEnd_Checked(object sender, RoutedEventArgs e)
        {
            datePickerEnd.IsEnabled = true;
            timePickerEnd.IsEnabled = true;
        }

        private void checkBoxEnd_Unchecked(object sender, RoutedEventArgs e)
        {
            datePickerEnd.IsEnabled = false;
            timePickerEnd.IsEnabled = false;
        }


        private void buttonBrowse_Click(object sender, RoutedEventArgs e)
        {
            Boolean found;
            GlobalString gs = new GlobalString();

            OpenFileDialog openDialog = new OpenFileDialog();
            try
            {
                openDialog.InitialDirectory = gs.GetSpecialFolder(CSIDL_PERSONAL);
                openDialog.DefaultExt = ".csv";
                openDialog.Filter =
                                    "Chart File (*.csv)|*.csv|" +
                                    "SSI Report File (*.ssi)|*.ssi|" +
                                    "LDP Report File (*.ldp)|*.ldp|" +
                                    "LDP/SSI Report File (*.txt)|*.txt|" +
                                    "All Files (*.*)|*.*";
                openDialog.FilterIndex = 1;
                openDialog.Title = "Import Chart Data";

                openDialog.CheckFileExists = true;
                if(openDialog.ShowDialog() == true)
                {
                    found = DisplayChartFile(openDialog.FileName);
                    if(found) 
                    {
                        //Caption = string.Format('Chart %d - %s',[FChartInstance,
                        //                  ExtractFileName(openDialog.FileName)]);
                    }
                    else
                    {
                        ClearChartWindow();
                        MessageBox.Show("File does not contain valid data.");
                    }

                }
            }
            catch
            {
            }
        }

/************************************************************************
* NAME:    TabSetChartChange
* FUNCTIONAL DESCRIPTION: select type of chart to display for SSI
* INPUT PARAMETERS: Sender : TObject
*                   NewTab : Integer   selected tab
* OUTPUT PARAMETERS:
*                   AllowChange : Boolean
* RETURN: none
************************************************************************/
//procedure TfrmChartTimeSeries.TabSetChartChange(Sender: TObject; NewTab: Integer;
//  var AllowChange: Boolean);
//}
        private void tabControlChart_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            const int CBEMA_TAB_INDEX = 0;

            Boolean reset,doCBEMA;
            //  TTimeSeriesInputSettingsSSI mySSI;

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            FChartTimeSeries.SaveChartActiveChannels();
            // read input controls
            if (ChartType == TTimeSeriesChartType.ctSSI)
            {
                //        mySSI = (TTimeSeriesInputSettingsSSI) FChartTimeSeries.EventData.InputSettings;
                (FChartTimeSeries.EventData.InputSettings as TTimeSeriesInputSettingsSSI).GetChartInputControlSettings();
                doCBEMA = (tabControlChart.SelectedIndex == CBEMA_TAB_INDEX);
                reset = CBEMAChartVisible(doCBEMA);
                //if (DoCBEMA)
                //{
                //    ChartHMICBEMA.SetFocus;
                //}
                //else
                //{
                //    ChartHMIDisplay.SetFocus;
                //}
                UpdateChart(reset);
            }
        }

        private void buttonPointer_Click(object sender, RoutedEventArgs e)
        {
            const string TOOLBAR_MSG = "   Drag left button (down/right) to Zoom, Right button to Translate";

            FChartTimeSeries.ChartDisplay.Aspect.HorizOffset = 0;
            FChartTimeSeries.ChartDisplay.Aspect.VertOffset = 0;
            FChartTimeSeries.ChartDisplay.Aspect.Zoom = 100;

            labelHelp.Content = TOOLBAR_MSG;
            FChartTimeSeries.ChartRotate = false;
            FChartTimeSeries.ChartTranslate = false;
            FChartTimeSeries.ChartZoom = false;
            FChartTimeSeries.ChartThickness = false;
            FChartTimeSeries.ChartActive = false;
            FChartTimeSeries.ChartDisplay.Panning.Allow = Steema.TeeChart.WPF.ScrollModes.Both;  // re-enable panning
            FChartTimeSeries.ChartDisplay.Zoom.Allow = true;
        }

        private void buttonRotate_Click(object sender, RoutedEventArgs e)
        {
            const string ROTATE_MSG = "   Drag Chart to Rotate";

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            labelHelp.Content = ROTATE_MSG;
            FChartTimeSeries.ChartRotate = true;
            FChartTimeSeries.ChartTranslate = false;
            FChartTimeSeries.ChartZoom = false;
            FChartTimeSeries.ChartThickness = false;
            FChartTimeSeries.ChartDisplay.Zoom.Allow = false;
            if(!FChartTimeSeries.EventData.InputSettings.Chart3D) 
            {
                button3D_Click(sender,e);
            }
        }

        private void buttonTranslate_Click(object sender, RoutedEventArgs e)
        {
            const string TRANSLATE_MSG = "   Drag Chart to Translate";

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            labelHelp.Content = TRANSLATE_MSG;
            FChartTimeSeries.ChartRotate = false;
            FChartTimeSeries.ChartTranslate = true;
            FChartTimeSeries.ChartZoom = false;
            FChartTimeSeries.ChartThickness = false;
            FChartTimeSeries.ChartDisplay.Panning.Allow = Steema.TeeChart.WPF.ScrollModes.Both;  // disable panning
            FChartTimeSeries.ChartDisplay.Zoom.Allow = false;
            if(!FChartTimeSeries.EventData.InputSettings.Chart3D) 
            {
                button3D_Click(sender,e);
            }
        }

        private void buttonZoom_Click(object sender, RoutedEventArgs e)
        {
            const string ZOOM_MSG =  "   Drag Chart to Zoom";

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            labelHelp.Content = ZOOM_MSG;
            FChartTimeSeries.ChartRotate = false;
            FChartTimeSeries.ChartTranslate = false;
            FChartTimeSeries.ChartZoom = true;
            FChartTimeSeries.ChartThickness = false;
            FChartTimeSeries.ChartDisplay.Zoom.Allow = false;
            if(!FChartTimeSeries.EventData.InputSettings.Chart3D) 
            {
                button3D_Click(sender,e);
            }
        }

        private void buttonThick_Click(object sender, RoutedEventArgs e)
        {
            const string DEPTH_MSG = "   Drag Chart to change Depth";

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            labelHelp.Content = DEPTH_MSG;
            FChartTimeSeries.ChartRotate = false;
            FChartTimeSeries.ChartTranslate = false;
            FChartTimeSeries.ChartZoom = false;
            FChartTimeSeries.ChartThickness = true;
            FChartTimeSeries.ChartDisplay.Zoom.Allow = false;
            if(!FChartTimeSeries.EventData.InputSettings.Chart3D) 
            {
                button3D_Click(sender,e);
            }

        }

        private void button3D_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            FChartTimeSeries.EventData.InputSettings.Chart3D = !FChartTimeSeries.EventData.InputSettings.Chart3D;
            FChartTimeSeries.ChartDisplay.Aspect.View3D = FChartTimeSeries.EventData.InputSettings.Chart3D;
            SetChartLineStyles();
        }

        private void buttonCursor_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            FChartTimeSeries.EventData.InputSettings.ChartQuery = !FChartTimeSeries.EventData.InputSettings.ChartQuery;
            FChartTimeSeries.CursorTool.Active = FChartTimeSeries.EventData.InputSettings.ChartQuery;
            FChartTimeSeries.CursorTool.Snap = FChartTimeSeries.EventData.InputSettings.ChartQuery;
            FChartTimeSeries.CursorTool.FollowMouse = true;
            FChartTimeSeries.CursorTool.Style = Steema.TeeChart.WPF.Tools.CursorToolStyles.Both;

            if(FChartTimeSeries.EventData.InputSettings.ChartQuery) 
            {
                //FChartTimeSeries.ChartDisplay.Header.Text = " ";   // add title for feedback
            }
            else
            {
                FChartTimeSeries.InitChartTitles();      // reset titles to original
            }
        }

        private void buttonClock_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            FChartTimeSeries.EventData.InputSettings.ChartAxisDateTime = !FChartTimeSeries.EventData.InputSettings.ChartAxisDateTime;
            SetDateTimeLabels();
        }

        private void buttonLabel_Click(object sender, RoutedEventArgs e)
        {

            Steema.TeeChart.WPF.Styles.Bar BarSeries;
            Steema.TeeChart.WPF.Styles.Points PointSeries;
            TTimeSeriesInputSettingsSSI mySSI;

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");
            if(FChartTimeSeries.EventData.RawDataTable.Count < 1)   // no data loaded into chart
            {
                return;
            }

            FChartTimeSeries.EventData.InputSettings.ChartLabels = !ChartTimeSeries.EventData.InputSettings.ChartLabels;

            if(FChartType == TTimeSeriesChartType.ctSSI)   // SSI has a bar series with event indicators
            {
                mySSI = (TTimeSeriesInputSettingsSSI) FChartTimeSeries.EventData.InputSettings;
                if(mySSI.ChartCBEMA) 
                {
                    System.Diagnostics.Debug.Assert((ChartHMICBEMA.Series[0] is Steema.TeeChart.WPF.Styles.Points),"Not a Point Series");

                    PointSeries = (Steema.TeeChart.WPF.Styles.Points) ChartHMICBEMA.Series[0];
                    PointSeries.Marks.Visible = FChartTimeSeries.EventData.InputSettings.ChartLabels;
                }
                else
                {
                    if(ChartHMIDisplay.Series[ChartHMIDisplay.Series.Count-1] is Steema.TeeChart.WPF.Styles.Bar) 
                    {
                        BarSeries = (Steema.TeeChart.WPF.Styles.Bar ) ChartHMIDisplay.Series[ChartHMIDisplay.Series.Count-1];
                        BarSeries.Marks.Visible = FChartTimeSeries.EventData.InputSettings.ChartLabels;
                    }
                }
            }
            if((FChartType == TTimeSeriesChartType.ctLDP) &&
                (ChartHMIDisplay.Series[ChartHMIDisplay.Series.Count-1] is Steema.TeeChart.WPF.Styles.Points)) 
            {
                PointSeries = (Steema.TeeChart.WPF.Styles.Points) ChartHMIDisplay.Series[ChartHMIDisplay.Series.Count-1];
                PointSeries.Marks.Visible = FChartTimeSeries.EventData.InputSettings.ChartLabels;
            }
        }

        private void buttonInfo_Click(object sender, RoutedEventArgs e)
        {
            //if(FDeviceInfoDialog.TabControlDeviceInfo.Tabs.Count > 0) 
            //{
            //    if(!FDeviceInfoDialog.Visible) 
            //    {
            //        ResetDeviceInformation;
            //        FDeviceInfoDialog.Show;
            //        }
            //    else
            //    {
            //        FDeviceInfoDialog.Hide;
            //    }
            //}
        }

        private void buttonReset_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is null");

            FChartTimeSeries.AutoScaleAxes();
            FChartTimeSeries.ChartDisplay.Aspect.HorizOffset = 0;
            FChartTimeSeries.ChartDisplay.Aspect.VertOffset = 0;
            FChartTimeSeries.ChartDisplay.Aspect.Zoom = 100;
            FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent = 10;

            FChartTimeSeries.ChartDisplay.Aspect.Orthogonal = false;
            FChartTimeSeries.ChartDisplay.Aspect.Rotation = 0;
            FChartTimeSeries.ChartDisplay.Aspect.Elevation = 0;
            FChartTimeSeries.ChartDisplay.UpdateLayout();
        }

        private void buttonEdit_Click(object sender, RoutedEventArgs e)
        {
            FChartTimeSeries.ChartDisplay.ShowEditor();
            //EditChartPage(Self, FChartTimeSeries.ChartDisplay, teeEditPagingPage);  // display chart edit dialog
        }

        private void scrollBarTransparency_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            int i;

            if (FChartTimeSeries == null) return;

            FChartTimeSeries.EventData.InputSettings.ChartTransparency = Convert.ToInt32(100 * scrollBarTransparency.Value);

            for (i = 0; i < FChartTimeSeries.ChartVariableCount; i++)        // for each channel
            {
                switch (FChartTimeSeries.EventData.InputSettings.ChartStyle)
                {
                    case TTimeSeriesChartRenderType.sctLine:
                        break;

                    case TTimeSeriesChartRenderType.sctArea:
                        (ChartHMIDisplay.Series[i] as Steema.TeeChart.WPF.Styles.Area).Transparency = ChartTimeSeries.EventData.InputSettings.ChartTransparency;  // percent transparency
                        break;

                    default:
                        System.Diagnostics.Debug.Assert(false, "Invalid chart style");
                        break;
                }
            }
       }

        private void comboBoxChartType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

            if (scrollBarTransparency != null)
            {
                System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null, "EventData is null");
                FChartTimeSeries.EventData.InputSettings.ChartStyle = (TTimeSeriesChartRenderType)comboBoxChartType.SelectedIndex;
                if (comboBoxChartType.SelectedIndex == 0)  // Line
                {
                    labelTransparency.Visibility = Visibility.Collapsed;
                    scrollBarTransparency.Visibility = Visibility.Collapsed;
                    scrollBarTransparency.IsEnabled = false;
                }
                else                                    // area
                {
                    labelTransparency.Visibility = Visibility.Visible;
                    scrollBarTransparency.Visibility = Visibility.Visible;
                    scrollBarTransparency.IsEnabled = true;
                }
                FChartTimeSeries.SaveChartActiveChannels();
                Display(false);
                FChartTimeSeries.ChartDisplay.UpdateLayout();
            }
        }

/************************************************************************
* NAME:  ComboBoxDateFormatChange
* FUNCTIONAL DESCRIPTION: Change default import date format option
* INPUT PARAMETERS: not used
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        private void comboBoxDate_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            string myFormat;
            GlobalString gs = new GlobalString();
            string[] MDY_OPTIONS = new string[3] { "MDY", "YMD", "DMY" };

            
            SetDateFormat(MDY_OPTIONS[comboBoxDate.SelectedIndex]);
            myFormat = gs.GetDateControlFormat(FDateFormat);
            //datePickerBegin.Format = myFormat;
            //datePickerEnd.Format = myFormat;
            ResetDeviceInformation();
            if(FChartType == TTimeSeriesChartType.ctSSI) 
            {
                FillEventListViewSSI(false);
            }
            else if(FChartType == TTimeSeriesChartType.ctLDP) 
            {
                FillEventListViewLDP(false);
            }
            SetDateTimeLabels();
        }

/************************************************************************
* NAME:    ComboBoxScaleChange
* FUNCTIONAL DESCRIPTION:  change between primary and secondary units
* INPUT PARAMETERS: not used
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        private void comboBoxScale_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int channelCount;
            float[] channelScale ;
            int i,j;
            TTimeSeriesLDPChannelRecord channelRecord ;
            Steema.TeeChart.WPF.Styles.SeriesCollection chartSeries;
            Boolean doSecondary;


            if(ChartType != TTimeSeriesChartType.ctLDP) return;

            chartSeries = ChartHMIDisplay.Series;
            if(chartSeries.Count < 1) return;    // no data loaded into chart


            if(comboBoxScale.SelectedIndex == -1)   // invalid selection, reset to Secondary
            {
                comboBoxScale.SelectedIndex = 1;
                comboBoxScale.Text = "Secondary";
            }

            if(FChartTimeSeries.EventData.InputSettings.ChartSecondaryUnits =
                            (comboBoxScale.Text == "Secondary"))  return;  // no change, just exit

            FChartTimeSeries.EventData.InputSettings.ChartSecondaryUnits = (comboBoxScale.Text == "Secondary");

            channelCount = FChartTimeSeries.EventData.LDPChannelTable.Count;  // number of channels to display
            channelScale = new float[channelCount];    // scale factors for channels

            for (j = 0; j< FChartTimeSeries.EventData.LDPChannelTable.Count; j++)
            {
                channelRecord = ChartTimeSeries.EventData.LDPChannelTable[j];  // get event summary record
                channelScale[j] = channelRecord.ChannelScale;
            }

            doSecondary = ChartTimeSeries.EventData.InputSettings.ChartSecondaryUnits;

            if(doSecondary) 
            {
                labelHelp.Content = "   Units are Secondary";
            }
            else
            {
                labelHelp.Content = "   Units are Primary";
            }

            /*The chartSeries might contain one more series than the number of channels.
            It might have an event marker series at the end of the list.
            */
            System.Diagnostics.Debug.Assert((FChartTimeSeries.ChartVariableCount <= chartSeries.Count),
                            "Invalid chart series count.");

            for (j = 0; j< FChartTimeSeries.ChartVariableCount; j++)       // for each channel
            {
                for (i = 0; i<chartSeries[j].Count; i++)        // for point in each channel
                {
                    if(doSecondary) 
                    {
                        chartSeries[j].XValues[i] = chartSeries[j].YValues[i]/channelScale[j];
                    }
                    else
                    {
                        chartSeries[j].YValues[i] = chartSeries[j].YValues[i]*channelScale[j];
                    }
                }
            }
            FChartTimeSeries.UpdateEventScale();
            FChartTimeSeries.AutoScaleAxes();

        }

/******************************************************************************
* MODULE NAME ...  ButtonEventUpdateClick
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Update chart based on new event filter settings
*
* INPUT PARAMETERS:
*                    Sender : TObject
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        private void buttonUpdate_Click(object sender, RoutedEventArgs e)
        {
            Boolean status;

            status = false;
            switch (FChartType)
            {
                case TTimeSeriesChartType.ctSSI:
                    GetChartInputControlSettingsSSI();
                    status = FChartTimeSeries.FillEventIndexTable(true);   // generate list before sizing panels to force autosize
                break;

                case TTimeSeriesChartType.ctLDP:
                    GetChartInputControlSettingsLDP();
                    status = FChartTimeSeries.FillEventIndexTable(true);   // generate list before sizing panels to force autosize
                break;

                default:
                    System.Diagnostics.Debug.Assert(false,"Invalid chart type");
                break;
            }

            if(status)         // there is valid data to chart (may or may not be events)
            {
                switch( FChartType)
                {
                    case TTimeSeriesChartType.ctSSI:
                        FillEventListViewSSI(true);
                    break;

                    case TTimeSeriesChartType.ctLDP:
                        FillEventListViewLDP(true);
                    break;
                }
                UpdateChart(true);
            }
        }
/******************************************************************************
* MODULE NAME ...  ButtonEventClearClick
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Disable event display checkboxes
*
* INPUT PARAMETERS:
*                    Sender : TObject
*
* OUTPUT PARAMETERS:
*                    None:
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        private void buttonUncheckAllEvents_Click(object sender, RoutedEventArgs e)
        {
            int i;
            TTimeSeriesIndexRecord indexRecord;

            if (listViewChart.Items.Count == 0) return;    // nothing to do
            listViewChart.UnselectAll();

            for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
            {
                indexRecord = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                indexRecord.Checked = false;
            }
            UpdateChart(true);
        }
/************************************************************************
* NAME:  sbSelectAllClick
* FUNCTIONAL DESCRIPTION:  select all items in event summary
* INPUT PARAMETERS: Sender : TObject
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        private void buttonCheckAllEvents_Click(object sender, RoutedEventArgs e)
        {
            int i;
            TTimeSeriesIndexRecord indexRecord;

            if (listViewChart.Items.Count == 0) return;    // nothing to do

            listViewChart.SelectAll();

            for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
            {
                indexRecord = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                indexRecord.Checked = true;
            }
            UpdateChart(true);
        }

/******************************************************************************
* MODULE NAME ...  FormMouseWheel
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Event handler for mouse wheel events
*
* INPUT PARAMETERS:
*                    Sender : TObject
*                    Shift : TShiftState   set containing list of depressed control keys
*                    WheelDelta : Integer  change in mouse wheel position
*                    MousePos : TPoint     screen coord. of mouse
*
* OUTPUT PARAMETERS:
*                    Handled: Boolean   true/false indicating whether event was handled
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        private void ChartHMICBEMA_MouseWheel(object sender, MouseWheelEventArgs e)
        {
//procedure TfrmChartTimeSeries.FormMouseWheel(Sender: TObject; Shift: TShiftState;
//  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);

           ChartHMIDisplayMouseWheel(sender,e);

        }

/******************************************************************************
* MODULE NAME ...  ChartHMIDisplayMouseDown
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Event handler for mouse down events
*
* INPUT PARAMETERS:
*                    Sender : TObject
*                    Button : TMouseButton
*                    Shift : TShiftState   set containing list of depressed control keys
*                    X,Y : Integer     screen coord. of mouse
*
* OUTPUT PARAMETERS:
*                    Handled: Boolean   true/false indicating whether event was handled
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        private void ChartHMICBEMA_MouseDown(object sender, MouseButtonEventArgs e)
        {
            Boolean isShift, isCtrl, isAlt;
            Point myPoint;

            isShift = (Keyboard.IsKeyDown(Key.LeftShift) || Keyboard.IsKeyDown(Key.RightShift));
            isCtrl = (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl));
            isAlt = (Keyboard.IsKeyDown(Key.LeftAlt) || Keyboard.IsKeyDown(Key.RightAlt));
            myPoint = e.GetPosition(this);

            ////procedure TfrmChartTimeSeries.ChartHMIDisplayMouseDown(Sender: TObject;
            ////  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
            if ((e.RightButton == MouseButtonState.Pressed) && (isCtrl || isShift || isAlt))
            {
                FChartTimeSeries.ChartDisplay.Panning.Allow = Steema.TeeChart.WPF.ScrollModes.None;  // disable during rotation
                FChartTimeSeries.MouseX = Convert.ToInt32(myPoint.X);
                FChartTimeSeries.MouseY = Convert.ToInt32(myPoint.Y);
            }

            if (e.LeftButton == MouseButtonState.Pressed)
            {
                if (FChartTimeSeries.ChartRotate || FChartTimeSeries.ChartTranslate ||
                FChartTimeSeries.ChartZoom || FChartTimeSeries.ChartThickness)
                {
                    FChartTimeSeries.ChartActive = true;
                    FChartTimeSeries.MouseX = Convert.ToInt32(myPoint.X);
                    FChartTimeSeries.MouseY = Convert.ToInt32(myPoint.Y);
                }
            }
        }
/******************************************************************************
* MODULE NAME ...  ChartHMIDisplayMouseUp
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Event handler for mouse up events
*
* INPUT PARAMETERS:
*                    Sender : TObject
*                    Button : TMouseButton
*                    Shift : TShiftState   set containing list of depressed control keys
*                    X,Y : Integer     screen coord. of mouse
*
* OUTPUT PARAMETERS:
*                    Handled: Boolean   true/false indicating whether event was handled
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        private void ChartHMICBEMA_MouseUp(object sender, MouseButtonEventArgs e)
        {
            //procedure TfrmChartTimeSeries.ChartHMIDisplayMouseUp(Sender: TObject;
            //  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

            if (e.RightButton == MouseButtonState.Released)
            {
                FChartTimeSeries.ChartDisplay.Panning.Allow = Steema.TeeChart.WPF.ScrollModes.Both;  // re-enable after rotation
            }

            if (e.LeftButton == MouseButtonState.Released)
            {
                FChartTimeSeries.ChartActive = false;  // disable dynamic operations
            }
        }

        public void ChartCursor_Change(object sender, Steema.TeeChart.WPF.Tools.CursorChangeEventArgs e)
        {
            Point p = new Point();
            p.X = e.XValue;
            p.Y = e.YValue;
        }
/******************************************************************************
* MODULE NAME ...  ChartHMIDisplayMouseMove
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Event handler for mouse move events
*
* INPUT PARAMETERS:
*                    Sender : TObject
*                    Shift : TShiftState   set containing list of depressed control keys
*                    X,Y : Integer     screen coord. of mouse
*
* OUTPUT PARAMETERS:
*                    Handled: Boolean   true/false indicating whether event was handled
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
* 2008.12.18  Kent Myers        Add extra title for mouse cursor data feedback if necessary
*******************************************************************************/
        private void ChartHMICBEMA_MouseMove(object sender, MouseEventArgs e)
        {
            ////procedure TfrmChartTimeSeries.ChartHMIDisplayMouseMove(Sender: TObject;
            ////  Shift: TShiftState; X, Y: Integer);

            int deltaX, deltaY;
            double zoom2D;
            Boolean isShift, isCtrl, isAlt;
            Point myPoint;

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null, "EventData is null");
            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData.InputSettings != null, "InputSettings is null");

            isShift = (Keyboard.IsKeyDown(Key.LeftShift) || Keyboard.IsKeyDown(Key.RightShift));
            isCtrl = (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl));
            isAlt = (Keyboard.IsKeyDown(Key.LeftAlt) || Keyboard.IsKeyDown(Key.RightAlt));
            myPoint = e.GetPosition(this);

            deltaX = Convert.ToInt32(myPoint.X) - FChartTimeSeries.MouseX;
            deltaY = Convert.ToInt32(myPoint.Y) - FChartTimeSeries.MouseY;
            FChartTimeSeries.MouseX = Convert.ToInt32(myPoint.X);
            FChartTimeSeries.MouseY = Convert.ToInt32(myPoint.Y);
            if(((e.LeftButton == MouseButtonState.Pressed) && FChartTimeSeries.ChartActive &&
                    (FChartTimeSeries.ChartRotate || FChartTimeSeries.ChartTranslate || FChartTimeSeries.ChartZoom ||
                    FChartTimeSeries.ChartThickness))) 
            {
                if(FChartTimeSeries.EventData.InputSettings.Chart3D && FChartTimeSeries.ChartActive) 
                {
                    if(FChartTimeSeries.ChartRotate) 
                    {
                        FChartTimeSeries.ChartDisplay.Aspect.Orthogonal = false;
                        FChartTimeSeries.ChartDisplay.Aspect.Rotation = (FChartTimeSeries.ChartDisplay.Aspect.Rotation + deltaX) % 360;
                        FChartTimeSeries.ChartDisplay.Aspect.Elevation = ((FChartTimeSeries.ChartDisplay.Aspect.Elevation - deltaY) %90) + 270;
                    }
                    if(FChartTimeSeries.ChartTranslate) 
                    {
                        FChartTimeSeries.ChartDisplay.Aspect.HorizOffset = FChartTimeSeries.ChartDisplay.Aspect.HorizOffset + deltaX;
                        FChartTimeSeries.ChartDisplay.Aspect.VertOffset = FChartTimeSeries.ChartDisplay.Aspect.VertOffset + deltaY;
                    }
                    if(FChartTimeSeries.ChartZoom) 
                    {
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom = FChartTimeSeries.ChartDisplay.Aspect.Zoom + deltaX;
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom = FChartTimeSeries.ChartDisplay.Aspect.Zoom - deltaY;
                    }
                    if(FChartTimeSeries.ChartThickness) 
                    {
                        if(((FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent + deltaX) < 101) &&
                            ((FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent + deltaX) > 0)) 
                        {
                            FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent =
                            FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent + deltaX;
                        }
                        else
                        {
                            if((FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent + deltaX) >= 101) 
                            {
                                FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent = 100;
                            }
                            else
                            {
                                FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent = 1;
                            }
                        }
                    }
                }
            }
            else
            {
                if((e.RightButton == MouseButtonState.Pressed) && (isCtrl || isAlt || isShift)) 
                {
                    if(isShift)      // do 3D rotation
                    {
                        FChartTimeSeries.ChartDisplay.Aspect.Elevation = ((FChartTimeSeries.ChartDisplay.Aspect.Elevation - deltaY) %90) + 270;
                        FChartTimeSeries.ChartDisplay.Aspect.Rotation = (FChartTimeSeries.ChartDisplay.Aspect.Rotation + deltaX) % 360;
                    }
                    else if(isCtrl)       // do 2D zoom
                    {
                        zoom2D = 100.0 - deltaY;
                        FChartTimeSeries.ChartDisplay.Zoom.ZoomPercent(zoom2D);
                    }
                    else if(isAlt)        // do 3D zoom
                    {
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom =
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom - deltaY;
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom =
                        FChartTimeSeries.ChartDisplay.Aspect.Zoom + deltaX;
                    }
                }
                else
                {   // get X,Y values from curve
                    DisplayMouseQueryFeedback(e);
                }
            }
            FChartTimeSeries.EventData.InputSettings.Rotation = FChartTimeSeries.ChartDisplay.Aspect.Rotation;
            FChartTimeSeries.EventData.InputSettings.Zoom = FChartTimeSeries.ChartDisplay.Aspect.Zoom;
            FChartTimeSeries.EventData.InputSettings.HorizOffset = FChartTimeSeries.ChartDisplay.Aspect.HorizOffset;
            FChartTimeSeries.EventData.InputSettings.VertOffset = FChartTimeSeries.ChartDisplay.Aspect.VertOffset;
            FChartTimeSeries.EventData.InputSettings.Elevation = FChartTimeSeries.ChartDisplay.Aspect.Elevation;
            FChartTimeSeries.EventData.InputSettings.Percent3D = FChartTimeSeries.ChartDisplay.Aspect.Chart3DPercent;
        }
/************************************************************************
* NAME:   ChartHMIDisplayGetAxisLabel
* FUNCTIONAL DESCRIPTION:  This function is used to revise the x-axis label
*                          if the Date/Time option is active.
* INPUT PARAMETERS:
*                    Sender : TChartAxis
*                    Series : TChartSeries
*                    ValueIndex : Integer
*                    LabelText  : string
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        private void ChartHMIDisplay_GetAxisLabel(object sender, Steema.TeeChart.WPF.GetAxisLabelEventArgs e)
        {
            //procedure TfrmChartTimeSeries.ChartHMIDisplayGetAxisLabel(Sender: TChartAxis;
            //  Series: TChartSeries; ValueIndex: Integer; var LabelText: string);

            double value, delta;
            int ivalue,idelta,jDate,jDate1,lastRecord;
            TTimeSeriesRecord dataRecord;
            TTimeSeriesIndexRecord indexRecord;
            Boolean date_label, no_label, found, minFound, maxFound;
            int i,i1,i2,number,count,ecount,ecount2,Left,Right,iMin,iMax;
            TLimits[] eLimits;     // lower/upper limit of each event interval
            string myLabel,TimeString;
            Boolean allEvents;
            GlobalString gs = new GlobalString();
            TFormatSettings myFormat = gs.InitTFormatSettings();
            TTimeSeriesInputSettingsSSI mySSI;

            FChartTimeSeries.EventData.InitDateTimeFormat(TDateFormatType.ftInternational, ref myFormat);
            gs.DateFormat2DigitYear(ref myFormat);
            /*
            *      Records are stored && sorted by record number, so the
            *      record number must be used to look up each record,
            *      determine the time/date && produce a label for the
            *      x-axis.  Special logic is used to determine whether
            *      two consecutive labels (the current label && the label to the left)
            *      have the same date.  If so, the current label will not include the date.
            */

            System.Diagnostics.Debug.Assert(FChartTimeSeries.EventData != null,"EventData is nil");

            if(FChartTimeSeries.EventData.InputSettings.ChartAxisDateTime &&
                (sender == FChartTimeSeries.ChartDisplay.Axes.Bottom)) 
            {
                switch(FChartType)
                {
                    case TTimeSeriesChartType.ctUndefined:
                    case TTimeSeriesChartType.ctLDP:
                    break;

                    case TTimeSeriesChartType.ctSSI:
                        mySSI = (TTimeSeriesInputSettingsSSI) FChartTimeSeries.EventData.InputSettings; 
                        if(mySSI.ChartCBEMA) return;
                    break;

                    default:
                        System.Diagnostics.Debug.Assert(false,"Invalid chart type");
                    break;
                }

                myLabel = e.LabelText.Replace(",","");  // remove commas form present label (thousands)
                value = gs.NonLocaleStrToFloat(myLabel);
                ivalue = Convert.ToInt32(Math.Round(value));         // record number in range 0 to total # active records
                allEvents = true;   // assume all events are displayed
                iMin = Convert.ToInt32(Math.Truncate(FChartTimeSeries.ChartDisplay.Axes.Bottom.Minimum));  // minimum record number on X axis
                iMax = Convert.ToInt32(Math.Truncate(FChartTimeSeries.ChartDisplay.Axes.Bottom.Maximum));  // maximum record number on X axis

                // update present record limits to actual record limits based on active events

                number = 0;  // number of active event records
                ecount = 0;  // number of active events
                Left = 1;   // lower limit if no active events
                Right = FChartTimeSeries.EventData.RawDataTable.Count;  // upper limit if no active events
                if((FChartType == TTimeSeriesChartType. ctSSI) && (FChartTimeSeries.EventData.IndexTable.Count > 0))      // events are present
                {
                    Left = -1;
                    lastRecord = 0;
                    for(i=0; i< FChartTimeSeries.EventData.IndexTable.Count; i++)
                    {
                        indexRecord = FChartTimeSeries.EventData.IndexTable[i];  // get event summary record
                        if(!indexRecord.Checked)  continue; // event not active, skip it

                        ecount++;     // increment active event count
                        if(Left < 0)     // Left not found yet, this is start of first active record
                        {
                            Left = indexRecord.FirstRecord;
                        }
                        if(Right < indexRecord.LastRecord + 1)    // skip if sub-event has a smaller right bound
                        {
                            Right = indexRecord.LastRecord + 1;
                        }
                        if(indexRecord.LastRecord > lastRecord)   // test for sub-event
                        {
                            number = number + indexRecord.LastRecord - indexRecord.FirstRecord + 1;   // number of active records so far
                        }
                        lastRecord = indexRecord.LastRecord;
                    }
                    if(Left < 0)  Left = 0;  // no active events found

                    // Present label is outside valid range of record numbers, just return empty label
                    if((ivalue < 0) || ((number > 0) && (ivalue > number))) 
                    {
                        e.LabelText = " ";
                        return;     // record is invalid
                    }

                    // find record "ivalue" in list of active records

                    if(ecount > 0)     // subset of events are active
                    {
                        allEvents = false;
                        eLimits = new TLimits[ecount];
                    }
                    else               // all events are active by default
                    {
                        eLimits = new TLimits[FChartTimeSeries.EventData.IndexTable.Count];
                    }

                    number = 0;    // initialize number of active records
                    ecount2 = 0;   // initialize active event count
                    found = false;  // assume value is not in active interval
                    minFound = false;  // assume minimum rec # not found yet
                    maxFound = false;  // assume maximum rec # not found yet

                    // Convert present record number && min/max record numbers to actual record numbers
                    lastRecord = 0;  // test for sub-events
                    for(i=0; i< FChartTimeSeries.EventData.IndexTable.Count; i++)
                    {
                        indexRecord = ChartTimeSeries.EventData.IndexTable[i];  // get event summary record
                        if(!allEvents && !indexRecord.Checked)  continue;  // skip inactive record

                        if(indexRecord.LastRecord <= lastRecord)  continue;  // skip sub-events
                        lastRecord = indexRecord.LastRecord;

                        eLimits[ecount2].limit1 = indexRecord.FirstRecord;  // remember upper limit of each event interval
                        eLimits[ecount2].limit2 = indexRecord.LastRecord;  // remember upper limit of each event interval
                        ecount2++;     // increment active event count
                        count = indexRecord.LastRecord - indexRecord.FirstRecord + 1;  // number of records in this event
                        if(!found && (ivalue > number) && (ivalue <= (number + count)))   // record is in this interval
                        {
                            found = true;      // record is in this event interval
                            ivalue = indexRecord.FirstRecord + ivalue - number;
                        }
                        if(!minFound && (iMin > number) && (iMin <= (number + count))) 
                        {
                            minFound = true;     // minimum is in this event interval
                            iMin = indexRecord.FirstRecord + iMin - number;
                        }
                        if(!maxFound && (iMax > number) && (iMax <= (number + count))) 
                        {
                            maxFound = true;     // maximum is in this event interval
                            iMax = indexRecord.FirstRecord + iMax - number;
                        }
                        number = number + count;  // number of active records
                    }

                    if(!minFound)   // before {ning of the first event
                    {
                        iMin = Left;
                    }
                    if(!maxFound)   // after end of the last event
                    {
                        iMax = Right;
                    }
                }
                else
                {           // no events, all records in use
                    found = true;  // assume value is in active interval
                    eLimits = new TLimits[1];
                    eLimits[0].limit1 = 1;
                    eLimits[0].limit2 = FChartTimeSeries.EventData.RawDataTable.Count;
                }

                date_label = false;   // assume we are not displaying the date
                no_label = true;      // assume that no label will be displayed
                if(found && (ivalue >= Left) && (ivalue <= Right)) 
                {
                    no_label = false;  // some label will be displayed
                    delta = FChartTimeSeries.ChartXDelta;  // delta obtained in separate event handler to prevent recursion
                    idelta = Convert.ToInt32(Math.Round(delta))+1;  // record interval between labels
                    if(idelta < 1) 
                    {
                        idelta = 1;
                    }

                    // See if this is the leftmost || rightmost axis label
                    if(((ivalue - idelta) <= Left) || ((ivalue + idelta) >= Right) ||
                        ((ivalue - idelta) <= iMin) || ((ivalue + idelta) >= iMax)) 
                    {
                        date_label = true;     // display the date
                    }
                    else
                    {
                        i1 = FChartTimeSeries.WhichInterval(ivalue - idelta, eLimits);  // event interval containing previous label
                        i2 = FChartTimeSeries.WhichInterval(ivalue + idelta, eLimits);  // event interval containing next label

                        if(i1 != i2)  // neighboring labels are from a different event
                        {
                            date_label = true;   // display the date at interval boundaries
                        }
                        else
                        {
                            dataRecord = FChartTimeSeries.EventData.RawDataTable[ivalue-1];   // record for current label
                            jDate = gs.DateToJulian(dataRecord.DateTime);
                            dataRecord = FChartTimeSeries.EventData.RawDataTable[ivalue-idelta-1];  // record for previous label
                            jDate1 = gs.DateToJulian(dataRecord.DateTime);
                            if(jDate != jDate1)   // date is different for neighboring labels
                            {
                                date_label = true;     // display the date
                            }
                        }
                    }
                }

                if(no_label)     // no label should be displayed
                {
                    e.LabelText = " ";
                }
                else
                {                // some label should be displayed
                    dataRecord = FChartTimeSeries.EventData.RawDataTable[ivalue-1];  // record for previous label
                    if(FChartType == TTimeSeriesChartType.ctSSI) 
                    {
                        TimeString = " " + string.Format(TIME_WITH_MS,dataRecord.DateTime);
                    }
                    else
                    {
                        TimeString = " " + string.Format(TIME_WITH_SEC,dataRecord.DateTime);
                    }

                    if (date_label)    // date should be displayed
                    {
                        e.LabelText = gs.DateToStr(dataRecord.DateTime, myFormat) + TimeString;  // native language
                    }
                    else              // only time should be displayed
                    {
                        e.LabelText = TimeString;
                    }
                }
            }
        }
/******************************************************************************
* MODULE NAME ...  ChartHMIDisplayGetNextAxisLabel
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  called just before displaying axis labels
*
* INPUT PARAMETERS:
*                    Sender : TChartAxis
*                    LabelIndex : Integer  index of current label
*                    LabelValue : double   value of current label
*                    Stop  : boolean       true/false to stop callback
*
* OUTPUT PARAMETERS:
*                    None:
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*  NOTE: This callback is necessary in order to obtain the increment between
*  consecutive labels.  If CalcIncrement() is called in the
*  ChartHMIDisplayGetAxisLabel callback, a recursion occurs during a zoom.
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        private void ChartHMIDisplay_GetNextAxisLabel(object sender, Steema.TeeChart.WPF.GetNextAxisLabelEventArgs e)
        {
            //procedure TfrmChartTimeSeries.ChartHMIDisplayGetNextAxisLabel(Sender: TChartAxis;
            //  LabelIndex: Integer; var LabelValue: Double; var Stop: Boolean);
            if(sender == ChartHMIDisplay.Axes.Bottom) 
            {
                ChartTimeSeries.ChartXDelta = ChartTimeSeries.ChartDisplay.Axes.Bottom.CalcIncrement;
                //ChartTimeSeries.ChartXDelta = sender.CalcIncrement;
            }
        }

/************************************************************************
* NAME:   ChartHMIDisplayClickLegend
* FUNCTIONAL DESCRIPTION:  update yvalue of SER event markers
* INPUT PARAMETERS: not used
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        private void ChartHMIDisplay_ClickLegend(object sender, MouseEventArgs e)
        {
            FChartTimeSeries.UpdateEventScale();
        }

/******************************************************************************
* MODULE NAME ...  ButtonEventCaptureClick
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Capture event summary to clipboard
*
* INPUT PARAMETERS:
*                    Sender : TObject
*
* OUTPUT PARAMETERS:
*                    None:
*
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.09.24  Kent Myers        Original implementation
*******************************************************************************/
        private void buttonCopyEvent_Click(object sender, RoutedEventArgs e)
        {
            int i;
            string myText;
            TTimeSeriesIndexRecord indexRecord;
            GlobalString gs = new GlobalString();
            const string HEADING_VSSI1 = "\t\t\t\t\t\t\t\tEvent\tITIC\t";
            const string HEADING_VSSI2 = "\tEvent\tType\tDate\t Time\t\tDuration\tDepth\tRegion\t" +
                "VaMin\tVaMax\tVbMin\tVbMax\tVcMin\tVcMax";
            const string HEADING_LDP = "\tEvent\tDate\t Time\t\tDescription";

            if (FChartTimeSeries.EventData.IndexTable.Count == 0) return;    // nothing to do
            myText = "";

            switch(ChartType) 
            {
                case TTimeSeriesChartType.ctLDP: 
                    myText = HEADING_LDP + CRLF;
                    for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
                    {
                        indexRecord = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                        if(indexRecord.Checked)
                        {
                            myText = myText + "[X]" + TAB;
                        }
                        else 
                        {
                            myText = myText + "[ ]" + TAB;
                        }
                        myText = myText + indexRecord.EventName + TAB;
                        myText = myText + string.Format(DATETIME_WITH_SEC,indexRecord.DateTime) + TAB;
                        myText = myText + indexRecord.Description + CRLF;
                    }

                break;

                case TTimeSeriesChartType.ctSSI:
                    myText = HEADING_VSSI1 + CRLF + HEADING_VSSI2 + CRLF;
                    for (i = 0; i < FChartTimeSeries.EventData.IndexTable.Count; i++)
                    {
                        indexRecord = FChartTimeSeries.EventData.IndexTable[i]; // get the event index record
                        if(indexRecord.Checked)
                        {
                            myText = myText + "[X]" + TAB;
                        }
                        else 
                        {
                            myText = myText + "[ ]" + TAB;
                        }
                        myText = myText + indexRecord.EventName + TAB;
                        myText = myText + indexRecord.Description + TAB;
                        myText = myText + string.Format(DATETIME_WITH_MS,indexRecord.DateTime) + TAB;
                        myText = myText + gs.DurationToTimeString(indexRecord.Duration) + TAB;
                        myText = myText + string.Format("{0,5:0.0}",indexRecord.Magnitude) + TAB;
                        myText = myText + FChartTimeSeries.CBEMAViolationString(indexRecord.Violation) + TAB;
                        myText = myText + string.Format("{0,5:0.0}",indexRecord.VAMin) + TAB;
                        myText = myText + string.Format("{0,5:0.0}",indexRecord.VAMax) + TAB;
                        myText = myText + string.Format("{0,5:0.0}",indexRecord.VBMin) + TAB;
                        myText = myText + string.Format("{0,5:0.0}",indexRecord.VBMax) + TAB;
                        myText = myText + string.Format("{0,5:0.0}",indexRecord.VCMin) + TAB;
                        myText = myText + string.Format("{0,5:0.0}",indexRecord.VCMax) + CRLF;
                    }
                break;

                default:
                    System.Diagnostics.Debug.Assert(false,"Invalid chart type");
                break;
            }

            Clipboard.SetText(myText);

        }


/******************************************************************************
* MODULE NAME ...  SpeedButtonCopyClick
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Kent Myers
*
* FUNCTIONAL DESCRIPTION:  Event handler for copy button in chart toolbar
*
* INPUT PARAMETERS:
*                    Sender : TObject
*
* OUTPUT PARAMETERS:
*                    None:
* RETURN VALUE:
*                    None:
*
* ERRORS:
*
* APPLICATION NOTES & EXAMPLE:
*
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2008.11.12  Kent Myers        Original implementation
*******************************************************************************/
        private void buttonCopy_Click(object sender, RoutedEventArgs e)
        {
            //FChartTimeSeries.ChartDisplay.BufferedDisplay = False;
            //FChartTimeSeries.ChartDisplay.CopyToClipboardBitmap;
            //FChartTimeSeries.ChartDisplay.BufferedDisplay = True;
            //try
            //{
                FChartTimeSeries.ChartDisplay.Export.Image.JPEG.CopyToClipboard();
            //}
            //catch
            //{
            //    throw new Exception();
            //}
        }

        private void buttonNotepadChart_Click(object sender, RoutedEventArgs e)
        {
            UpdateChartHandler();      // send ASCII data to chart
        }

        private void buttonNotepadReport_Click(object sender, RoutedEventArgs e)
        {
            UpdateReportHandler();   // retrieve report form chart

        }

        private void buttonNotepadSave_Click(object sender, RoutedEventArgs e)
        {
            string fileName,myText;

            SaveFileDialog saveDialog = new SaveFileDialog();
            try
            {
                fileName = string.Format("Chart Export {0}", string.Format("{0:yyyy-MMM-dd}", DateTime.Now));
                saveDialog.FileName = fileName;

                saveDialog.DefaultExt = "txt";
                saveDialog.Filter = "Chart file (*.txt)|*.txt";
                saveDialog.FilterIndex = 1;
                saveDialog.Title = "Export Chart Data";

                if (saveDialog.ShowDialog() == true)
                {
                    myText = richTextNotepad.ToString();
                    if(myText.Contains("TextBox:"))
                    {
                        myText = myText.Substring(myText.IndexOf(":") + 2);  // Strip off control prefix
                    }
                    SaveStringToFile(saveDialog.FileName, myText);
                }
            }
            catch
            {
            }
        }

        private void buttonNotepadClear_Click(object sender, RoutedEventArgs e)
        {
            richTextNotepad.Clear();
        }

        private void buttonNotepadHelp_Click(object sender, RoutedEventArgs e)
        {
            const string 
                HELP_MSG = "Chart - Paste LDP or VSSI data (text or CSV) into the text area and click Chart." + CRLF +
            "Get Report - Click Get Report to display a report of charted data in the text area." + CRLF +
            "Save... - Click Save to export the content of the text area to a file.";

            MessageBox.Show(HELP_MSG);   

        }
    }
}
