﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using System.Globalization;

namespace SELGlobalStringMethods
{
    public enum TDateFormatType { ftEnglish_US = 0, ftInternational };

    public struct  TFormatSettings
    {
        public TDateFormatType FormatType;
        public char ThousandSeparator;
        public char DecimalSeparator;
        public char DateSeparator;
        public char TimeSeparator;
        public string ShortDateFormat;
        public string LongDateFormat;
        public string ShortTimeFormat;
        public string LongTimeFormat;
    }

    public class GlobalString
    {
        public GlobalString()
        {
        }
        /************************************************************************
        * NAME:       LoadFromFile
        * FUNCTIONAL DESCRIPTION:    Read file into string list
        * INPUT PARAMETERS: filePath : string  file name
        * OUTPUT PARAMETERS: records : string list of records
        * RETURN: result : NA
        ************************************************************************/

        public void LoadFromFile(string filePath, List<string> records)
        {
            using (StreamReader r = new StreamReader(filePath))
            {
                string line;
                records.Clear();  // erase previous content
                while ((line = r.ReadLine()) != null)
                {
                    records.Add(line);
                }
            }
        }
        /************************************************************************
        * NAME:       ParseString
        * FUNCTIONAL DESCRIPTION:    Parse record into tokens
        * INPUT PARAMETERS: delim : string delim 
        *                   record: string  record
        * OUTPUT PARAMETERS: NA
        * RETURN: result : tokens : string array of tokens
        ************************************************************************/

        public void ParseString(char delim, string record, List<string> tokens)
        {
            string[] mytokens;
            string sdelim = delim.ToString();
            record = record.Trim();
            if (record.StartsWith(sdelim))
                record = record.Remove(0, 1);

            if (delim == ' ')
            {
                mytokens = record.Split(null);  // using white space as a delimiter
            }
            else
            {
                mytokens = record.Split(delim);
            }

            tokens.Clear();  // remove previous content
            for (int i = 0; i < mytokens.Length; i++)
            {
                if (mytokens[i].Length == 0) continue;  // skip null tokens
                mytokens[i] = mytokens[i].Replace("\"", "");  // remove embedded double-quotes
                tokens.Add(mytokens[i].Trim());
            }
        }
        /************************************************************************
        * NAME:       ContainsText
        * FUNCTIONAL DESCRIPTION:    Test string for substring case insensitive
        * INPUT PARAMETERS: haystack : string  
        *                   needle : string
        * OUTPUT PARAMETERS: none
        * RETURN: result : boolean true if needle found in haystack
        ************************************************************************/
        public Boolean ContainsText(string haystack, string needle)
        {
            string myNeedle = needle;  // just in case needle was a constant
            if (myNeedle.Length > haystack.Length) return false;

            try
            {
                //if (myNeedle.Length == 1)
                //{
                    return (haystack.ToUpper().IndexOf(myNeedle.ToUpper()) >= 0);
                //}
                //else
                //{
                //    Regex searcher = new Regex(myNeedle, RegexOptions.Singleline);
                //    Match match = searcher.Match(haystack);
                //    return match.Success;
                //}
            }
            catch
            {
                return false;
            }
        }
        /************************************************************************
        * NAME:       ContainsText
        * FUNCTIONAL DESCRIPTION:    Test string for substring case insensitive
        * INPUT PARAMETERS: haystack : string  
        *                   needle : char
        * OUTPUT PARAMETERS: none
        * RETURN: result : boolean true if needle found in haystack
        ************************************************************************/
        public Boolean ContainsText(string haystack, char needle)
        {
            return ContainsText(haystack, needle.ToString());
        }
        /************************************************************************
        * NAME:       CompareText
        * FUNCTIONAL DESCRIPTION:    Test string for string equality case insensitive
        * INPUT PARAMETERS: source : string  
        *                   target : string
        * OUTPUT PARAMETERS: none
        * RETURN: result : boolean true if needle found in haystack
        ************************************************************************/
        public Boolean CompareText(string source, string target)
        {
            return string.Equals(source, target, StringComparison.CurrentCultureIgnoreCase);
        }
        /************************************************************************
        * NAME:       TryStrToInt
        * FUNCTIONAL DESCRIPTION:    Test conversion of string to integer
        * INPUT PARAMETERS: svalue : string  
        * OUTPUT PARAMETERS: ivalue : integer
        * RETURN: result : boolean true conversion succeeds
        ************************************************************************/
        public Boolean TryStrToInt(string svalue, ref int ivalue)
        {
            try
            {
                ivalue = Convert.ToInt32(svalue);
                return true;
            }
            catch
            {
                return false;
            }
        }
        /************************************************************************
        * NAME:       TryNonLocaleStrToFloat
        * FUNCTIONAL DESCRIPTION:    Test conversion of string to float
        * INPUT PARAMETERS: svalue : string  
        * OUTPUT PARAMETERS: fvalue : float
        * RETURN: result : boolean true conversion succeeds
        ************************************************************************/
        public Boolean TryNonLocaleStrToFloat(string svalue, ref float fvalue)
        {
            try
            {
                fvalue = Convert.ToSingle(svalue);
                return true;
            }
            catch
            {
                return false;
            }
        }
        /************************************************************************
        * NAME:       TryNonLocaleStrToFloat
        * FUNCTIONAL DESCRIPTION:    Test conversion of string to float
        * INPUT PARAMETERS: svalue : string  
        * OUTPUT PARAMETERS: dvalue : double
        * RETURN: result : boolean true conversion succeeds
        ************************************************************************/
        public Boolean TryNonLocaleStrToFloat(string svalue, ref double dvalue)
        {
            try
            {
                dvalue = Convert.ToDouble(svalue);
                return true;
            }
            catch
            {
                return false;
            }
        }
        /************************************************************************
        * NAME:       TryNonLocaleStrToFloat
        * FUNCTIONAL DESCRIPTION:    Test conversion of string to float
        * INPUT PARAMETERS: svalue : string  
        * OUTPUT PARAMETERS: dvalue : double
        * RETURN: result : boolean true conversion succeeds
        ************************************************************************/
        public float NonLocaleStrToFloat(string svalue)
        {
            return Convert.ToSingle(svalue);
        }
        /************************************************************************
       * NAME:       NonLocaleFloatToStr
       * FUNCTIONAL DESCRIPTION:    convert float to non-locale string
       * INPUT PARAMETERS: fvalue : float  
       * OUTPUT PARAMETERS: 
       * RETURN: result : string representation of floating number in non-local format
       ************************************************************************/
        public string NonLocaleFloatToStr(float fvalue)
        {
            string svalue;
            svalue = String.Format("{0:f}", fvalue);
            return svalue.Replace(GetDecimalSeparator(), ".");
        }
        /************************************************************************
       * NAME:       NonLocaleFloatToStr
       * FUNCTIONAL DESCRIPTION:    convert double to non-locale string
       * INPUT PARAMETERS: fvalue : double  
       * OUTPUT PARAMETERS: 
       * RETURN: result : string representation of floating number in non-local format
       ************************************************************************/
        public string NonLocaleFloatToStr(double dvalue)
        {
            string svalue;
            svalue = String.Format("{0:f}", dvalue);
            return svalue.Replace(GetDecimalSeparator(), ".");
        }
        /************************************************************************
        * NAME:       InternalizeData
        * FUNCTIONAL DESCRIPTION:    Convert users decimal separator to a '.'
        * INPUT PARAMETERS: svalue : string  
        * OUTPUT PARAMETERS: na
        * RETURN: result : string converted string
        ************************************************************************/
        public string InternalizeData(string svalue)
        {
            string decimalSeparator;
            string tempString;

            decimalSeparator = GetDecimalSeparator().ToUpper();
            tempString = svalue.Replace(decimalSeparator[0],'.');

            return tempString;
        }
        /************************************************************************
        * NAME:       GetDecimalSeparator
        * FUNCTIONAL DESCRIPTION:    Get native decimal separator
        * INPUT PARAMETERS: na 
        * OUTPUT PARAMETERS: na
        * RETURN: result : string decimal separator
        ************************************************************************/
        public string GetDecimalSeparator()
        {
            string decimalSeparator;

            decimalSeparator = String.Format("{0:f1}", 1.1F);
            return decimalSeparator[1].ToString();
        }

/************************************************************************
* NAME:       InitTFormatSettings
* FUNCTIONAL DESCRIPTION:    Initialize a TFormatSettings record
* INPUT PARAMETERS: formatType : TDateFormatType  (ftEnglish_US, ftInternational)
*                   dateFormat : string  short date format
*                  myFormat : TFormatSettings
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void InitTFormatSettings(TDateFormatType formatType, string dateFormat, ref TFormatSettings myFormat )
        {
            string myDateTime;

            myFormat.ShortDateFormat = dateFormat;
            myFormat.FormatType = formatType;

            if(formatType == TDateFormatType.ftEnglish_US)    // force US English date format
            {
                myFormat.DateSeparator = '/';
            }
            else
            {
                myDateTime = String.Format("{0:MM/dd/yyyy}", DateTime.Now);
                //myDateTime = FormatDateTime("MM/DD/YYYY", DateTime.Now);
                if(ContainsText(myDateTime,"-")) 
                {
                    myFormat.DateSeparator = '-';
                }
                else
                {
                    myFormat.DateSeparator = '/';
                }
            }
            myFormat.TimeSeparator = ':';
            myFormat.DecimalSeparator = '.';
            myFormat.ThousandSeparator = '.';
        }
/************************************************************************
* NAME:       InitTFormatSettings
* FUNCTIONAL DESCRIPTION:    Initialize a TFormatSettings record
* INPUT PARAMETERS: formatType : TDateFormatType  (ftEnglish_US, ftInternational)
*                   dateFormat : string  short date format
*                  myFormat : TFormatSettings
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public TFormatSettings InitTFormatSettings()
        {
            TFormatSettings myFormat;
            myFormat.FormatType = TDateFormatType.ftEnglish_US;
            myFormat.ThousandSeparator = ',';
            myFormat.DecimalSeparator = '.';
            myFormat.DateSeparator = '/';
            myFormat.TimeSeparator = ':';
            myFormat.ShortDateFormat = "MM/dd/yy";
            myFormat.LongDateFormat = "MM/dd/yyyy";
            myFormat.ShortTimeFormat = "hh:mm:ss";
            myFormat.LongTimeFormat = "hh:mm:ss.fff";

            return myFormat;
        }
        /************************************************************************
        * NAME:       StrToDateTime
        * FUNCTIONAL DESCRIPTION:    parse date/time string to DateTime object
        * INPUT PARAMETERS: myDateTime : string  contains date time string
        * OUTPUT PARAMETERS: none
        * RETURN:      myFormat : TFormatSettings
        ************************************************************************/
        public DateTime StrToDateTime(string myDateTime)
        {
            return DateTime.Parse(myDateTime);
        }
        /************************************************************************
        * NAME:       StrToDateTime
        * FUNCTIONAL DESCRIPTION:    parse date/time string to DateTime object
        * INPUT PARAMETERS: myDateTime : string  contains date time string
        * OUTPUT PARAMETERS: none
        * RETURN:      myFormat : TFormatSettings
        ************************************************************************/
        public DateTime StrToDateTime(string myDateTime, TFormatSettings myFormat)
        {
            //IFormatProvider aFormat;
            //return Convert.ToDateTime(myString, aFormat);
            if (myFormat.FormatType == TDateFormatType.ftEnglish_US)
            {
                return DateTime.Parse(myDateTime);
                //IFormatProvider culture = new CultureInfo("en-US", true);
                //return DateTime.ParseExact(myDateTime, myFormat.LongDateFormat + " " + myFormat.ShortTimeFormat, culture);
            }
            else
            {
                return DateTime.Parse(myDateTime);
            }
        }
        /************************************************************************
        * NAME:       DateToStr
        * FUNCTIONAL DESCRIPTION:    format DateTime as string
        * INPUT PARAMETERS: myDateTime : string  contains date time string
        * OUTPUT PARAMETERS: none
        * RETURN:      myFormat : TFormatSettings
        ************************************************************************/
        public string DateToStr(DateTime myDateTime, TFormatSettings myFormat)
        {
            string dateFormat;
            dateFormat = "{0:" + myFormat.ShortDateFormat + "}";
            return String.Format(dateFormat, myDateTime);
        }
/******************************************************************************
* MODULE NAME ...  DateToJulian
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Bruce Scallorn
* 
* FUNCTIONAL DESCRIPTION:
* Converts the Date at the TDateTime to the Julian day for that date.
* 
* INPUT PARAMETERS:
*               JulianDay: TDateTime              
* 
* OUTPUT PARAMETERS:
*                    None:                        
* 
* RETURN VALUE:
*                  result: integer                
* 
* ERRORS:
* 
* APPLICATION NOTES & EXAMPLE:
* Unit Test Notes:                                Unit Tested:
* 
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2002.11.20  Bruce Scallorn    Original code
* 2006.01.31  Bruce Scallorn    modified calculation method
*******************************************************************************/
        public int DateToJulian(DateTime JulianDay)
        {
            int Year;
            int Month;
            int Day;
            DateTime DayOne;

            Year = JulianDay.Year;
            Month = JulianDay.Month;
            Day = JulianDay.Day;

            DayOne  = new DateTime(Year, 1, 1);
            return (Convert.ToInt32(JulianDay.Subtract(DayOne).TotalDays) + 1);
        }

/************************************************************************
* NAME:       GetDateControlFormat
* FUNCTIONAL DESCRIPTION:  convert English format string to international format
* INPUT PARAMETERS:      dateFormat : string  English format string
* OUTPUT PARAMETERS: none
* RETURN: result : string  updated format string for date time picker control
************************************************************************/
        public string GetDateControlFormat(string dateFormat)
        {
            string myFormat;
            string myDate;

            // First test for abreviated date format and convert to standard format
            dateFormat = GetDateFormat(dateFormat);
            //if(dateFormat == "MDY") 
            //{
            //    dateFormat = "MM/dd/yyyy";
            //}
            //else
            //if(dateFormat == "DMY") 
            //{
            //    dateFormat = "dd/MM/yyyy";
            //}
            //else
            //if(dateFormat == "YMD") 
            //{
            //    dateFormat = "yyyy/MM/dd";
            //}

            // change month to upper case for date time picker convention
            myFormat = dateFormat.Replace('m','M');

            // test default date to determine standard date separator
            //myDate = FormatDateTime("MM/DD/YYYY",DateTime.Now);
            myDate = String.Format("{0:MM/dd/yyyy}", DateTime.Now);
            if(ContainsText(myDate,"-"))     // international date
            {
                myFormat = myFormat.Replace('/','-');
            }
            return myFormat;
        }
/************************************************************************
* NAME:     GetDateFormat
* FUNCTIONAL DESCRIPTION:  Get the date format used for parsing dates
* INPUT PARAMETERS: aDateFormat : string   MDY, YMD, DMY
* OUTPUT PARAMETERS: none
* RETURN:           none
************************************************************************/
        public string GetDateFormat(string aDateFormat)
        {
            string myDateFormat;

            if((aDateFormat == "") || (aDateFormat == "MDY")) 
            {
                myDateFormat = "MM/dd/yyyy";
            }
            else if(aDateFormat == "DMY") 
            {
                myDateFormat = "dd/MM/yyyy";
            }
            else if (aDateFormat == "YMD")
            {
                myDateFormat = "yyyy/MM/dd";
            }
            else
            {
                myDateFormat = aDateFormat;
            }
            //else
            //{
            //    myDateFormat = " ";
            //    System.Diagnostics.Debug.Assert(false,"Invalid date format");
            //}
            return myDateFormat;
        }
/************************************************************************
* NAME:       DateFormat2DigitYear
* FUNCTIONAL DESCRIPTION:    Convert date format to 2-digit year
* INPUT PARAMETERS:      myFormat : TFormatSettings
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void DateFormat2DigitYear(ref TFormatSettings myFormat)
        {
            if(myFormat.ShortDateFormat[1] == 'M') 
            {
                myFormat.ShortDateFormat = "MM/dd/yy";
            }
            else if(myFormat.ShortDateFormat[1] == 'd') 
            {
                myFormat.ShortDateFormat = "dd/MM/yy";
            }
            else
            {
                myFormat.ShortDateFormat = "yy/MM/dd";
            }
        }
        /************************************************************************
        * NAME:       CompareDateTime
        * FUNCTIONAL DESCRIPTION:    Compare two DateTime objects
        * INPUT PARAMETERS:      a,b : DateTime  
        * OUTPUT PARAMETERS: none
        * RETURN: result : int (0 if equal ,-1 if a<b ,1 if a>b)
        ************************************************************************/
        public int CompareDateTime(DateTime a, DateTime b)
        {
            return DateTime.Compare(a, b);
        }
        /************************************************************************
        * NAME:       InterpolateDateTime
        * FUNCTIONAL DESCRIPTION:    Compare two DateTime objects
        * INPUT PARAMETERS:      a,b : DateTime  
        * OUTPUT PARAMETERS: none
        * RETURN: result : DateTime 
        ************************************************************************/
        public DateTime InterpolateDateTime(DateTime a, DateTime b, double myX)
        {
            double fraction;
            long ticks;

            fraction = myX - System.Math.Truncate(myX);   // interpolation between limits
            ticks = Convert.ToInt64(Convert.ToDouble(b.Subtract(a).Ticks) * Convert.ToDouble(fraction));
            return a.AddTicks(ticks);
        }
/************************************************************************
* NAME:       DurationToTimeString
* FUNCTIONAL DESCRIPTION:    Convert date/time duration to time string (hhh:mm:ss.fff)
* INPUT PARAMETERS:      aTimeString : string   time string in hhh:mm:ss.fff
*                        duration : TDateTime  duration
* OUTPUT PARAMETERS: none
* RETURN: none
************************************************************************/
        public void DurationToTimeString(out string aTimeString, TimeSpan duration)
        {
            aTimeString = String.Format("{0:d3}:{1:d2}:{2:d2}.{3:d3}",
                duration.Hours, duration.Minutes, duration.Seconds, duration.Milliseconds);
        }
        public string DurationToTimeString(TimeSpan duration)
        {
            return String.Format("{0:d3}:{1:d2}:{2:d2}.{3:d3}",
                duration.Hours, duration.Minutes, duration.Seconds, duration.Milliseconds);
        }
/******************************************************************************
* MODULE NAME ...  DateToJulian
* SYSTEM NAME ...  AcSELerator
* AUTHOR ........  Bruce Scallorn
* 
* FUNCTIONAL DESCRIPTION:
* Given the Year, Month and Day the Julian Day is returned.
* 
* INPUT PARAMETERS:
*                     Day: integer                
*                   Month: integer                
*                    Year: integer                
* 
* OUTPUT PARAMETERS:
*                    None:                        
* 
* RETURN VALUE:
*                  result: integer                
* 
* ERRORS:
* 
* APPLICATION NOTES & EXAMPLE:
* Unit Test Notes:                                Unit Tested:
* 
* REVISION HISTORY:
* REVISION    ORIGINATOR/       REVISION
* --DATE--    IMPLEMENTER       DESCRIPTION
* 2002.11.4  Bruce Scallorn    Original code
*******************************************************************************/
        public int DateToJulian(int Year, int Month, int Day)
        {
            DateTime DayOne = new DateTime(Year,1,1);
            DateTime JulianDay = new DateTime(Year,Month,Day+1);

            return JulianDay.Subtract(DayOne).Days;
        }
        /************************************************************************
        * NAME:       DecodeTime
        * FUNCTIONAL DESCRIPTION:    Convert date/time duration to time string (hhh:mm:ss.fff)
        * INPUT PARAMETERS:      aTimeString : string   time string in hhh:mm:ss.fff
        *                        duration : TDateTime  duration
        * OUTPUT PARAMETERS: none
        * RETURN: none
        ************************************************************************/
        public void DecodeTime(TimeSpan duration, out int hour, out int minute, out int sec, out int msec)
        {
            hour = duration.Days * 24 + duration.Hours;
            minute = duration.Minutes;
            sec = duration.Seconds;
            msec = duration.Milliseconds;

        }
        /************************************************************************
        * NAME:       GetSpecialFolder
        * FUNCTIONAL DESCRIPTION:    get special folder path name
        * INPUT PARAMETERS:      typeSpecialFolder : System.Environment.SpecialFolder
        * OUTPUT PARAMETERS: none
        * RETURN: result : string   folder path name
        ************************************************************************/
        public string GetSpecialFolder(System.Environment.SpecialFolder typeSpecialFolder)
        {
            DirectoryInfo di = new DirectoryInfo(System.Environment.GetFolderPath(typeSpecialFolder));
            return di.FullName;
        }
    }
}
