﻿using System;
using System.ComponentModel;

namespace WpfChartViewer
{
    public class VSSIEvent : INotifyPropertyChanged
    {
        #region [ Constructor ]

        /// <summary>
        /// Initializes a new instance of the <see cref="VSSIEvent"/> class.
        /// </summary>
        /// <param name="eventChecked">The event state.</param>
        /// <param name="eventName">The event name.</param>
        /// <param name="eventType">The event type.</param>
        /// <param name="date">The date.</param>
        public VSSIEvent(Boolean eventChecked, String eventName, String eventType, String date, String time,
            string duration, string magnitude, string violation, string vaMin, string vaMax, string vbMin, string vbMax, string vcMin, string vcMax)
        {
            _eventChecked = eventChecked;
            _eventName = eventName;
            _eventType = eventType;
            _date = date;
            _time = time;
            _duration = duration;
            _magnitude = magnitude;
            _violation = violation;
            _vaMin = vaMin;
            _vaMax = vaMax;
            _vbMin = vbMin;
            _vbMax = vbMax;
            _vcMin = vcMin;
            _vcMax = vcMax;
        }

        #endregion

        #region [ Fields ]

        private Boolean _eventChecked;
        private String _eventName;
        private String _eventType;
        private String _date;
        private String _time;
        private String _duration;
        private String _magnitude;
        private String _violation;
        private String _vaMin;
        private String _vaMax;
        private String _vbMin;
        private String _vbMax;
        private String _vcMin;
        private String _vcMax;


        #endregion

        #region [ Properties ]

        /// <summary>
        /// Gets or sets the event checked state.
        /// </summary>
        /// <value>The check state.</value>
        public Boolean Checked
        {
            get { return _eventChecked; }
            set
            {
                if (_eventChecked == value) return;
                _eventChecked = value;
                OnPropertyChanged("Checked");
            }
        }
        
        /// <summary>
        /// Gets or sets the first name.
        /// </summary>
        /// <value>The first name.</value>
        public String EventName
        {
            get { return _eventName; }
            set
            {
                if (_eventName == value) return;
                _eventName = value;
                OnPropertyChanged("EventName");
            }

        }

        /// <summary>
        /// Gets or sets the last name.
        /// </summary>
        /// <value>The event type.</value>
        public String EventType
        {
            get { return _eventType; }
            set
            {
                if (_eventType == value) return;
                _eventType = value;
                OnPropertyChanged("EventType");
            }
        }

        /// <summary>
        /// Gets or sets the date.
        /// </summary>
        /// <value>The date.</value>
        public String Date
        {
            get { return _date; }
            set
            {
                if (_date == value) return;
                _date = value;
                OnPropertyChanged("Date");
            }
        }

        /// <summary>
        /// Gets or sets the time.
        /// </summary>
        /// <value>The time.</value>
        public String Time
        {
            get { return _time; }
            set
            {
                if (_time == value) return;
                _time = value;
                OnPropertyChanged("Time");
            }
        }

        /// <summary>
        /// Gets or sets the duration.
        /// </summary>
        /// <value>The duration.</value>
        public String Duration
        {
            get { return _duration; }
            set
            {
                if (_duration == value) return;
                _duration = value;
                OnPropertyChanged("Duration");
            }
        }

        /// <summary>
        /// Gets or sets the magnitude.
        /// </summary>
        /// <value>The magnitude.</value>
        public String Magnitude
        {
            get { return _magnitude; }
            set
            {
                if (_magnitude == value) return;
                _magnitude = value;
                OnPropertyChanged("Magnitude");
            }
        }

        /// <summary>
        /// Gets or sets the violation.
        /// </summary>
        /// <value>The violation.</value>
        public String Violation
        {
            get { return _violation; }
            set
            {
                if (_violation == value) return;
                _violation = value;
                OnPropertyChanged("Violation");
            }
        }

        /// <summary>
        /// Gets or sets the VaMin.
        /// </summary>
        /// <value>The VaMin.</value>
        public String VaMin
        {
            get { return _vaMin; }
            set
            {
                if (_vaMin == value) return;
                _vaMin = value;
                OnPropertyChanged("VaMin");
            }
        }

        /// <summary>
        /// Gets or sets the VaMax.
        /// </summary>
        /// <value>The VaMax.</value>
        public String VaMax
        {
            get { return _vaMax; }
            set
            {
                if (_vaMax == value) return;
                _vaMax = value;
                OnPropertyChanged("VaMax");
            }
        }

        /// <summary>
        /// Gets or sets the VbMin.
        /// </summary>
        /// <value>The VbMin.</value>
        public String VbMin
        {
            get { return _vbMin; }
            set
            {
                if (_vbMin == value) return;
                _vbMin = value;
                OnPropertyChanged("VbMin");
            }
        }

        /// <summary>
        /// Gets or sets the VbMax.
        /// </summary>
        /// <value>The VbMax.</value>
        public String VbMax
        {
            get { return _vbMax; }
            set
            {
                if (_vbMax == value) return;
                _vbMax = value;
                OnPropertyChanged("VbMax");
            }
        }

        /// <summary>
        /// Gets or sets the VcMin.
        /// </summary>
        /// <value>The VcMin.</value>
        public String VcMin
        {
            get { return _vcMin; }
            set
            {
                if (_vcMin == value) return;
                _vcMin = value;
                OnPropertyChanged("VcMin");
            }
        }

        /// <summary>
        /// Gets or sets the VcMax.
        /// </summary>
        /// <value>The VcMax.</value>
        public String VcMax
        {
            get { return _vcMax; }
            set
            {
                if (_vcMax == value) return;
                _vcMax = value;
                OnPropertyChanged("VcMax");
            }
        }



        #endregion

        #region [ Events ]

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region [ Methods ]

        /// <summary>
        /// Called when [property changed].
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        protected void OnPropertyChanged(String propertyName)
        {
            if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}

